# ------------------------------------------------------------------------------
#  Methods:
#     - ldelete
#
#     - Graph:InitCanvasSetting
#     - Graph:InitImages
#     - Graph:CreateButtons
#     - Graph:GetNodeId            = function to get enumerated name for a type
#
#  Variables:
#     - slideGeometryTypes       = names of all geometry primitives
#     - slideTypes               = all slide primitives for buttons
#     - slideName($type)         = array for enumerating names for each type
#     
#     - paletteSelection         = None, or name of paletteButtong selected
#     - instanceWidth            = width between adjacent instances
#     - instanceRadius           = radius of instance circle
#
#     - imageWidth, imageHeight
#     - textHeight
#     - instanceTailHeight
#     - instanceHeadHeight
#     - canvasHeight, canvasWidth
#     - canvasTotalHeight, canvasTotalWidth
#
#     - instancableTypes
#
# ------------------------------------------------------------------------------

proc Graph:InitCanvasSetting {} {
    global slideGeometryTypes slideTypes instancableTypes slideName

    ####################################################
    # slide types and names
    set slideGeometryTypes { \
	    beziercurve bezierpatch bsplinecurve bsplinepatch \
	    cone cylinder cyclide face object point polyline \
	    sphere sweep torus }

    set slideTypes { \
	    arealight camera fog group instance light render \
	    surface \
	    beziercurve bezierpatch bsplinecurve bsplinepatch \
	    cone cylinder cyclide face object point polyline \
	    sphere sweep torus }
    # window viewport

    set instancableTypes { \
	    arealight camera group light \
	    beziercurve bezierpatch bsplinecurve bsplinepatch \
	    cone cylinder cyclide object polyline \
	    sphere sweep torus }

    # NOT INSTANCABLE
    # fog instance render face point 
    
    foreach type $slideTypes {
	set slideName($type) 1
    }

    ####################################################
    # mouse variables
    global paletteSelection

    set paletteSelection None

    ####################################################
    # instance variables
    global instanceWidth instanceRadius
    
    set instanceWidth 70
    set instanceRadius 8

    ####################################################
    # sizes
    global imageWidth imageHeight textHeight instanceTailHeight instanceHeadHeight
    global canvasHeight canvasWidth canvasTotalHeight canvasTotalWidth

    set imageWidth 36
    set imageHeight 36
    set instanceHeadHeight 0
    set textHeight [expr $imageHeight+10]
    set instanceTailHeight [expr $textHeight+10]

#    set textHeight -10
#    set instanceHeadHeight [expr $textHeight - 10]
#    set instanceTailHeight [expr $imageHeight]
}

proc Graph:InitImages {} {
    global slideTypes imageWidth graph_dir
    
    foreach type $slideTypes {
	if { $type != "instance" } {
	    image create photo $type
	    $type configure -file [file join $graph_dir images [subst $type][subst $imageWidth].gif]
	}
    }
}

proc Graph:GetNodeId { type } {
    global slideName

    set thisId $slideName($type)
    set slideName($type) [expr $thisId + 1]

    return $thisId
}

proc Graph:CreateButtons { w } {
    global slideGeometryTypes

    frame $w.layer1
    frame $w.layer2

    pack $w.layer1 -side top -fill x
    pack $w.layer2 -side top -fill x

    set layer1Types {\
	    arealight camera surface \
	    sphere torus cyclide sweep polyline beziercurve bezierpatch }

    set layer2Types {\
	    light group \
	    cylinder cone object face point bsplinecurve bsplinepatch }

    foreach type $layer1Types {
	Button $w.layer1.[subst $type] -image $type -helptext "$type"\
		-command "Graph:PaletteCreateNode $type" 
#		-armcommand "Graph:PaletteCreateNode $type" 

	if { ($type == "sweep") || ($type == "camera") || ($type == "arealight") || \
		($type == "surface") } {
	    $w.layer1.[subst $type] configure -state disabled
	}
    }	
    
    foreach type $layer2Types {
	Button $w.layer2.[subst $type] -image $type -helptext "$type" \
		-command "Graph:PaletteCreateNode $type" 
#		-armcommand "Graph:PaletteCreateNode $type" 

	if { ($type == "light") || \
	    ($type == "object") || ($type == "face") || ($type == "point") } {
	    $w.layer2.[subst $type] configure -state disabled
	}
    }	
    
    frame $w.layer1.space1 -width 36 -height 36
    frame $w.layer1.space2 -width 36 -height 36

    frame $w.layer2.space1 -width 36 -height 36
    frame $w.layer2.space2 -width 36 -height 36
    frame $w.layer2.space3 -width 44 -height 36

    pack $w.layer1.camera $w.layer1.arealight $w.layer1.space1\
	    $w.layer1.surface $w.layer1.space2 \
	    $w.layer1.sphere $w.layer1.torus $w.layer1.cyclide $w.layer1.sweep \
	    $w.layer1.polyline $w.layer1.beziercurve $w.layer1.bezierpatch \
	    -side left
    
    pack $w.layer2.space3 $w.layer2.light $w.layer2.space1 \
	    $w.layer2.group $w.layer2.space2\
	    $w.layer2.cylinder $w.layer2.cone $w.layer2.object $w.layer2.face \
	    $w.layer2.point $w.layer2.bsplinecurve $w.layer2.bsplinepatch \
	    -side left
}
