# Note: For the GUI to work the BWidget Library must be included (this is done in main.tcl)

global gui_dir

set editor_dir [file join $gui_dir editor]
source [file join $editor_dir editor.tcl]
set graph_dir [file join $gui_dir graph]
source [file join $graph_dir graph.tcl]

#############################################################################

proc GUI:Create { } {
#    global guiSystem
    global progress progressText guiExists

    set guiExists 1

#    set cur [.slfWINDOW.toglarea cget -cursor]
#    puts "cur: $cur"
#    .slfWINDOW config -cursor watch
#    . config -cursor left_ptr

    ProgressDlg .progress -parent . -width 25 -height 1 -maximum 22 \
	    -textvariable progressText -variable progress

    set progress 0
    set progressText "Loading..."

#    set dir [pwd]
    Editor:Create

#    if { $guiSystem == "irix" } {
#	cd "/project/cs/sequin/caffe/slide/lib/slide/gui/graph/"
#    } elseif { $guiSystem == "nt" } {	
#	cd "S:\\slide\\lib\\slide\\gui\\graph\\"
#    } else {
#	puts "ERROR in GUI:Create: SYSTEM not NT or IRIX"
#	exit
#    }

    incr progress

    Graph:Create
#    cd $dir

    #####################################
    # set variables
    DynamicHelp::configure -delay 300

    # Set bindings for main menu commands
    Main:SetBindings .graph
    Main:SetBindings .editor

    # set variables
    #####################################

    incr progress
    set progressText "Done"

    destroy .progress
#    puts "progress max = $progress"
}

proc GUI:Clear { } {

    Editor:Clear
    Graph:Clear
}

proc GUI:Destroy { } {
    global guiExists

    Editor:Destroy
    Graph:Destroy

    set guiExists 0
}

proc GUI:WriteSLIDE { } {
    global guiExists

    if { $guiExists == 1 } {
	Graph:SetSLIDEGraphPositions
    }

    slideWrite
}

proc GUI:InitGlobals { } {
    global textHighlightColor paleYellow paleOrange arrowColor
    global buttonFont labelFont entryFont progress

    set paleYellow #fefe28
    set paleOrange #fe7a3e
    set oliveGreen #8e9e4c
    set textHighlightColor $paleOrange
    set arrowColor #707070

    set buttonFont {Helvetica -14 bold}
    set entryFont  {Helvetica -12 bold}
    set labelFont  {Helvetica 11 bold}

    set progress 0

    global badNameMsg validNameMsg usedNameMsg

    set badNameMsg  "can not be used as a name." 
    set usedNameMsg "

This name is already used."
    set validNameMsg "

Names must start with an \"_\" or a letter.
All other characters can be an \"_\", letter, or number.
(i.e. names must satisfy the regular expression: 
\[a-zA-Z_\]\[a-zA-Z_0-9\]*)

Examples: a, A, _, a1, A__bC3_"

}

#########################################################################################

proc Global:CheckSLIDEName { } {
    global graphEntryTextId editNodeUserName userName 

    # Check the new name for correct syntax and if it's already used
    set len [string length $editNodeUserName]
    
    for {set i 0} {$i < $len} {incr i} {
	
	set c [string index $editNodeUserName $i]
	
	if { !(($i == 0) && ([string match {[a-zA-Z_]} $c])) && \
		!(($i > 0) && ([string match {[a-zA-Z0-9_]} $c])) } {
	    
	    break
	}
    }
    
    set oldName [.graph.canvas itemcget $graphEntryTextId -text]
    
    set alreadyInUse 0
    if { [info exists userName($editNodeUserName)] } {
	if { ($userName($editNodeUserName) == 1) && \
		($editNodeUserName != $oldName) } {
	    set alreadyInUse 1
	    set i 0
	}
    }

    if { ("" == $editNodeUserName) || ($i < $len) || ($alreadyInUse == 1) } {
	global entryFont labelFont badNameMsg validNameMsg usedNameMsg entryHideError

	set entryHideError 1

	# name already used
	if { $alreadyInUse == 1} {
	    set msg "\"$editNodeUserName\" $badNameMsg $usedNameMsg"
	} else {
	    set error "
	    
\"[string index $editNodeUserName $i]\" is the first incorrect character."
	    set msg "\"$editNodeUserName\" $badNameMsg $error $validNameMsg"
	}
	
	MessageDlg .msg -parent .graph -type ok -icon error \
		-justify left -font $labelFont -message $msg
	destroy .msg

	return [list bad $i]
    }
    return [list ok 0]
}

#########################################################################################

GUI:InitGlobals

puts "loaded caffe/slide/lib/slide/gui/gui.tcl"
