# ------------------------------------------------------------------------------
#  Methods:
#     - CreateOptionParameterFrame
#     - SetSLIDEOption
#
#     - CreateRibEntryFrame
#     
#     - GeomMenu:CreateNumberParameterFrame
#     - GeomMenu:DeltaCmd
#     - GeomMenu:SliderStep
#     - GeomMenu:Slider
#     - GeomMenu:SetEntryValue
#     - GeomMenu:EntryValue
#     - GeomMenu:UpdateSliderValue
#     - GeomMenu:ParseEntry
#     
#     - GeomMenu:UndoList
#     - GeomMenu:RedoList
#     - GeomMenu:UndoCmd
#     - GeomMenu:RedoCmd
#     - GeomMenu:SaveAlert
#     - GeomMenu:SaveCmd
#     - GeomMenu:SetSLIDEValues
#     - GeomMenu:SetValues
#     - GeomMenu:SetParameterValues
#     
# ------------------------------------------------------------------------------

proc CreateOptionParameterFrame { parent param y } {
    global [subst $param]Help [subst $param]Value [subst $param]List
    global labelFont

    set base $parent.$param
    frame $base -relief groove
    place $base -x 0 -y $y -width 375 -height 100

    set paramList [expr $[subst $param]List]
    
    Label $base.label -borderwidth 1 -font $labelFont -text $param \
	    -helptext [expr $[subst $param]Help]
    label $base.eq  -borderwidth 1 -font $labelFont -text = 
    ComboBox $base.menu -textvariable [subst $param]Value -font $labelFont\
	    -entrybg white -selectbackground red \
	    -values $paramList -height [expr [llength $paramList]] \
	    -modifycmd "SetSLIDEOption $param" -editable 0
	    
    place $base.label -x 10 -y 0 -anchor nw -bordermode ignore 
    place $base.eq    -x 80 -y 0 -anchor nw -bordermode ignore 
    place $base.menu  -x 95 -y 0 -anchor nw -bordermode ignore 
}

proc SetSLIDEOption { param } {
    global editNodeName editNodeType [subst $param]Value

    slideSetParameters $editNodeName $editNodeType 1 $param [expr $[subst $param]Value]
}

proc CreateRibEntryFrame { parent y } {
    global ribbeginHelp  ribbeginValue
    global ribendHelp  ribendValue
    global labelFont entryFont textHighlightColor buttonFont

    set base $parent.rib
    frame $base -relief groove
    place $base -x 0 -y $y -width 400 -height 200

    Label $base.label1 -borderwidth 1 -font $labelFont -text "ribbegin" \
	    -helptext $ribbeginHelp
    label $base.eq1  -borderwidth 1 -font $labelFont -text = 
    Entry $base.var1 -textvariable ribbeginValue -font $entryFont \
	    -background white  -selectbackground $textHighlightColor

    Label $base.label2 -borderwidth 1 -font $labelFont -text "ribend" \
	    -helptext $ribendHelp
    label $base.eq2  -borderwidth 1 -font $labelFont -text = 
    entry $base.var2 -textvariable ribendValue -font $entryFont \
	    -background white  -selectbackground $textHighlightColor

    Button $base.save -relief raised -text S -font $buttonFont -command "RibSave"\
	    -helptext "save ribbegin and ribend strings"


    place $base.label1 -x 10  -y 0 -anchor nw -bordermode ignore 
    place $base.eq1    -x 80  -y 0 -anchor nw -bordermode ignore 
    place $base.var1   -x 95  -y 0 -height 22 -width 265 -anchor nw -bordermode ignore 
    place $base.save   -x 364 -y 0 -height 54 -width 26  -anchor nw -bordermode ignore 

    place $base.label2 -x 10  -y 30 -anchor nw -bordermode ignore 
    place $base.eq2    -x 80  -y 30 -anchor nw -bordermode ignore 
    place $base.var2   -x 95  -y 30 -height 22 -width 265 -anchor nw -bordermode ignore 
}

proc RibSave { } {
    global editNodeName editNodeType editStandAlone ribbeginValue ribendValue

    if { $editStandAlone == 0 } {

	slideSetParameters $editNodeName $editNodeType 0 ribbegin $ribbeginValue ribend $ribendValue"
    }
}

#########################################################################################
# geometry frame

proc GeomMenu:CreateGeometry { type base parameters } {
    global buttonFont

    set sw [ScrolledWindow $base.sw -relief sunken -borderwidth 2]
    set lb [ListBox $sw.lb -deltay 55 -padx 0 -winpadx 0 -borderwidth 0 \
	    -highlightthickness 0 -dropenabled 0 -dragenabled 0]

    set i 0
    foreach param $parameters {

	set f [GeomMenu:CreateNumberParameterFrame $lb $type $param]
	$lb insert end foo[subst $i] -text "foo" -window $f
	incr i 
    }
    $sw setwidget $lb

    place $sw -x 10 -y 10 -width 375 -height 270

    # SAVE UNDO Buttons
    Button $base.save -text "Save" -command "GeomMenu:SaveCmd $type"   -font $buttonFont\
	    -state disabled -helptext "Save current settings"
    Button $base.undo  -text "Undo"  -command "GeomMenu:UndoCmd $type" -font $buttonFont\
	    -state disabled -helptext "Restore last Save"
    Button $base.redo  -text "Redo"  -command "GeomMenu:RedoCmd $type" -font $buttonFont\
	    -state disabled -helptext "Restore last Undo"

    place $base.save -x 10  -y 290 -width 100 -anchor nw -bordermode ignore 
    place $base.undo -x 150 -y 290 -width 100 -anchor nw -bordermode ignore 
    place $base.redo -x 288 -y 290 -width 100 -anchor nw -bordermode ignore 
}

proc GeomMenu:CreateOptions { base options } {

    set y 20
    foreach option $options {

	CreateOptionParameterFrame $base $option $y
	incr y 60
    }

    CreateRibEntryFrame $base $y
}

proc GeomMenu:CreateNumberParameterFrame { parent type param } {
    global [subst $param]Max [subst $param]Min  [subst $param]AbsMax [subst $param]AbsMin 
    global [subst $param]Value [subst $param]Step [subst $param]Entry
    global [subst $param]Help
    global labelFont entryFont textHighlightColor

    set base $parent.$param
#    frame $base -width 350 -height 55
    frame $base -width 352 -height 55

    Label $base.label -borderwidth 1 -font $labelFont -text $param \
	    -helptext [expr $[subst $param]Help]
    label $base.eq  -borderwidth 1 -font $labelFont -text = 
    Entry $base.var -textvariable [subst $param]Entry -font $entryFont \
	    -background white -selectbackground $textHighlightColor \
	    -command "GeomMenu:SetEntryValue $type $param" \
	    -helptext "[subst $param] value. 
Press <Return> to apply"

    set [subst $param]Entry [expr $[subst $param]Value]

    bind $base.var <KeyRelease>    "GeomMenu:EntryValue $type $param"

    scale $base.slider -width 16 -font {Helvetica -12} -borderwidth 2 \
	    -orient horiz -showvalue 0 \
	    -from [expr $[subst $param]Min] -to [expr $[subst $param]Max] \
	    -resolution [expr $[subst $param]Step] -variable [subst $param]Value \
	    -command "GeomMenu:Slider $type $param"

    Button $base.delta -relief raised -text D -font {Symbol -17} \
	    -helptext "show/hide slider step value" \
	    -command "GeomMenu:DeltaCmd $type $param"\

    Entry $base.min -textvariable [subst $param]Min -relief groove -font $entryFont \
	    -selectbackground $textHighlightColor \
	    -helptext "sets the minimum slider value
([expr $[subst $param]AbsMin] < v < [expr $[subst $param]AbsMax])" 

    Entry $base.max -textvariable [subst $param]Max -relief groove  -font $entryFont \
	    -selectbackground $textHighlightColor -justify right\
	    -helptext "sets the maximum slider value
([expr $[subst $param]AbsMin] < v < [expr $[subst $param]AbsMax])" 

    Entry $base.step -textvariable [subst $param]Step -font $entryFont \
	    -background white -selectbackground $textHighlightColor \
	    -helptext "sets the slider step value 
(if step value > 0 then the slider value is
rounded to an even multiple of this step)"
    bind $base.step <KeyRelease> "GeomMenu:SliderStep $type $param"

    label $base.cover -text ""

    place $base.label   -x 2   -y 1                       -anchor nw -bordermode ignore 
    place $base.eq      -x 90  -y 1                       -anchor nw -bordermode ignore
    place $base.var     -x 110 -y 1  -width 48 -height 22 -anchor nw -bordermode ignore
    place $base.slider  -x 170 -y 0  -width 125           -anchor nw -bordermode ignore 
    place $base.min     -x 172 -y 24 -width 50 -height 19 -anchor nw   
    place $base.delta   -x 222 -y 24 -width 20 -height 19 -anchor nw   
    place $base.max     -x 242 -y 24 -width 50 -height 19 -anchor nw   
    place $base.step    -x 300 -y 1  -width 48 -height 22 -anchor nw   
    place $base.cover   -x 300 -y 1  -width 48 -height 22 -anchor nw   

    raise $base.delta   $base.slider

    return $base
}

proc GeomMenu:DeltaCmd { type param } {
    global [subst $param]On [subst $type]GeomFrame

    set fr [expr $[subst $type]GeomFrame]

    if { [expr $[subst $param]On] } {
	raise $fr.sw.lb.$param.cover $fr.sw.lb.$param.step
	set [subst $param]On 0
    } else {
	raise $fr.sw.lb.$param.step $fr.sw.lb.$param.cover 
	set [subst $param]On 1
    }
}

proc GeomMenu:SliderStep { type param } {
    global [subst $param]Step [subst $type]GeomFrame
    
    set fr [expr $[subst $type]GeomFrame]

    # PARSE
    set ent [$fr.sw.lb.$param.step get]
    set newEnt [GeomMenu:ParseEntry $ent]

    # Check for partial number
    if { ($newEnt == "partial") } {
	
	return "partial"
    }

    set [subst $param]Step $newEnt
    
    # Check for parsed partial number
    if { ("" == $newEnt) || ("." == $newEnt) || ("-" == $newEnt) } {

	return
    }
    
    ####

    $fr.sw.lb.$param.slider configure -resolution [expr $[subst $param]Step]
}

proc GeomMenu:Slider { type param foo } {
    global [subst $param]Min [subst $param]Max [subst $param]AbsMin [subst $param]AbsMax
    global [subst $param]Value [subst $param]Entry [subst $type]GeomFrame
    global [subst $type][subst $param]SliderInit

    set fr [expr $[subst $type]GeomFrame]

    # Enforce Range
    if { [expr $[subst $param]Min] < [expr $[subst $param]AbsMin] } {

	set [subst $param]Min [expr $[subst $param]AbsMin]

    } elseif { [expr $[subst $param]Min] > [expr $[subst $param]AbsMax] } {
	
	set [subst $param]Min [expr $[subst $param]AbsMax]
    }
    if { [expr $[subst $param]Max] > [expr $[subst $param]AbsMax] } {
     
	set [subst $param]Max [expr $[subst $param]AbsMax]

    } elseif { [expr $[subst $param]Max] < [expr $[subst $param]AbsMin]  } {

	set [subst $param]Max [expr $[subst $param]AbsMin]
    }

    # Update slider
    $fr.sw.lb.$param.slider configure -from [expr $[subst $param]Min]
    $fr.sw.lb.$param.slider configure -to   [expr $[subst $param]Max]

    # Update entry
    set [subst $param]Entry [expr $[subst $param]Value]

    global editNodeName editNodeType editStandAlone
    if { ([expr $[subst $type][subst $param]SliderInit] != 0) } {

	GeomMenu:SaveAlert $type

	if { ($editStandAlone == 0) } {
	    
	    slideSetParameters $editNodeName $editNodeType 1 $param [expr $[subst $param]Value]
	}
    }
    set [subst $type][subst $param]SliderInit 1

    #puts "range: ([$fr.sw.lb.$param.slider cget -from],[$fr.sw.lb.$param.slider cget -to])"
}

proc GeomMenu:SetEntryValue { type param } {
    global editNodeName editNodeType editStandAlone [subst $param]Value
    global setEntryValue [subst $type]GeomFrame

    set fr [expr $[subst $type]GeomFrame]

    set setEntryValue 1
    $fr.sw.lb.$param.var configure -foreground black
    focus .editor

    if { $editStandAlone == 0 } {
	
	slideSetParameters $editNodeName $editNodeType 1 $param [expr $[subst $param]Value]
    }
}

proc GeomMenu:EntryValue { type param } {
    global [subst $param]AbsMin [subst $param]Min [subst $param]AbsMax [subst $param]Max 
    global [subst $param]Value [subst $param]Entry [subst $type]GeomFrame
    global setEntryValue

    set fr [expr $[subst $type]GeomFrame]

    GeomMenu:SaveAlert $type

    if { $setEntryValue == 0 } {
	$fr.sw.lb.$param.var configure -foreground red
    } 
    set setEntryValue 0

    set ent [$fr.sw.lb.$param.var get]
    set newEnt [GeomMenu:ParseEntry $ent]

    # Check for partial number
    if { ($newEnt == "partial") } {

	return "partial"
    }

    set [subst $param]Entry $newEnt

    # Check for parsed partial number
    if { ("" == $newEnt) || ("." == $newEnt) || ("-" == $newEnt) } {

	return
    }

    # Update the slider
    GeomMenu:UpdateSliderValue $type $param [expr $[subst $param]Entry + 0]
}

proc GeomMenu:UpdateSliderValue { type param value } {
    global [subst $param]Min [subst $param]Max [subst $param]AbsMin [subst $param]AbsMax
    global [subst $param]Value [subst $param]Entry
    global [subst $type]GeomFrame

    set fr [expr $[subst $type]GeomFrame]

    $fr.sw.lb.$param.slider configure -from [expr $[subst $param]AbsMin]
    $fr.sw.lb.$param.slider configure -to   [expr $[subst $param]AbsMax]

    set [subst $param]Value $value
    # TCL updates this variable
    set value [expr $[subst $param]Value]
    set [subst $param]Entry $value

    if { $value > [expr $[subst $param]Max] } {

	puts "here: $value >  [expr $[subst $param]Max] "

	if { [expr 2*$value] > [expr $[subst $param]AbsMax] } {
	    set [subst $param]Max [expr $[subst $param]AbsMax]
	} else {
	    set [subst $param]Max [expr $value * 2]
	}
    } elseif { $value < [expr $[subst $param]Min] } {

	puts "there: $value <  [expr $[subst $param]Min] "

	if { [expr 2*$value] > [expr $[subst $param]AbsMax] } {
	    set [subst $param]Min [expr $[subst $param]AbsMin]
	} else {
	    set [subst $param]Min [expr $value / 2]
	}
    }
    $fr.sw.lb.$param.slider configure -to   [expr $[subst $param]Max]
    $fr.sw.lb.$param.slider configure -from [expr $[subst $param]Min]
}

proc GeomMenu:ParseEntry { ent } {

    #puts "ent: $ent"

    set len [string length $ent]
    set str ""
    set decimals 0

    # Check for partial number
    if { ("" == $ent) || ("." == $ent) || ("-" == $ent) } {

	return "partial"
    }
    
    # Check and Correct for valid number
    for {set i 0} {$i < $len} {incr i} {
	
	set c [string index $ent $i]
	
	if { (($i == 0) && ([string match - $c])) || \
		([string match {[0-9]} $c])       } {

	    append str $c	    

	} elseif { ([string match \. $c]) } {

	    incr decimals
	    
	    if { $decimals == 1 } {
		
		append str $c	    
	    }
	}
    }
    return $str
}

#########################################################################################
# geomFrame functions



#########################################################################################
#  Undo Save Redo

proc GeomMenu:UndoList { type cmd values } {
    global [subst $type]UndoList [subst $type]GeomFrame

    set undoList [expr $[subst $type]UndoList]
    set fr       [expr $[subst $type]GeomFrame]

    if { $cmd == "push" } {

	lappend undoList $values

    } elseif { $cmd == "pop" } {

	set lastIndex [expr [llength $undoList] - 1]
	set values [lindex $undoList $lastIndex]
	set undoList [lreplace $undoList $lastIndex $lastIndex]
	GeomMenu:RedoList $type push $values

	set values [lindex $undoList [expr $lastIndex-1]]
	GeomMenu:SetValues $type $values 0

    } elseif { $cmd == "top" } {

	set lastIndex [expr [llength $undoList] - 1]
	set values [lindex $undoList $lastIndex]
	GeomMenu:SetValues $type $values 0
    }

    if { 1 == [llength $undoList] } {	
	$fr.undo configure -state disabled
    } elseif { 1 < [llength $undoList] } {
	$fr.undo configure -state normal
    }

    set [subst $type]UndoList $undoList

    return $values
}

proc GeomMenu:RedoList { type cmd values } {
    global [subst $type]RedoList [subst $type]GeomFrame
    
    set redoList [expr $[subst $type]RedoList]
    set fr       [expr $[subst $type]GeomFrame]

    if { $cmd == "push" } {
	lappend redoList $values
	$fr.redo configure -state normal

    } elseif { $cmd == "pop" } {

	set lastIndex [expr [llength $redoList] - 1]
	set values [lindex $redoList $lastIndex]
	set redoList [lreplace $redoList $lastIndex $lastIndex]
	GeomMenu:UndoList $type push $values

	GeomMenu:SetValues $type $values 0

	if { $redoList == "" } {
	    
	    $fr.redo configure -state disabled
	}
    }

    set [subst $type]RedoList $redoList

    return $values
}

proc GeomMenu:UndoCmd { type } {
    global [subst $type]GeomFrame

    set fr [expr $[subst $type]GeomFrame]

    if { "disabled" == [$fr.save cget -state] } {

	set values [GeomMenu:UndoList $type pop 0]
    } else {

	#puts "Restoring last saved values"
	set values [GeomMenu:UndoList $type top 0]
    }
    GeomMenu:SetSLIDEValues $type $values

    $fr.save configure -state disabled

    global [subst $type]UndoList [subst $type]RedoList
    set undoList [expr $[subst $type]UndoList]
    set redoList [expr $[subst $type]RedoList]

    puts "undo($type): $undoList"
    puts "redo($type): $redoList"
}

proc GeomMenu:RedoCmd { type } {
    global [subst $type]GeomFrame

    set fr [expr $[subst $type]GeomFrame]

    set values [GeomMenu:RedoList $type pop 0]
    GeomMenu:SetSLIDEValues $type $values

    $fr.save configure -state disabled

    global [subst $type]UndoList [subst $type]RedoList
    set undoList [expr $[subst $type]UndoList]
    set redoList [expr $[subst $type]RedoList]

    puts "undo($type): $undoList"
    puts "redo($type): $redoList"
}

proc GeomMenu:SaveAlert { type } {
    global [subst $type]GeomFrame

    set fr [expr $[subst $type]GeomFrame] 

    $fr.undo configure -state normal
    $fr.save configure -state normal

    #$fr.save configure -foreground red
    #$fr.save configure -activeforeground red
}

proc GeomMenu:SaveCmd { type } {
    global radiusValue zminValue zmaxValue zslicesValue thetamaxValue thetaslicesValue
    global solidValue surfaceValue shadingValue lodValue ribbeginValue ribendValue
    global [subst $type]GeomFrame
    
    set fr [expr $[subst $type]GeomFrame] 
    
    # unalert
    #$fr.sw.lb.$param.var configure -foreground black
    $fr.save configure -state disabled
    $fr.save configure -foreground black
    $fr.save configure -activeforeground black

    set values "{radius $radiusValue} {zmin $zminValue} {zmax $zmaxValue} {zslices $zslicesValue} {thetamax $thetamaxValue} {thetaslices $thetaslicesValue}"

    GeomMenu:SetSLIDEValues $type $values
    GeomMenu:UndoList $type push $values

    global [subst $type]UndoList [subst $type]RedoList
    set undoList [expr $[subst $type]UndoList]
    set redoList [expr $[subst $type]RedoList]

    puts "undo($type): $undoList"
    puts "redo($type): $redoList"
}

proc GeomMenu:SetSLIDEValues { type values } {
    global editNodeName editStandAlone

    set argc [llength $values]

    set i 0
    foreach pair $values {
	
	set param [lindex $pair 0]
	set value [lindex $pair 1]

	set arg[subst $i] $param
	incr i 1
	set arg[subst $i] $value
	incr i 1
    }

    set cmd "slideSetParameters $editNodeName $type 1"
    for {set i 0} {$i < [expr 2*$argc]} {incr i 1} {
	
	set cmd [concat $cmd "\$arg[subst $i]" ]
    }

    if { $editStandAlone == 0 } {

	eval $cmd
    }
}

proc GeomMenu:SetValues { type values resetUndoList } {

    # Set values
    foreach pair $values {

	set param [lindex $pair 0]
	set value [lindex $pair 1]

	GeomMenu:UpdateSliderValue $type $param $value
    }

    # Undo list
    if { $resetUndoList == 1 } {

	global [subst $type]UndoList [subst $type]RedoList
	set [subst $type]UndoList {}
	set [subst $type]RedoList {}
    }
}

proc GeomMenu:SetParameterValues { type paramValues } {
    global solidValue surfaceValue shadingValue lodValue ribbeginValue ribendValue

    # Set options
    set values ""

    foreach pair $paramValues {
	
	set param [lindex $pair 0]
	set value [lindex $pair 1]
	
	if { $value == "(null)" } {
	    set value ""
	}
	
	if { ($param == "solid") || ($param == "surface") || ($param == "shading") || \
		($param == "lod") || ($param == "ribbegin") || ($param == "ribend") } {

	    set [subst $param]Value $value
	} else {

	    lappend values $pair
	}
    }

    GeomMenu:SetValues $type $values 1
    GeomMenu:UndoList $type push $values
}
