proc ScaleToByte { value } {
    return [expr int($value * 255)]
}

proc RemovePeriods { name } {
    regsub -all {\.} $name _ y
    return $y
}

proc DeclareGlobal { name } {
    set varName [eval {RemovePeriods $name}]
    global $varName
    return $varName
}

proc CreateButton { parent name text object method } {
    set root $parent.$name

    button $root -text $text -command "$object $method"
}

proc CreateCheckButton { parent name text object field method } {
    set root $parent.$name
    set var "[subst $object]_[subst $field]"

    checkbutton $root -text $text -variable $var -command "$object $method \$$var"
}

#list is an enumerated list { {name0 value0} ... }
proc CreateChoiceBox { parent name text object field method enumList } {
    set root $parent.$name

    menubutton $root -text $text -menu $root.choicemenu -relief raised -bd 2

    menu $root.choicemenu
    set varChoice "[subst $object]_[subst $field]"

    foreach choice $enumList {
	set name [eval {lindex $choice 0}]
	set value [eval {lindex $choice 1}]
	$root.choicemenu add radiobutton -label $name -variable $varChoice -value $value -command "$object $method $value"
    }
}

proc CreateEnumLabel {name enumList} {
    set maxWidth 0
    foreach choice $enumList {
	set enumName [eval {lindex $choice 0}]
	if { [string length $enumName] > $maxWidth } {
	    set maxWidth [string length $enumName]
	}
    }
    label $name -width $maxWidth -anchor w -text [lindex [lindex $enumList 0] 0]
}

proc CreateScale { parent name text object field method tickints orient } {

    set root $parent.$name
    set var "[subst $object]_[subst $field]"
    set varMin "[subst $var]_Min"
    set varMax "[subst $var]_Max"
    set varResolution "[subst $var]_Resolution"
    
    global $varMin $varMax $varResolution

#puts "varMin = $varMin"
#puts "min = [subst $$varMin]"
#puts "varMax = $varMax"
#puts "max = [subst $$varMax]"

    set tickinterval [expr ([subst $$varMax] - [subst $$varMin]) / ($tickints)]

#puts "root = $root"
#puts "text = $text"
#puts "varMin = $varMin"
#puts "min = [subst $$varMin]"
#puts "varMax = $varMax"
#puts "max = [subst $$varMax]"
#puts "varResolution = $varResolution"
#puts "resolution = [subst $$varResolution]"
#puts "tickinterval = $tickinterval"
#puts "orient = $orient"
#puts "var = $var"
#puts "object = $object"
#puts "method = $method"

    scale $root -label $text -from [subst $$varMin] -to [subst $$varMax] \
	    -resolution [subst $$varResolution] -tickinterval $tickinterval \
	    -orient $orient -var $var -command "$object $method"
}

proc SetScaleVectorHelper { object field method indexes index value } {

    set varBase "[subst $object]_[subst $field]"
    set size [llength $indexes]
    set command_string "$object $method"

    for {set i 0} {$i < $size} {incr i} {
	if { $i == $index } {
	    append command_string " " $value 
	} else {
	    set var [subst $varBase][lindex $indexes $i]
	    global $var
	    append command_string " " [subst $$var]
	}	    
    }
    
    eval $command_string
}

proc CreateScaleVector { parent name text object field method tickints orient indexes } {
    set root $parent.$name
    set varBase "[subst $object]_[subst $field]"
    set varMin "[subst $varBase]_Min"
    set varMax "[subst $varBase]_Max"
    set varResolution "[subst $varBase]_Resolution"

    global $varMin $varMax $varResolution

    set tickinterval [expr ([subst $$varMax] - [subst $$varMin]) / ($tickints)]

    set size [llength $indexes]

    for {set i 0} {$i < $size} {incr i} {
	set index [lindex $indexes $i]
	set var "[subst $varBase][subst $index]"
	set rootCurr "[subst $root][subst $index]" 
	set textCurr "[subst $text][subst $index]"

	scale $rootCurr -label $textCurr -from [subst $$varMin] -to [subst $$varMax] \
	    -resolution [subst $$varResolution] -tickinterval $tickinterval \
	    -orient $orient -var $var \
	    -command "SetScaleVectorHelper $object $field $method {$indexes} $i"
    }
}

proc PopupVectorDialog { name text object field method tickints orient indexes } {
    set root .$name

    toplevel $root

    frame $root.scales -relief raised -bd 2
    
    CreateScaleVector $root.scales $name "" $object $field $method $tickints $orient \
	    $indexes

    set size [llength $indexes]

    for {set i 0} {$i < $size} {incr i} {
	pack $root.scales.[subst $name][lindex $indexes $i] -side top -fill x
    }

    pack $root.scales -side top -fill x -expand 1
}

proc CreateVectorDialog { parent name text object field method tickints orient indexes } {
    set root $parent.$name

    button $root -text $text -command "PopupVectorDialog $name $text $object $field \
	    $method $tickints $orient {$indexes}"
}

proc SetScaleColorHelper { object field method indexes index sample value } {

    set varBase "[subst $object]_[subst $field]"
    set size [llength $indexes]
    set command_string "$object $method"

    for {set i 0} {$i < $size} {incr i} {
#	if { $i == $index } {
#	    append command_string " " $value 
#	} else {
	    set var [subst $varBase][lindex $indexes $i]
	    global $var
	    append command_string " " [subst $$var]
#	}	    
    }

    eval $command_string

    # Update the color of the sample
    set varR [subst $varBase][lindex $indexes 0]
    set varG [subst $varBase][lindex $indexes 1]
    set varB [subst $varBase][lindex $indexes 2]

    global $varR $varG $varB
    set byteR [ScaleToByte [subst $$varR]]
    set byteG [ScaleToByte [subst $$varG]]
    set byteB [ScaleToByte [subst $$varB]]

    $sample conf -bg [format \#%02x%02x%02x $byteR $byteG $byteB]
}

proc CreateScaleColor { parent name text object field method tickints orient indexes } {
    set root $parent.$name
    set varBase "[subst $object]_[subst $field]"
    set varMin "[subst $varBase]_Min"
    set varMax "[subst $varBase]_Max"
    set varResolution "[subst $varBase]_Resolution"
    set sample [subst $root]Sample

    global $varMin $varMax $varResolution

    set tickinterval [expr ([subst $$varMax] - [subst $$varMin]) / ($tickints)]

    frame $sample -width 200 -height 100 -bg [format \#%02x%02x%02x 0 0 0]

    set size [llength $indexes]

    for {set i 0} {$i < $size} {incr i} {
	set index [lindex $indexes $i]
	set var "[subst $varBase][subst $index]"
	set rootCurr "[subst $root][subst $index]" 
	set textCurr "[subst $text][subst $index]"

	scale $rootCurr -label $textCurr -from [subst $$varMin] -to [subst $$varMax] \
	    -resolution [subst $$varResolution] -tickinterval $tickinterval \
	    -orient $orient -var $var \
	    -command "SetScaleColorHelper $object $field $method {$indexes} $i $sample"
    }
}

proc PopupColorDialog { name text object field method tickints orient indexes } {
    set root .$name

    toplevel $root

    frame $root.scales -relief raised -bd 2
    
    CreateScaleColor $root.scales $name "" $object $field $method $tickints $orient \
	    $indexes

    set size [llength $indexes]

    for {set i 0} {$i < $size} {incr i} {
	pack $root.scales.[subst $name][lindex $indexes $i] -side top -fill x
    }

    pack $root.scales.[subst $name]Sample -side top -fill x

    pack $root.scales -side top -fill x -expand 1
}

proc CreateColorDialog { parent name text object field method tickints orient indexes } {
    set root $parent.$name

    button $root -text $text -command "PopupColorDialog $name $text $object $field \
	    $method $tickints $orient {$indexes}"
}

proc CreateMenuButton { parent name text } {
    set root $parent.$name

    set f {Helvetica 11 bold}

    menubutton $root -font $f -text $text -underline 0 -menu $root.menu
}

proc CreateMenu { parent name listMenuItems} {
    set root $parent.$name

    menu $root

    set f {Helvetica 11 bold}

    foreach menuItem $listMenuItems {
	set type [eval {lindex $menuItem 0}]

	if { $type == "command" } {
	    set text [eval {lindex $menuItem 1}]
	    set command [eval {lindex $menuItem 2}]
	    set underline [eval {lindex $menuItem 3}]
	    set acc [eval {lindex $menuItem 4}]

	    if { $underline == -1 } {
		$root add command -font $f -label $text -command $command -accelerator $acc

	    } else {
		$root add command -font $f -label $text -command $command -accelerator $acc\
			-underline $underline 
	    }

	} elseif { $type == "checkbutton" } {
	    set text [eval {lindex $menuItem 1}]
	    set object [eval {lindex $menuItem 2}]
	    set field [eval {lindex $menuItem 3}]
	    set method [eval {lindex $menuItem 4}]
	    set underline [eval {lindex $menuItem 5}]
	    set acc [eval {lindex $menuItem 6}]

	    set var "[subst $object]_[subst $field]"

	    if { $underline == -1 } {
		$root add checkbutton -font $f -label $text -variable $var \
			-command "$object $method \$$var" -accelerator $acc 

		#puts "A: -command $object $method \$$var"

	    } else {
		$root add checkbutton -font $f -label $text -variable $var \
			-command "$object $method \$$var" -accelerator $acc \
			-underline $underline
	    }

	} elseif { $type == "radiobutton" } {
	    set text [eval {lindex $menuItem 1}]
	    set value [eval {lindex $menuItem 2}]
	    set object [eval {lindex $menuItem 3}]
	    set field [eval {lindex $menuItem 4}]
	    set method [eval {lindex $menuItem 5}]

	    set var "[subst $object]_[subst $field]"

	    $root add radiobutton -font $f -label $text -value $value -variable $var \
		    -command "$object $method \$$var"

	} elseif { $type == "cascade" } {
	    set name [eval {lindex $menuItem 1}]
	    set text [eval {lindex $menuItem 2}]
	    set subListMenuItems [eval {lindex $menuItem 3}]

	    $root add cascade -label $text -font $f -menu $root.$name
	    CreateMenu $root $name $subListMenuItems
	
	} elseif { $type == "separator" } {

	    $root add separator
	}
    }
}

proc CreateMenuBar { parent name listMenus } {
    set root $parent.$name

    frame $root -relief raised -bd 2

    foreach submenu $listMenus {
	set subname [eval {lindex $submenu 0}]
	set text [eval {lindex $submenu 1}]
	set side [eval {lindex $submenu 2}]
	set listMenuItems [eval {lindex $submenu 3}]

	CreateMenuButton $root $subname $text
        CreateMenu $root.$subname menu $listMenuItems

	pack $root.$subname -side $side
    }
}

proc CreateToolBar { parent name list } {
    set root $parent.$name
    set i 0

    frame $root -relief raised -bd 2
    foreach tool $list {
	set type [eval {lindex $tool 0}]

	if { $type == "button" } {
	    set text [eval {lindex $tool 1}]
	    set object [eval {lindex $tool 2}]
	    set method [eval {lindex $tool 3}]

	    button $root.button$i -text $text -command "$object $method"

	    pack $root.button$i -side left -padx 2 -pady 2 -expand 1
	} elseif { $type == "text" } {
	    set initValue [eval {lindex $tool 1}]

	    label $root.text$i -text $initValue

	    pack $root.text$i -side left -padx 2 -pady 2 -expand 1
	} elseif { $type == "textOut" } {
	    set object [eval {lindex $tool 1}]
	    set field [eval {lindex $tool 2}]
	    #set initValue [eval {lindex $tool 3}]
	    set varText "[subst $object]_[subst $field]"

	    label $root.textOut$i -width 10 -anchor w -text "Jordan" -textvariable $varText -relief sunken

	    pack $root.textOut$i -side left -padx 2 -pady 2 -expand 1
	}
	incr i
    }
}

proc DestroyWindow { parent name } {
    if { $name == {} } {
	return false
    }

    if { $parent == {} } {
	set root .$name
    } elseif { $parent == "." } {
	set root .$name
    } else {
	set root $parent.$name
    }

    set varWindow $name

    destroy $root
}

proc CreateWindow { parent name originX originY sizeX sizeY bResize iAspectRatio } {
    if { $name == {} } {
	return false
    }

    if { $parent == {} } {
	set root .$name
    } elseif { $parent == "." } {
	set root .$name
    } else {
	set root $parent.$name
    }

    set windowName [string trimleft $name "slf"]
    wm title    $root "Display for Window: $windowName"
    wm protocol $root WM_DELETE_WINDOW {set foo 1}

    set screenWidth  [winfo screenwidth .]
    set screenHeight [winfo screenheight .]
    global guiWindow 
    set guiWindow $root

    ######################################################################
    # Define the Square virtual screen which SLIDE addresses

    global SLF_FULL_SCREEN
    if { $iAspectRatio == $SLF_FULL_SCREEN } {
	set virtualScreenWidth   $screenWidth
	set virtualScreenHeight  $screenHeight
	set virtualScreenOriginX 0 
	set virtualScreenOriginY 0 

	# Change the size and origin parameters so that we use the whole screen
	set sizeX 1.0
	set sizeY 1.0
	set originX 0.0
	set originX 0.0

    } else {
	if { $screenWidth < $screenHeight } {
	    set virtualScreenWidth $screenWidth
	} else {
	    set virtualScreenWidth $screenHeight
	}
	set virtualScreenHeight $virtualScreenWidth
	set virtualScreenOriginX [expr int(($screenWidth  - $virtualScreenWidth)/2)] 
	set virtualScreenOriginY [expr int(($screenHeight - $virtualScreenHeight)/2)] 
    }

    if { $sizeX <= 0 } {
	set sizeX 0.5
    }
    if { $sizeY <= 0 } {
	set sizeY 0.5
    }
    if { $originX < 0 } {
	set originX 0.25
    }
    if { $originY < 0 } {
	set originY 0.25
    }

    set pixelX      [expr int($virtualScreenOriginX + $virtualScreenWidth * $originX)]
    set pixelY      [expr int($virtualScreenOriginY + $virtualScreenHeight * $originY)]
    set pixelWidth  [expr int($virtualScreenWidth  * $sizeX)]
    set pixelHeight [expr int($virtualScreenHeight * $sizeY)]

    set varWindow $name 


    frame $root.menubar 

    CreateMenuBar $root.menubar options \
      [list \
	[list options Options left \
	  [list \
            [list cascade cameramode "Camera Mode" \
	      [list \
	        [list checkbutton "Enabled" $varWindow crystalBallEnabled setCrystalBallEnabled -1 "<u>"] \
	        [list separator] \
	        [list radiobutton "Pause"    1 $varWindow cuiType setCUIType ] \
	        [list radiobutton "CAD"      2 $varWindow cuiType setCUIType ] \
	        [list radiobutton "FLY"      3 $varWindow cuiType setCUIType ] \
	        [list radiobutton "EXPLORE"  4 $varWindow cuiType setCUIType ] \
	        [list separator] \
	        [list command     "Reset" "$varWindow resetCamera" -1 ""] \
	      ] \
	    ] \
	    [list cascade draw "Draw" \
	      [list \
	        [list checkbutton "Normals" $varWindow drawNormals setDrawNormals 0 "<n>"] \
	        [list checkbutton "Bounds" $varWindow drawBounds setDrawBounds 0 "<b>"] \
	        [list checkbutton "CDA Ops" $varWindow drawCDAOperands setDrawCDAOperands 5 "<o>"] \
                [list checkbutton "Clip Bound" $varWindow drawClipBound setDrawClipBound 0 "<c>"] \
	        [list checkbutton "Viewport Bound" $varWindow drawViewportBound setDrawViewportBound 0 "<v>"] \
              ] \
	    ] \
            [list cascade cull "Cull" \
	      [list \
	        [list checkbutton "Back Face" $varWindow backFaceCullingEnabled setBackFaceCullingEnabled -1 "<x>"] \
	        [list checkbutton "Bound" $varWindow boundCullingEnabled setBoundCullingEnabled -1 "<r>"] \
              ] \
	    ]\
            [list cascade appearance "Appearance" \
	      [list \
	        [list checkbutton "Lighting Enabled" $varWindow lightingEnabled setLightingEnabled 0 "<l>"] \
	        [list checkbutton "Fog Enabled" $varWindow fogEnabled setFogEnabled 0 "<f>"] \
                [list checkbutton "Texture Mapping Enabled" $varWindow textureMappingEnabled setTextureMappingEnabled 0 "<t>"] \
              ] \
	    ] \
            [list cascade bounds "Bounds" \
	      [list \
                [list checkbutton "Color Dyn Bounds" $varWindow colorDynamicBounds setColorDynamicBounds 6 "<d>"] \
	        [list checkbutton "Color Altered Bounds" $varWindow colorChangedBounds setColorChangedBounds 6 "<a>"] \
	        [list checkbutton "Check Bound Altered" $varWindow checkBoundChanged setCheckBoundChanged -1 "<h>"] \
	      ] \
            ] \
            [list cascade dynamics "Dynamics" \
	      [list \
	        [list command "Start" "$varWindow handleStartDynamics" 0 "<s>"] \
	        [list command "Pause" "$varWindow handlePauseDynamics" 0 "<p>"] \
	      ] \
	    ] \
	  ] \
	] \
      ]

    # menu radiobutton variables
    set cuiType "[subst $varWindow]_cuiType"
    global $cuiType
    set $cuiType 2



    set f {Helvetica 11 bold}

    frame $root.readout -relief raised -bd 2 
    label $root.readout.ut  -text "  Update Time =" 
    label $root.readout.utv -textvariable [subst $varWindow]_updateTime 
    label $root.readout.space  -text "   " 
    label $root.readout.rt  -text "Render Time =" -height 2
    label $root.readout.rtv -textvariable [subst $varWindow]_renderTime
    pack $root.readout.ut $root.readout.utv \
	    $root.readout.space \
	    $root.readout.rt $root.readout.rtv -side left -fill both 


    pack $root.menubar.options -side left -fill both
    
    frame $root.menubar.mouse -relief raised -bd 2

    Button $root.menubar.mouse.s -text "s" -command "Window:MouseMode $varWindow sel" \
	    -helptext "Selection Mode"
    Button $root.menubar.mouse.c -text "c" -command "Window:MouseMode $varWindow cam" \
	    -helptext "Camera Mode"

    pack $root.menubar.mouse.c $root.menubar.mouse.s -side left -fill both
    pack $root.menubar.mouse -side left -fill both
    pack $root.menubar -side top -fill x
    pack $root.readout -side top -fill x


    ##################################################
    # Togl Widget
    #

    # Do this so that we will know the correct offset to use to place the window
    wm geometry $root +0+0
    update idletasks

    togl $root.toglarea -width $pixelWidth -height $pixelHeight -rgba true -double true -depth true -stencil true -ident %name

    # Must set bindings so <Control-b> and <Key-b> are different events in toglarea
    Main:SetBindings $root.toglarea
    bind $root.toglarea <KeyPress> "$varWindow handleKeyboard %K 0 %x %y"

    #bind $root.toglarea <KeyRelease> "$varWindow handleKeyboard %K 1 %x %y"
    bind $root.toglarea <ButtonPress> "$varWindow handleMouseButton %b 0 %x %y"
    bind $root.toglarea <ButtonRelease> "$varWindow handleMouseButton %b 1 %x %y"
    bind $root.toglarea <B1-Motion> "$varWindow handleMouseMotion 1 %x %y"
    bind $root.toglarea <B2-Motion> "$varWindow handleMouseMotion 2 %x %y"
    bind $root.toglarea <B3-Motion> "$varWindow handleMouseMotion 3 %x %y"

    wm minsize $root 1 1
    #wm maxsize $root 640 480
    wm maxsize $root $screenWidth $screenHeight

    if { $bResize } {
	wm resizable $root 1 1
    } else {
	wm resizable $root 0 0
    }

    if { $iAspectRatio == 0 } {
	wm aspect $root $pixelWidth $pixelHeight $pixelWidth $pixelHeight
    }

    focus $root.toglarea

    pack $root.toglarea -side left -fill both -expand 1

    # Place the window in the correct location
#    set winHeight [expr [winfo reqheight $root.menubar] + [winfo reqheight $root.readout1] + [winfo reqheight $root.readout2] + [winfo reqheight $root.toglarea]]
    set winHeight [expr [winfo reqheight $root.menubar] + [winfo reqheight $root.toglarea]]

    set pixelY [expr $screenHeight - $winHeight - $pixelY]

#    wm geometry $root +$pixelX+$pixelY
# jane

    set x [expr int($originX*$screenWidth)]
    set y [expr int($originY*$screenHeight)]
    wm geometry $root +$x+$y

    $root.toglarea config -cursor exchange
    $root.menubar.mouse.c configure -relief sunken

    return true
}

proc ResizeWindow { } {
#    global guiSystem
    global env
    global guiMode guiWindow guiGeom

    # Assumes one window
    if { $guiMode == "Editor" } {

	# Save current geometry
	set guiGeom [wm geometry $guiWindow]

	switch $env(SYSTEM) {
	    "IRIX" {
		wm geometry $guiWindow 420x451+124+92
	    }
	    "NT" {
		wm geometry $guiWindow 420x451+120+65
	    }
	    default {
		puts "ERROR ResizeWindow: System not IRIX or NT"
		exit
	    }
	}	
    } else {
	wm geometry $guiWindow $guiGeom
    }
}

proc Window:MouseMode { varWindow type } {

    #set var [subst $varWindow]_selectionEnabled
    #global $var

    if { $type == "cam" } {

	eval "$varWindow setSelectionEnabled 0"
    } else {

	eval "$varWindow setSelectionEnabled 1"
    }
}

proc Window:SelectionMode { window bSelection } {

    if { $bSelection == 0 } { 
	
	# crystal ball 
	$window.menubar.mouse.c configure -relief sunken
	$window.menubar.mouse.s configure -relief raised

	$window.toglarea config -cursor exchange

    } else {                  

	# selection
	$window.menubar.mouse.c configure -relief raised
	$window.menubar.mouse.s configure -relief sunken

	$window.toglarea config -cursor left_ptr
    }
}

puts "loaded caffe/slide/lib/slide/window.tcl"

