proc ScaleToByte { value } {
    return [expr int($value * 255)]
}

proc RemovePeriods { name } {
    regsub -all {\.} $name _ y
    return $y
}

proc DeclareGlobal { name } {
    set varName [eval {RemovePeriods $name}]
    global $varName
    return $varName
}

proc CreateButton { parent name text object method } {
    set root $parent.$name

    button $root -text $text -command "$object $method"
}

proc CreateCheckButton { parent name text object field method } {
    set root $parent.$name
    set var "[subst $object]_[subst $field]"

    checkbutton $root -text $text -variable $var -command "$object $method \$$var"
}

#list is an enumerated list { {name0 value0} ... }
proc CreateChoiceBox { parent name text object field method enumList } {
    set root $parent.$name

    menubutton $root -text $text -menu $root.choicemenu -relief raised -bd 2

    menu $root.choicemenu
    set varChoice "[subst $object]_[subst $field]"

    foreach choice $enumList {
	set name [eval {lindex $choice 0}]
	set value [eval {lindex $choice 1}]
	$root.choicemenu add radiobutton -label $name -variable $varChoice -value $value -command "$object $method $value"
    }
}

proc CreateEnumLabel {name enumList} {
    set maxWidth 0
    foreach choice $enumList {
	set enumName [eval {lindex $choice 0}]
	if { [string length $enumName] > $maxWidth } {
	    set maxWidth [string length $enumName]
	}
    }
    label $name -width $maxWidth -anchor w -text [lindex [lindex $enumList 0] 0]
}

proc CreateScale { parent name text object field method tickints orient } {

    set root $parent.$name
    set var "[subst $object]_[subst $field]"
    set varMin "[subst $var]_Min"
    set varMax "[subst $var]_Max"
    set varResolution "[subst $var]_Resolution"
    
    global $varMin $varMax $varResolution

#puts "varMin = $varMin"
#puts "min = [subst $$varMin]"
#puts "varMax = $varMax"
#puts "max = [subst $$varMax]"

    set tickinterval [expr ([subst $$varMax] - [subst $$varMin]) / ($tickints)]

#puts "root = $root"
#puts "text = $text"
#puts "varMin = $varMin"
#puts "min = [subst $$varMin]"
#puts "varMax = $varMax"
#puts "max = [subst $$varMax]"
#puts "varResolution = $varResolution"
#puts "resolution = [subst $$varResolution]"
#puts "tickinterval = $tickinterval"
#puts "orient = $orient"
#puts "var = $var"
#puts "object = $object"
#puts "method = $method"

    scale $root -label $text -from [subst $$varMin] -to [subst $$varMax] \
	    -resolution [subst $$varResolution] -tickinterval $tickinterval \
	    -orient $orient -var $var -command "$object $method"
}

proc SetScaleVectorHelper { object field method indexes index value } {

    set varBase "[subst $object]_[subst $field]"
    set size [llength $indexes]
    set command_string "$object $method"

    for {set i 0} {$i < $size} {incr i} {
	if { $i == $index } {
	    append command_string " " $value 
	} else {
	    set var [subst $varBase][lindex $indexes $i]
	    global $var
	    append command_string " " [subst $$var]
	}	    
    }
    
    eval $command_string
}

proc CreateScaleVector { parent name text object field method tickints orient indexes } {
    set root $parent.$name
    set varBase "[subst $object]_[subst $field]"
    set varMin "[subst $varBase]_Min"
    set varMax "[subst $varBase]_Max"
    set varResolution "[subst $varBase]_Resolution"

    global $varMin $varMax $varResolution

    set tickinterval [expr ([subst $$varMax] - [subst $$varMin]) / ($tickints)]

    set size [llength $indexes]

    for {set i 0} {$i < $size} {incr i} {
	set index [lindex $indexes $i]
	set var "[subst $varBase][subst $index]"
	set rootCurr "[subst $root][subst $index]" 
	set textCurr "[subst $text][subst $index]"

	scale $rootCurr -label $textCurr -from [subst $$varMin] -to [subst $$varMax] \
	    -resolution [subst $$varResolution] -tickinterval $tickinterval \
	    -orient $orient -var $var \
	    -command "SetScaleVectorHelper $object $field $method {$indexes} $i"
    }
}

proc PopupVectorDialog { name text object field method tickints orient indexes } {
    set root .$name

    toplevel $root

    frame $root.scales -relief raised -bd 2
    
    CreateScaleVector $root.scales $name "" $object $field $method $tickints $orient \
	    $indexes

    set size [llength $indexes]

    for {set i 0} {$i < $size} {incr i} {
	pack $root.scales.[subst $name][lindex $indexes $i] -side top -fill x
    }

    pack $root.scales -side top -fill x -expand 1
}

proc CreateVectorDialog { parent name text object field method tickints orient indexes } {
    set root $parent.$name

    button $root -text $text -command "PopupVectorDialog $name $text $object $field \
	    $method $tickints $orient {$indexes}"
}

proc SetScaleColorHelper { object field method indexes index sample value } {

    set varBase "[subst $object]_[subst $field]"
    set size [llength $indexes]
    set command_string "$object $method"

    for {set i 0} {$i < $size} {incr i} {
#	if { $i == $index } {
#	    append command_string " " $value 
#	} else {
	    set var [subst $varBase][lindex $indexes $i]
	    global $var
	    append command_string " " [subst $$var]
#	}	    
    }

    eval $command_string

    # Update the color of the sample
    set varR [subst $varBase][lindex $indexes 0]
    set varG [subst $varBase][lindex $indexes 1]
    set varB [subst $varBase][lindex $indexes 2]

    global $varR $varG $varB
    set byteR [ScaleToByte [subst $$varR]]
    set byteG [ScaleToByte [subst $$varG]]
    set byteB [ScaleToByte [subst $$varB]]

    $sample conf -bg [format \#%02x%02x%02x $byteR $byteG $byteB]
}

proc CreateScaleColor { parent name text object field method tickints orient indexes } {
    set root $parent.$name
    set varBase "[subst $object]_[subst $field]"
    set varMin "[subst $varBase]_Min"
    set varMax "[subst $varBase]_Max"
    set varResolution "[subst $varBase]_Resolution"
    set sample [subst $root]Sample

    global $varMin $varMax $varResolution

    set tickinterval [expr ([subst $$varMax] - [subst $$varMin]) / ($tickints)]

    frame $sample -width 200 -height 100 -bg [format \#%02x%02x%02x 0 0 0]

    set size [llength $indexes]

    for {set i 0} {$i < $size} {incr i} {
	set index [lindex $indexes $i]
	set var "[subst $varBase][subst $index]"
	set rootCurr "[subst $root][subst $index]" 
	set textCurr "[subst $text][subst $index]"

	scale $rootCurr -label $textCurr -from [subst $$varMin] -to [subst $$varMax] \
	    -resolution [subst $$varResolution] -tickinterval $tickinterval \
	    -orient $orient -var $var \
	    -command "SetScaleColorHelper $object $field $method {$indexes} $i $sample"
    }
}

proc PopupColorDialog { name text object field method tickints orient indexes } {
    set root .$name

    toplevel $root

    frame $root.scales -relief raised -bd 2
    
    CreateScaleColor $root.scales $name "" $object $field $method $tickints $orient \
	    $indexes

    set size [llength $indexes]

    for {set i 0} {$i < $size} {incr i} {
	pack $root.scales.[subst $name][lindex $indexes $i] -side top -fill x
    }

    pack $root.scales.[subst $name]Sample -side top -fill x

    pack $root.scales -side top -fill x -expand 1
}

proc CreateColorDialog { parent name text object field method tickints orient indexes } {
    set root $parent.$name

    button $root -text $text -command "PopupColorDialog $name $text $object $field \
	    $method $tickints $orient {$indexes}"
}

proc CreateMenuButton { parent name text } {
    set root $parent.$name

    menubutton $root -text $text -underline 0 -menu $root.menu
}

proc CreateMenu { parent name listMenuItems} {
    set root $parent.$name

    menu $root

    foreach menuItem $listMenuItems {
	set type [eval {lindex $menuItem 0}]

	if { $type == "command" } {
	    set text [eval {lindex $menuItem 1}]
	    set command [eval {lindex $menuItem 2}]

	    $root add command -label $text -command $command
	} elseif { $type == "checkbutton" } {
	    set text [eval {lindex $menuItem 1}]
	    set object [eval {lindex $menuItem 2}]
	    set field [eval {lindex $menuItem 3}]
	    set method [eval {lindex $menuItem 4}]

	    set var "[subst $object]_[subst $field]"

	    $root add checkbutton -label $text -variable $var \
		    -command "$object $method \$$var"
	} elseif { $type == "radiobutton" } {
	    set text [eval {lindex $menuItem 1}]
	    set value [eval {lindex $menuItem 2}]
	    set object [eval {lindex $menuItem 3}]
	    set field [eval {lindex $menuItem 4}]
	    set method [eval {lindex $menuItem 5}]

	    set var "[subst $object]_[subst $field]"

	    $root add radiobutton -label $text -value $value -variable $var \
		    -command "$object $method \$$var"
	} elseif { $type == "cascade" } {
	    set name [eval {lindex $menuItem 1}]
	    set text [eval {lindex $menuItem 2}]
	    set subListMenuItems [eval {lindex $menuItem 3}]

	    $root add cascade -label $text -menu $root.$name
	    CreateMenu $root $name $subListMenuItems
	}
    }
}

proc CreateMenuBar { parent name listMenus } {
    set root $parent.$name

    frame $root -relief raised -bd 2

    foreach submenu $listMenus {
	set subname [eval {lindex $submenu 0}]
	set text [eval {lindex $submenu 1}]
	set side [eval {lindex $submenu 2}]
	set listMenuItems [eval {lindex $submenu 3}]

	CreateMenuButton $root $subname $text
        CreateMenu $root.$subname menu $listMenuItems

	pack $root.$subname -side $side
    }
}

proc CreateToolBar { parent name list } {
    set root $parent.$name
    set i 0

    frame $root -relief raised -bd 2
    foreach tool $list {
	set type [eval {lindex $tool 0}]

	if { $type == "button" } {
	    set text [eval {lindex $tool 1}]
	    set object [eval {lindex $tool 2}]
	    set method [eval {lindex $tool 3}]

	    button $root.button$i -text $text -command "$object $method"

	    pack $root.button$i -side left -padx 2 -pady 2 -expand 1
	} elseif { $type == "text" } {
	    set initValue [eval {lindex $tool 1}]

	    label $root.text$i -text $initValue

	    pack $root.text$i -side left -padx 2 -pady 2 -expand 1
	} elseif { $type == "textOut" } {
	    set object [eval {lindex $tool 1}]
	    set field [eval {lindex $tool 2}]
	    #set initValue [eval {lindex $tool 3}]
	    set varText "[subst $object]_[subst $field]"

	    label $root.textOut$i -width 10 -anchor w -text "Jordan" -textvariable $varText -relief sunken
#	    label $root.textOut$i -width 10 -anchor w -text [set $varText] -textvariable $varText -relief sunken

	    pack $root.textOut$i -side left -padx 2 -pady 2 -expand 1
	}
	incr i
    }
}



proc CreateWindow { parent name } {
    if { $name == {} } {
	return false
    }

    if { $parent == {} } {
	set root .$name
    } elseif { $parent == "." } {
	set root .$name
    } else {
	set root $parent.$name
    }

    set varWindow $name

#    toplevel $root

    CreateMenuBar $root menubar \
	    [list \
	      [list file File left \
	        [list \
	          [list command "w: Write" "$varWindow handleWrite"] \
	          [list command "esc: Exit" slideExit] \
                ] \
	      ] \
	      [list options Options left \
	        [list \
	          [list checkbutton "n: Draw Normals" $varWindow drawNormals setDrawNormals] \
	          [list checkbutton "b: Draw Bounds" $varWindow drawBounds setDrawBounds] \
	          [list checkbutton "d: Color Dyn Bounds" $varWindow colorDynamicBounds setColorDynamicBounds] \
	          [list checkbutton "a: Color Altered Bounds" $varWindow colorChangedBounds setColorChangedBounds] \
	          [list checkbutton "h: Check Bound Altered" $varWindow checkBoundChanged setCheckBoundChanged] \
	          [list checkbutton "f: Draw Clip Bound" $varWindow drawClipBound setDrawClipBound] \
	          [list checkbutton "v: Draw Viewport Bound" $varWindow drawViewportBound setDrawViewportBound] \
	          [list checkbutton "c: Clip" $varWindow clipEnabled setClipEnabled] \
	          [list checkbutton "x: Back Face Cull" $varWindow backFaceCullingEnabled setBackFaceCullingEnabled] \
	          [list checkbutton "z: OpenGL Z-Buffer" $varWindow zbufferEnabled setZBufferEnabled] \
	          [list checkbutton "r: Bound Cull" $varWindow boundCullingEnabled setBoundCullingEnabled] \
	          [list checkbutton "y: Scan Convert" $varWindow scanConvertEnabled setScanConvertEnabled] \
	          [list checkbutton "i: Use Winding Number Inside Test" $varWindow windingEnabled setWindingEnabled] \
	          [list checkbutton "u: Crystal Ball Enabled" $varWindow crystalBallEnabled setCrystalBallEnabled] \
	          [list checkbutton "q: Crazy Clipped Rotate" $varWindow rotateClippedEnabled setRotateClippedEnabled] \
	          [list command "s: Start Dynamics" "$varWindow handleStartDynamics"] \
	          [list command "' ': Pause Dynamics" "$varWindow handlePauseDynamics"] \
		] \
	      ] \
	      [list help Help right \
	        [list \
	          [list command "slidescan" ""] \
                ] \
	      ] \
	    ]

    CreateToolBar $root readout1 \
	    [list \
	      [list text "Frames Per Second:  "] \
	      [list text "Requested = "] \
	      [list textOut $varWindow framesPerSecondRequested] \
	    ]
    CreateToolBar $root readout2 \
	    [list \
	      [list text "Update Time = "] \
	      [list textOut $varWindow updateTime] \
	      [list text "Render Time = "] \
	      [list textOut $varWindow renderTime] \
	    ]

    pack $root.menubar $root.readout1 $root.readout2 -side top -fill x

    ##################################################
    # Togl Widget
    #
    togl $root.toglarea -width 400 -height 400 -rgba true -double true -depth true -stencil true -ident %name

    bind $root.toglarea <KeyPress> "$varWindow handleKeyboard %K 0 %x %y"
    #bind $root.toglarea <KeyRelease> "$varWindow handleKeyboard %K 1 %x %y"
    bind $root.toglarea <ButtonPress> "$varWindow handleMouseButton %b 0 %x %y"
    bind $root.toglarea <ButtonRelease> "$varWindow handleMouseButton %b 1 %x %y"
    bind $root.toglarea <B1-Motion> "$varWindow handleMouseMotion 1 %x %y"
    bind $root.toglarea <B2-Motion> "$varWindow handleMouseMotion 2 %x %y"
    bind $root.toglarea <B3-Motion> "$varWindow handleMouseMotion 3 %x %y"

    wm minsize $root 1 1
    wm maxsize $root 640 480
    #wm maxsize $root 1000 1000
    wm resizable $root 1 1

    focus $root.toglarea

    pack $root.toglarea -side left -fill both -expand 1
    #pack $root -fill both -expand 1

    return true
}

proc DestroyWindow { parent name } {
    if { $name == {} } {
	return false
    }

    if { $parent == {} } {
	set root .$name
    } elseif { $parent == "." } {
	set root .$name
    } else {
	set root $parent.$name
    }

    set varWindow $name

    destroy $root
}


