######################################################################
# SLIDE Polyhedral Geometry Generators
#

##################################################
# CreateSLIDENonuniform
#   Parameters
#     pcName     - A unique name that will be used as part of the 
#                  name of each SLIDE primitive which is created 
#     uiSides    - Number of sides of the regular polygonal base
#     fHeight    - Height of the pyramid
#
#   Returns the name of the object "o$pcName"
#
proc CreateSLIDEPyramid { pcName fHeight ppfPoints } {
    set lPoints {}
    for { set i 0 } { $i < [llength $ppfPoints] } { incr i } {
	slide create point p[subst $pcName]_[expr $i] \
		[list [lindex [lindex $ppfPoints $i] 0] \
		      [lindex [lindex $ppfPoints $i] 1] \
		      0 ]
	set lPoints [concat p[subst $pcName]_[expr $i] $lPoints]
    }
    slide create point p[subst $pcName] [list 0 0 $fHeight]

    set lFaces {}

    slide create face f[subst $pcName] $lPoints
    lappend lFaces f[subst $pcName]

    set j [expr [llength $ppfPoints] - 1]
    for { set i 0 } { $i < [llength $ppfPoints] } { incr i } {
	slide create face f[subst $pcName]_[expr $i] \
		[list p[subst $pcName] \
                      p[subst $pcName]_[expr $j] \
                      p[subst $pcName]_[expr $i] ]

        lappend lFaces f[subst $pcName]_[expr $i]
	set j $i
    }

    slide create object o[subst $pcName] $lFaces    
}

##################################################
# CreateSLIDENonuniform
#   Parameters
#     pcName     - A unique name that will be used as part of the 
#                  name of each SLIDE primitive which is created 
#     uiSides    - Number of sides of the regular polygonal base
#     fHeight    - Height of the pyramid
#
#   Returns the name of the object "o$pcName"
#
proc CreateSLIDENonuniformGrid { pcName uiUSlices uiVSlices } {
  set fXDelta [expr 2.0 / $uiUSlices]
  set fYDelta [expr 2.0 / $uiVSlices]

  set fX -1.0
  for { set uiU 0 } { $uiU <= $uiUSlices } { incr uiU } {
    set fY -1.0
    for { set uiV 0 } { $uiV <= $uiVSlices } { incr uiV } {
	slide create point p[subst $pcName]_[expr $uiU]_[expr $uiV] \
		[list [expr $fX] \
		      [expr $fY] \
		      0 ]
#		      [lindex [lindex $ppfHeight $uiV] $uiU] ]

      set fY [expr $fY + $fYDelta]
    }
    set fX [expr $fX + $fXDelta]
  }

  set lFaces {}
  for { set uiU 0 } { $uiU < $uiUSlices } { incr uiU } {
    for { set uiV 0 } { $uiV < $uiVSlices } { incr uiV } {
	slide create face f[subst $pcName]_[expr $uiU]_[expr $uiV] \
		[list p[subst $pcName]_[expr $uiU]_[expr $uiV] \
                      p[subst $pcName]_[expr $uiU + 1]_[expr $uiV] \
                      p[subst $pcName]_[expr $uiU + 1]_[expr $uiV + 1] \
                      p[subst $pcName]_[expr $uiU]_[expr $uiV + 1] ]

        lappend lFaces f[subst $pcName]_[expr $uiU]_[expr $uiV]
    }
  }

  slide create object o[subst $pcName] $lFaces
}

##################################################
# CreateSLIDEGrid
#   Parameters
#     pcName     - A unique name that will be used as part of the 
#                  name of each SLIDE primitive which is created 
#     uiSides    - Number of sides of the regular polygonal base
#     fHeight    - Height of the pyramid
#
#   Returns the name of the object "o$pcName"
#
proc CreateSLIDEGrid { pcName uiUSlices uiVSlices {fWidth 2.0} {fHeight 2.0} } {
  set fXDelta [expr $fWidth / $uiUSlices]
  set fYDelta [expr $fHeight / $uiVSlices]

  set fX [expr -$fWidth/2.0]
  for { set uiU 0 } { $uiU <= $uiUSlices } { incr uiU } {
    set fY [expr -$fHeight/2.0]
    for { set uiV 0 } { $uiV <= $uiVSlices } { incr uiV } {
	slide create point p[subst $pcName]_[expr $uiU]_[expr $uiV] \
		[list [expr $fX] \
		      [expr $fY] \
		      0 ]
#		      [lindex [lindex $ppfHeight $uiV] $uiU] ]

      set fY [expr $fY + $fYDelta]
    }
    set fX [expr $fX + $fXDelta]
  }

  set lFaces {}
  for { set uiU 0 } { $uiU < $uiUSlices } { incr uiU } {
    for { set uiV 0 } { $uiV < $uiVSlices } { incr uiV } {
	slide create face f[subst $pcName]_[expr $uiU]_[expr $uiV] \
		[list p[subst $pcName]_[expr $uiU]_[expr $uiV] \
                      p[subst $pcName]_[expr $uiU + 1]_[expr $uiV] \
                      p[subst $pcName]_[expr $uiU + 1]_[expr $uiV + 1] \
                      p[subst $pcName]_[expr $uiU]_[expr $uiV + 1] ]

        lappend lFaces f[subst $pcName]_[expr $uiU]_[expr $uiV]
    }
  }

  slide create object o[subst $pcName] $lFaces

  return o[subst $pcName]
}

##################################################
# CreateSLIDERotationalSweep
#   Parameters
#     pcName     - A unique name that will be used as part of the 
#                  name of each SLIDE primitive which is created 
#     uiSides    - Number of sides of the regular polygonal base
#     fHeight    - Height of the pyramid
#
#   Returns the name of the object "o$pcName"
#
proc CreateSLIDERotationalSweep { pcName pcInstEdge uiSides } {
    slide create group g[subst $pcName]

    set fTheta 0.0
    set fThetaDelta [expr 360.0 / $uiSides]
    for { set i 0 } { $i < $uiSides } { incr i } {
	slide create instance i[subst $pcName][subst $i] g[subst $pcName] $pcInstEdge \
		-xforms [list [list rotate 0 0 1 $fTheta]]

	set fTheta [expr $fTheta + $fThetaDelta]
    }

    return g[subst $pcName]
}

##################################################
# CreateSLIDECubeFrameEdge
#   Parameters
#     pcName     - A unique name that will be used as part of the 
#                  name of each SLIDE primitive which is created 
#     fPercent   - Percentage size of the interior cube
#
#   Returns the name of the object "o$pcName"
#
proc CreateSLIDECubeFrameEdge { pcName fPercent } {
    set fEdge 1.0
    set fRadius [expr $fEdge/sqrt(2.0)]

    set fEdgeInner [expr $fEdge*$fPercent]
    set fRadiusInner [expr $fEdgeInner/sqrt(2.0)]

    set fHeight [expr ($fEdge - $fEdgeInner)/2.0]

    slide create point p[subst $pcName]_0_0 \
	    [list $fRadius 0 0]
    slide create point p[subst $pcName]_0_1 \
	    [list 0 $fRadius 0]

    slide create point p[subst $pcName]_1_0 \
	    [list $fRadiusInner 0 0]
    slide create point p[subst $pcName]_1_1 \
	    [list 0 $fRadiusInner 0]

    slide create point p[subst $pcName]_2_0 \
	    [list $fRadiusInner 0 -$fHeight]
    slide create point p[subst $pcName]_2_1 \
	    [list 0 $fRadiusInner -$fHeight]

    slide create face f[subst $pcName]_0 [list p[subst $pcName]_0_0 p[subst $pcName]_0_1 p[subst $pcName]_1_1 p[subst $pcName]_1_0]
    slide create face f[subst $pcName]_1 [list p[subst $pcName]_1_0 p[subst $pcName]_1_1 p[subst $pcName]_2_1 p[subst $pcName]_2_0]

    slide create object o[subst $pcName] [list f[subst $pcName]_0 f[subst $pcName]_1]

    return o[subst $pcName]
}

##################################################
# CreateSLIDECubeFrameEdgeAndCorner
#   Parameters
#     pcName     - A unique name that will be used as part of the 
#                  name of each SLIDE primitive which is created 
#     fPercent   - Percentage size of the interior cube
#
#   Returns the name of the object "o$pcName"
#
proc CreateSLIDECubeFrameEdgeAndCorner { pcName fPercent } {
    set fEdge 1.0
    set fRadius [expr $fEdge/sqrt(2.0)]

    set fEdgeInner [expr $fEdge*$fPercent]
    set fRadiusInner [expr $fEdgeInner/sqrt(2.0)]

    set fHeight [expr ($fEdge - $fEdgeInner)/2.0]

    set fRadiusMajor [expr ($fRadius + $fRadiusInner)/2.0]
    set fRadiusMinor [expr ($fRadius - $fRadiusInner)/2.0]

    slide create point p[subst $pcName]_0_0 \
	    [list [expr $fRadiusMajor] [expr $fRadiusMinor] 0]
    slide create point p[subst $pcName]_0_1 \
	    [list [expr $fRadiusMinor] [expr $fRadiusMajor] 0]

    slide create point p[subst $pcName]_1_0 \
	    [list [expr $fRadiusMajor - $fRadiusMinor] 0 0]
    slide create point p[subst $pcName]_1_1 \
	    [list 0 [expr $fRadiusMajor - $fRadiusMinor] 0]

    slide create point p[subst $pcName]_2_0 \
	    [list [expr $fRadiusMajor - $fRadiusMinor] 0 [expr -$fHeight]]
    slide create point p[subst $pcName]_2_1 \
	    [list 0 [expr $fRadiusMajor - $fRadiusMinor] [expr -$fHeight]]

    slide create point p[subst $pcName]_3_0 \
	    [list [expr $fRadiusMajor] [expr -$fRadiusMinor] 0]
    slide create point p[subst $pcName]_3_1 \
	    [list [expr $fRadiusMajor + $fRadiusMinor] 0 0]

    slide create face f[subst $pcName]_0 [list p[subst $pcName]_0_0 p[subst $pcName]_0_1 p[subst $pcName]_1_1 p[subst $pcName]_1_0]
    slide create face f[subst $pcName]_1 [list p[subst $pcName]_1_0 p[subst $pcName]_1_1 p[subst $pcName]_2_1 p[subst $pcName]_2_0]
    slide create face f[subst $pcName]_2 [list p[subst $pcName]_0_0 p[subst $pcName]_1_0 p[subst $pcName]_3_0 p[subst $pcName]_3_1]

    slide create object o[subst $pcName] [list f[subst $pcName]_0 f[subst $pcName]_1 f[subst $pcName]_2]

    return o[subst $pcName]
}

##################################################
# CreateSLIDEOctahedronFrameEdge
#   Parameters
#     pcName     - A unique name that will be used as part of the 
#                  name of each SLIDE primitive which is created 
#     fPercent   - Percentage size of the interior octahedron
#
#   Returns the name of the object "o$pcName"
#
proc CreateSLIDEOctahedronFrameEdge { pcName fPercent } {
    global SLF_PI
    set fEdge 1.0
    set fRadius [expr $fEdge/sqrt(3.0)]
    set fHeight [expr $fEdge/sqrt(6.0)]

    slide create point p[subst $pcName]_0_0 \
	    [list [expr $fRadius] 0 0]
    slide create point p[subst $pcName]_0_1 \
	    [list [expr -$fRadius*0.5] [expr $fRadius*sqrt(3.0)/2.0] 0]

    #set fR [expr $fEdge*((1.0 + 3.0*$fPercent)/(4.0*sqrt(3.0)))]
    set fR [expr $fEdge*$fPercent/sqrt(3.0)]

    slide create point p[subst $pcName]_1_0 \
	    [list [expr $fR] 0 0]
    slide create point p[subst $pcName]_1_1 \
	    [list [expr -$fR*0.5] [expr $fR*sqrt(3.0)/2.0] 0]

    set fR [expr $fEdge*$fPercent/sqrt(3.0)]
    set fZ [expr -$fEdge*(1.0 - $fPercent)/sqrt(6.0)]

    slide create point p[subst $pcName]_2_0 \
	    [list [expr $fR] 0 [expr $fZ]]
    slide create point p[subst $pcName]_2_1 \
	    [list [expr -$fR*0.5] [expr $fR*sqrt(3.0)/2.0] [expr $fZ]]

    slide create face f[subst $pcName]_0 [list p[subst $pcName]_0_0 p[subst $pcName]_0_1 p[subst $pcName]_1_1 p[subst $pcName]_1_0]
    slide create face f[subst $pcName]_1 [list p[subst $pcName]_1_0 p[subst $pcName]_1_1 p[subst $pcName]_2_1 p[subst $pcName]_2_0]

    slide create object o[subst $pcName] [list f[subst $pcName]_0 f[subst $pcName]_1]

    return o[subst $pcName]
}

##################################################
# CreateSLIDECubeGrid
#   Parameters
#     pcName     - A unique name that will be used as part of the 
#                  name of each SLIDE primitive which is created 
#     fPercent   - Percentage size of the interior cube
#
#   Returns the name of the object "o$pcName"
#
proc CreateSLIDECubeGrid { pcName uiSlices } {
    set pcFaceName [CreateSLIDEGrid [subst $pcName]_Face $uiSlices $uiSlices 1.0 1.0]

    return [CreateSLIDECubeSymmetry $pcName $pcFaceName 0]
}

##################################################
# CreateSLIDECubeFrame
#   Parameters
#     pcName     - A unique name that will be used as part of the 
#                  name of each SLIDE primitive which is created 
#     fPercent   - Percentage size of the interior cube
#
#   Returns the name of the object "o$pcName"
#
proc CreateSLIDECubeFrame { pcName fPercent } {
    set pcEdgeName [CreateSLIDECubeFrameEdge [subst $pcName]_Edge $fPercent]
    set pcFaceName [CreateSLIDERotationalSweep [subst $pcName]_Face $pcEdgeName 4]

    return [CreateSLIDECubeSymmetry $pcName $pcFaceName]
}

##################################################
# CreateSLIDECubeFrameWithCorners
#   Parameters
#     pcName     - A unique name that will be used as part of the 
#                  name of each SLIDE primitive which is created 
#     fPercent   - Percentage size of the interior cube
#
#   Returns the name of the object "o$pcName"
#
proc CreateSLIDECubeFrameWithCorners { pcName fPercent } {
    set pcEdgeName [CreateSLIDECubeFrameEdgeAndCorner [subst $pcName]_Edge $fPercent]
    set pcFaceName [CreateSLIDERotationalSweep [subst $pcName]_Face $pcEdgeName 4]

    return [CreateSLIDECubeSymmetry $pcName $pcFaceName]
}

##################################################
# CreateSLIDEOctahedronFrame
#   Parameters
#     pcName     - A unique name that will be used as part of the 
#                  name of each SLIDE primitive which is created 
#     fPercent   - Percentage size of the interior cube
#
#   Returns the name of the object "o$pcName"
#
proc CreateSLIDEOctahedronFrame { pcName fPercent } {
    set pcEdgeName [CreateSLIDEOctahedronFrameEdge [subst $pcName]_Edge $fPercent]
    set pcFaceName [CreateSLIDERotationalSweep [subst $pcName]_Face $pcEdgeName 3]

    return [CreateSLIDEOctahedronSymmetry $pcName $pcFaceName]
}

##################################################
# CreateSLIDETetrahedralFrameFace
#   Parameters
#     pcName     - A unique name that will be used as part of the 
#                  name of each SLIDE primitive which is created 
#     uiSides    - Number of sides of the regular polygon
#
#   Returns the name of the object "o$pcName"
#
proc CreateSLIDETetrahedralFrameFace { pcName fThickness } {
    global SLF_PI

    set uiSides 3
    set fEdge 1.0
    set fAlpha [expr asin(1.0/sqrt(3.0))]
    set fPhi [expr 2.0*$fAlpha]

    set fRadius [expr $fEdge/(2.0*cos($fAlpha))]
    set fHeight [expr $fEdge/(sqrt(3.0)*tan($fPhi))]

    set fTriRadius [expr $fEdge/sqrt(3.0)]

    set fTheta 0.0
    set fThetaDelta [expr 2.0*$SLF_PI/3.0]
    set i 0
    for { set i 0 } { $i < $uiSides } { incr i } {
	slide create point p[subst $pcName]_0_$i \
		[list [expr $fTriRadius*cos($fTheta)] \
		      [expr $fTriRadius*sin($fTheta)] \
		      0 ]

	set fTheta [expr $fTheta + $fThetaDelta]
    }

    set fTriRadius [expr $fTriRadius*$fThickness]

    set fTheta 0.0
    set fThetaDelta [expr 2.0*$SLF_PI/3.0]
    set i 0
    for { set i 0 } { $i < $uiSides } { incr i } {
	slide create point p[subst $pcName]_1_$i \
		[list [expr $fTriRadius*cos($fTheta)] \
		      [expr $fTriRadius*sin($fTheta)] \
		      0 ]

	set fTheta [expr $fTheta + $fThetaDelta]
    }

    set fRadius [expr $fRadius*$fThickness]

    set fTheta 0.0
    set fThetaDelta [expr 2.0*$SLF_PI/3.0]
    set i 0
    for { set i 0 } { $i < $uiSides } { incr i } {
	slide create point p[subst $pcName]_2_$i \
		[list [expr $fRadius*sin($fPhi)*cos($fTheta)] \
		      [expr $fRadius*sin($fPhi)*sin($fTheta)] \
		      [expr $fRadius*cos($fPhi) - $fHeight] ]

	set fTheta [expr $fTheta + $fThetaDelta]
    }

    slide create face f[subst $pcName]_0_0 [list p[subst $pcName]_0_0 p[subst $pcName]_0_1 p[subst $pcName]_1_1 p[subst $pcName]_1_0]
    slide create face f[subst $pcName]_0_1 [list p[subst $pcName]_0_1 p[subst $pcName]_0_2 p[subst $pcName]_1_2 p[subst $pcName]_1_1]
    slide create face f[subst $pcName]_0_2 [list p[subst $pcName]_0_2 p[subst $pcName]_0_0 p[subst $pcName]_1_0 p[subst $pcName]_1_2]

    slide create face f[subst $pcName]_1_0 [list p[subst $pcName]_1_0 p[subst $pcName]_1_1 p[subst $pcName]_2_1 p[subst $pcName]_2_0]
    slide create face f[subst $pcName]_1_1 [list p[subst $pcName]_1_1 p[subst $pcName]_1_2 p[subst $pcName]_2_2 p[subst $pcName]_2_1]
    slide create face f[subst $pcName]_1_2 [list p[subst $pcName]_1_2 p[subst $pcName]_1_0 p[subst $pcName]_2_0 p[subst $pcName]_2_2]

    slide create object o[subst $pcName] [list f[subst $pcName]_0_0 f[subst $pcName]_0_1 f[subst $pcName]_0_2 f[subst $pcName]_1_0 f[subst $pcName]_1_1 f[subst $pcName]_1_2]

    return o[subst $pcName]
}

##################################################
# CreateSLIDETetrahedralFrame
#   Parameters
#     pcName     - A unique name that will be used as part of the 
#                  name of each SLIDE primitive which is created 
#     uiSides    - Number of sides of the regular polygon
#
#   Returns the name of the object "o$pcName"
#
proc CreateSLIDETetrahedralFrame { pcName fThickness } {
    set pcObjectName [CreateSLIDETetrahedralFrameFace $pcName $fThickness]

    return [CreateSLIDETetrahedronSymmetry $pcName $pcObjectName]
}

##################################################
# CreateSLIDERegularPolygon
#   Parameters
#     pcName     - A unique name that will be used as part of the 
#                  name of each SLIDE primitive which is created 
#     uiSides    - Number of sides of the regular polygon
#
#   Returns the name of the object "o$pcName"
#
proc CreateSLIDERegularPolygon { pcName uiSides } {
    global SLF_PI

    set fEdge 1.0
    set fInteriorAngle [expr $SLF_PI * ( $uiSides - 2.0 ) / $uiSides]
    set fRadius [expr $fEdge / (2.0 * cos($fInteriorAngle/2.0))]

    set fTheta 0.0
    set fThetaDelta [expr 2.0 * $SLF_PI / $uiSides]
    set lPoints {}
    for { set i 0 } { $i < $uiSides } { incr i } {
	slide create point p[subst $pcName][subst $i] \
		[list [expr $fRadius * cos($fTheta)] [expr $fRadius * sin($fTheta)] 0]
	lappend lPoints p[subst $pcName][subst $i]
	set fTheta [expr $fTheta + $fThetaDelta]
    }

    slide create face f[subst $pcName] $lPoints
    slide create object o[subst $pcName] [list f[subst $pcName]]

    return o[subst $pcName]
}

##################################################
# CreateSLIDERegularPyramid
#   Parameters
#     pcName     - A unique name that will be used as part of the 
#                  name of each SLIDE primitive which is created 
#     uiSides    - Number of sides of the regular polygonal base
#     fHeight    - Height of the pyramid
#
#   Returns the name of the object "o$pcName"
#
proc CreateSLIDERegularPyramid { pcName uiSides fHeight} {
    global SLF_PI

    set fEdge 1.0
    set fInteriorAngle [expr $SLF_PI * ( $uiSides - 2.0 ) / $uiSides]
    set fRadius [expr $fEdge / (2.0 * cos($fInteriorAngle/2.0))]

    set fTheta 0.0
    set fThetaDelta [expr 2.0 * $SLF_PI / $uiSides]
    set i 0
    for { set i 0 } { $i < $uiSides } { incr i } {
	slide create point p[subst $pcName][subst $i] \
		[list [expr $fRadius * cos($fTheta)] [expr $fRadius * sin($fTheta)] 0]
	set fTheta [expr $fTheta + $fThetaDelta]
    }
    slide create point p[subst $pcName][subst $i] [list 0 0 $fHeight]
    
    set lFaces {}
    set j [expr $uiSides - 1]
    for { set i 0 } { $i < $uiSides } { incr i } {
	slide create face f[subst $pcName][subst $i] \
		[list p[subst $pcName][subst $uiSides] p[subst $pcName][subst $j] p[subst $pcName][subst $i]]
	lappend lFaces f[subst $pcName][subst $i]
	set j $i
    }

    slide create object o[subst $pcName] $lFaces

    return o[subst $pcName]
}

##################################################
# CreateSLIDEEquilateralPyramid
#   Parameters
#     pcName     - A unique name that will be used as part of the 
#                  name of each SLIDE primitive which is created 
#     uiSides    - Number of sides of the regular polygonal base
#     fHeight    - Height of the pyramid
#
#   Returns the name of the object "o$pcName"
#
proc CreateSLIDEEquilateralPyramid { pcName pcInstTriangle uiSides } {
    global SLF_PI

    set fEdge 1.0
    set fInteriorAngle [expr $SLF_PI * ( $uiSides - 2.0 ) / $uiSides]
    set fRadius [expr $fEdge / (2.0 * cos($fInteriorAngle/2.0))]

    set fTriRadius [expr $fEdge / sqrt(3.0)]
    set fTriProj [expr $fEdge / (2.0 * sqrt(3.0))]
    set fTriHeight [expr 3.0 * $fEdge / (2.0 * sqrt(3.0))]

    set fInteriorAngle [expr $SLF_PI * ( $uiSides - 2.0 ) / $uiSides]
    set fRadius [expr $fEdge / (2.0 * cos($fInteriorAngle/2.0))]
    set fProj [expr $fEdge * tan($fInteriorAngle/2.0) / 2.0]

    set fInclineAngle [expr acos($fProj/$fTriHeight)]
    set fInitialAngle 0.0
    if { !($uiSides & 1) } {
	set fInitialAngle [expr $SLF_PI / $uiSides]
    }

    slide create group g[subst $pcName]

    set fTheta 0.0
    set fThetaDelta [expr 2.0 * $SLF_PI / $uiSides]
    for { set i 0 } { $i < $uiSides } { incr i } {
	slide create instance i[subst $pcName][subst $i] g[subst $pcName] $pcInstTriangle \
		-xforms [list [list translate [expr $fTriProj] 0 0] \
		              [list rotate 0 1 0 [expr -$fInclineAngle*180.0/$SLF_PI]] \
		              [list translate [expr -$fProj] 0 0] \
		              [list rotate 0 0 1 [expr ($fInitialAngle + $fTheta)*180.0/$SLF_PI]] ]

	set fTheta [expr $fTheta + $fThetaDelta]
    }

    return g[subst $pcName]
}

##################################################
# CreateSLIDETetrahedronSymmetry
#   Parameters
#     pcName       - A unique name that will be used as part of the 
#                    name of each SLIDE primitive which is created 
#     pcInstTriangle - The name of the object to be instanced
#
#   Returns the name of the top level group "g$pcName"
#
proc CreateSLIDETetrahedronSymmetry { pcName pcInstTriangle } {
    global SLF_PI

    set fEdge 1.0

    set fInscribedRadius [expr $fEdge/(2.0*sqrt(6.0))]
    set fPhi [expr atan2(2.0*sqrt(2.0), 1.0)]
    set fPhi [expr -$fPhi * (180.0/$SLF_PI)]

    slide create group g[subst $pcName]

    set uiSides 3
    set fTheta 0.0
    set fThetaDelta [expr 360.0 / $uiSides]
    for { set i 0 } { $i < $uiSides } { incr i } {
	slide create instance i[subst $pcName][subst $i] g[subst $pcName] $pcInstTriangle \
		-xforms [list [list translate 0 0 [expr $fInscribedRadius]] \
		              [list rotate 0 1 0 $fPhi] \
		              [list rotate 0 0 1 $fTheta] ]

	set fTheta [expr $fTheta + $fThetaDelta]
    }

    slide create instance i[subst $pcName]3 g[subst $pcName] $pcInstTriangle \
	    -xforms [list [list translate 0 0 [expr $fInscribedRadius]] \
	                  [list rotate 1 0 0 180] ]

    return g[subst $pcName]
}

##################################################
# CreateSLIDECubeSymmetry
#   Parameters
#     pcName       - A unique name that will be used as part of the 
#                    name of each SLIDE primitive which is created 
#     pcInstSquare - The name of the object to be instanced
#
#   Returns the name of the top level group "g$pcName"
#
proc CreateSLIDECubeSymmetry { pcName pcInstSquare {bRotate45 1} } {
    set fEdge 1.0
    set fInscribedRadius [expr $fEdge / 2.0]
    set fRotate 0.0
    if { $bRotate45 } {
	set fRotate 45.0
    }

    slide create group g[subst $pcName]

    slide create instance i[subst $pcName]0 g[subst $pcName] $pcInstSquare \
	    -xforms [list [list translate 0 0 [expr $fInscribedRadius]] \
	                  [list rotate 0 0 1 $fRotate] ]
    slide create instance i[subst $pcName]1 g[subst $pcName] $pcInstSquare \
	    -xforms [list [list translate 0 0 [expr $fInscribedRadius]] \
	                  [list rotate 0 0 1 $fRotate] \
	                  [list rotate 1 0 0 90] ]
    slide create instance i[subst $pcName]2 g[subst $pcName] $pcInstSquare \
	    -xforms [list [list translate 0 0 [expr $fInscribedRadius]] \
	                  [list rotate 0 0 1 $fRotate] \
	                  [list rotate 0 1 0 90] ]
    slide create instance i[subst $pcName]3 g[subst $pcName] $pcInstSquare \
	    -xforms [list [list translate 0 0 [expr $fInscribedRadius]] \
	                  [list rotate 0 0 1 $fRotate] \
	                  [list rotate 1 0 0 -90] ]
    slide create instance i[subst $pcName]4 g[subst $pcName] $pcInstSquare \
	    -xforms [list [list translate 0 0 [expr $fInscribedRadius]] \
	                  [list rotate 0 0 1 $fRotate] \
	                  [list rotate 0 1 0 -90] ]
    slide create instance i[subst $pcName]5 g[subst $pcName] $pcInstSquare \
	    -xforms [list [list translate 0 0 [expr $fInscribedRadius]] \
	                  [list rotate 0 0 1 $fRotate] \
	                  [list rotate 1 0 0 180] ]

    return g[subst $pcName]
}

##################################################
# CreateSLIDEOctahedronSymmetry
#   Parameters
#     pcName       - A unique name that will be used as part of the 
#                    name of each SLIDE primitive which is created 
#     pcInstTriangle - The name of the object to be instanced
#
#   Returns the name of the top level group "g$pcName"
#
proc CreateSLIDEOctahedronSymmetry { pcName pcInstTriangle } {
    global SLF_PI

    set fEdge 1.0
    set fInscribedRadius [expr $fEdge / sqrt(6.0)]
    set fPhi [expr asin(sqrt(2.0/3.0))]
    set fPhi [expr -$fPhi*(180.0/$SLF_PI)]

    slide create group g[subst $pcName]

    slide create instance i[subst $pcName]0 g[subst $pcName] $pcInstTriangle \
	    -xforms [list [list translate 0 0 $fInscribedRadius] \
	                  [list rotate 0 1 0 [expr $fPhi]] ]
    slide create instance i[subst $pcName]0 g[subst $pcName] $pcInstTriangle \
	    -xforms [list [list translate 0 0 $fInscribedRadius] \
	                  [list rotate 0 1 0 [expr $fPhi]] \
	                  [list rotate 0 0 1 90] ]
    slide create instance i[subst $pcName]0 g[subst $pcName] $pcInstTriangle \
	    -xforms [list [list translate 0 0 $fInscribedRadius] \
	                  [list rotate 0 1 0 [expr $fPhi]] \
	                  [list rotate 0 0 1 180] ]
    slide create instance i[subst $pcName]0 g[subst $pcName] $pcInstTriangle \
	    -xforms [list [list translate 0 0 $fInscribedRadius] \
	                  [list rotate 0 1 0 [expr $fPhi]] \
	                  [list rotate 0 0 1 -90] ]

    slide create instance i[subst $pcName]0 g[subst $pcName] $pcInstTriangle \
	    -xforms [list [list translate 0 0 $fInscribedRadius] \
	                  [list rotate 0 1 0 180] \
	                  [list rotate 0 1 0 [expr $fPhi]] ]
    slide create instance i[subst $pcName]0 g[subst $pcName] $pcInstTriangle \
	    -xforms [list [list translate 0 0 $fInscribedRadius] \
	                  [list rotate 0 1 0 180] \
	                  [list rotate 0 1 0 [expr $fPhi]] \
	                  [list rotate 0 0 1 90] ]
    slide create instance i[subst $pcName]0 g[subst $pcName] $pcInstTriangle \
	    -xforms [list [list translate 0 0 $fInscribedRadius] \
	                  [list rotate 0 1 0 180] \
	                  [list rotate 0 1 0 [expr $fPhi]] \
	                  [list rotate 0 0 1 180] ]
    slide create instance i[subst $pcName]0 g[subst $pcName] $pcInstTriangle \
	    -xforms [list [list translate 0 0 $fInscribedRadius] \
	                  [list rotate 0 1 0 180] \
	                  [list rotate 0 1 0 [expr $fPhi]] \
	                  [list rotate 0 0 1 -90] ]

    return g[subst $pcName]
}

##################################################
# CreateSLIDECubeOctahedronSymmetry
#   Parameters
#     pcName       - A unique name that will be used as part of the 
#                    name of each SLIDE primitive which is created 
#     pcInstTriangle - The name of the object to be instanced
#     pcInstTriangle - The name of the object to be instanced
#
#   Returns the name of the top level group "g$pcName"
#
proc CreateSLIDECubeOctahedronSymmetry { pcName pcInstTriangle pcInstSquare } {
    global SLF_PI

    set fEdge 1.0
    set fInscribedRadius [expr $fEdge * sqrt(2.0/3.0)]
    set fPhi [expr asin(sqrt(2.0/3.0))]
    set fPhi [expr $fPhi*(180.0/$SLF_PI)]

    set fTheta 45.0
    set fThetaDelta 90.0

    slide create group g[subst $pcName]

    for { set i 0 } { $i < 4 } { incr i } {
	slide create instance i[subst $pcName]0 g[subst $pcName] $pcInstTriangle \
		-xforms [list [list translate 0 0 $fInscribedRadius] \
		              [list rotate 0 1 0 [expr $fPhi]] \
		              [list rotate 0 0 1 [expr $fTheta]] ]
	set fTheta [expr $fTheta + $fThetaDelta]
    }
    for { } { $i < 8 } { incr i } {
	slide create instance i[subst $pcName]0 g[subst $pcName] $pcInstTriangle \
		-xforms [list [list translate 0 0 $fInscribedRadius] \
		              [list rotate 0 1 0 180] \
		              [list rotate 0 1 0 [expr $fPhi]] \
		              [list rotate 0 0 1 [expr $fTheta]] ]
	set fTheta [expr $fTheta + $fThetaDelta]
    }

    ##################################################
    # Cube

    set fInscribedRadius [expr $fEdge / sqrt(2.0)]

    slide create instance i[subst $pcName]9 g[subst $pcName] $pcInstSquare \
	    -xforms [list [list translate 0 0 [expr $fInscribedRadius]] ]
    slide create instance i[subst $pcName]10 g[subst $pcName] $pcInstSquare \
	    -xforms [list [list translate 0 0 [expr $fInscribedRadius]] \
	                  [list rotate 1 0 0 90] ]
    slide create instance i[subst $pcName]11 g[subst $pcName] $pcInstSquare \
	    -xforms [list [list translate 0 0 [expr $fInscribedRadius]] \
	                  [list rotate 0 1 0 90] ]
    slide create instance i[subst $pcName]12 g[subst $pcName] $pcInstSquare \
	    -xforms [list [list translate 0 0 [expr $fInscribedRadius]] \
	                  [list rotate 1 0 0 -90] ]
    slide create instance i[subst $pcName]13 g[subst $pcName] $pcInstSquare \
	    -xforms [list [list translate 0 0 [expr $fInscribedRadius]] \
	                  [list rotate 0 1 0 -90] ]
    slide create instance i[subst $pcName]14 g[subst $pcName] $pcInstSquare \
	    -xforms [list [list translate 0 0 [expr $fInscribedRadius]] \
	                  [list rotate 1 0 0 180] ]

    return g[subst $pcName]
}

##################################################
# CreateSLIDEDodecahedronSymmetry
#   Parameters
#     pcName       - A unique name that will be used as part of the 
#                    name of each SLIDE primitive which is created 
#     pcInstPentagon - The name of the object to be instanced
#
#   Returns the name of the top level group "g$pcName"
#
proc CreateSLIDEDodecahedronSymmetry { pcName pcInstPentagon } {
    global SLF_PI

    # 36 degrees = $SLF_PI/5.0 radians

    set uiSides 5
    set fEdge 1.0

    set fInteriorAngle [expr $SLF_PI * ( $uiSides - 2.0 ) / $uiSides]
    set fRadius [expr $fEdge / (2.0 * cos($fInteriorAngle/2.0))]
    set fProj [expr $fEdge * tan($fInteriorAngle/2.0) / 2.0]

    set fAlpha [expr acos(1.0/(2.0*sin($SLF_PI/5.0)))]
    set fBeta [expr atan(tan($fAlpha)/sin($SLF_PI/5.0))]
    set fGamma [expr $SLF_PI/2.0 - $fAlpha - $fBeta]

    set fCircumscribedRadius [expr $fEdge / (2.0*sin($SLF_PI/5.0)*sin($fBeta))]
    set fInscribedRadius [expr $fEdge / (2.0*tan($SLF_PI/5.0)*tan($fAlpha))]

    set fPhi [expr 2.0*$fAlpha*(180.0/$SLF_PI)]

    slide create group g[subst $pcName]

    set j 0

    slide create instance i[subst $pcName][subst $j] g[subst $pcName] $pcInstPentagon \
	    -xforms [list [list translate 0 0 $fInscribedRadius] ]
    incr j

    set fTheta 36.0
    set fThetaDelta 72.0
    for { set i 0 } { $i < $uiSides } { incr i } {
	slide create instance i[subst $pcName][subst $j] g[subst $pcName] $pcInstPentagon \
		-xforms [list [list translate 0 0 $fInscribedRadius] \
		              [list rotate 0 1 0 [expr $fPhi]] \
		              [list rotate 0 0 1 [expr $fTheta]] ]
	incr j
	set fTheta [expr $fTheta + $fThetaDelta]
    }

    set fTheta 36.0
    set fThetaDelta 72.0
    for { set i 0 } { $i < $uiSides } { incr i } {
	slide create instance i[subst $pcName][subst $j] g[subst $pcName] $pcInstPentagon \
		-xforms [list [list translate 0 0 $fInscribedRadius] \
		              [list rotate 0 1 0 180] \
		              [list rotate 0 1 0 [expr $fPhi]] \
		              [list rotate 0 0 1 [expr $fTheta]] ]
	incr j

	set fTheta [expr $fTheta + $fThetaDelta]
    }

    slide create instance i[subst $pcName][subst $j] g[subst $pcName] $pcInstPentagon \
	    -xforms [list [list translate 0 0 $fInscribedRadius] \
	                  [list rotate 0 1 0 180] ]
    incr j

    return g[subst $pcName]
}

##################################################
# CreateSLIDEIcosahedronSymmetry
#   Parameters
#     pcName       - A unique name that will be used as part of the 
#                    name of each SLIDE primitive which is created 
#     pcInstTriangle - The name of the object to be instanced
#
#   Returns the name of the top level group "g$pcName"
#
proc CreateSLIDEIcosahedronSymmetry { pcName pcInstTriangle } {
    global SLF_PI

    # 36 degrees = $SLF_PI/5.0 radians

    set uiSides 5
    set fEdge 1.0

    set fInteriorAngle [expr $SLF_PI * ( $uiSides - 2.0 ) / $uiSides]
    set fRadius [expr $fEdge / (2.0 * cos($fInteriorAngle/2.0))]
    set fProj [expr $fEdge * tan($fInteriorAngle/2.0) / 2.0]

    set fAlpha [expr acos(1.0/(2.0*sin($SLF_PI/5.0)))]
    set fBeta [expr asin(sin($fAlpha)*2.0/sqrt(3.0))]
    set fGamma [expr $SLF_PI/2.0 - $fAlpha - $fBeta]

    set fCircumscribedRadius [expr $fEdge / (2.0*sin($fAlpha))]
    set fInscribedRadius [expr $fEdge / (sqrt(3.0)*tan($fBeta))]

    set fPhi0 [expr $fBeta]
    set fPhi1 [expr $fPhi0 + 2.0*$fGamma]
    
    set fPhi0 [expr $fPhi0 * (180.0/$SLF_PI)]
    set fPhi1 [expr $fPhi1 * (180.0/$SLF_PI)]
    
    slide create group g[subst $pcName]

    set j 0

    set fTheta 0.0
    set fThetaDelta [expr 72.0]
    for { set i 0 } { $i < $uiSides } { incr i } {
	slide create instance i[subst $pcName][subst $j] g[subst $pcName] $pcInstTriangle \
		-xforms [list [list translate 0 0 [expr $fInscribedRadius]] \
		              [list rotate 0 1 0 [expr -$fPhi0]] \
		              [list rotate 0 0 1 [expr $fTheta]] ]
	incr j
	set fTheta [expr $fTheta + $fThetaDelta]
    }
    set fTheta -180.0
    set fThetaDelta [expr 72.0]
    for { set i 0 } { $i < $uiSides } { incr i } {
	slide create instance i[subst $pcName][subst $j] g[subst $pcName] $pcInstTriangle \
		-xforms [list [list translate 0 0 [expr $fInscribedRadius]] \
		              [list rotate 0 1 0 [expr $fPhi1]] \
		              [list rotate 0 0 1 [expr $fTheta]] ]
	incr j
	set fTheta [expr $fTheta + $fThetaDelta]
    }
    set fTheta 0.0
    set fThetaDelta [expr 72.0]
    for { set i 0 } { $i < $uiSides } { incr i } {
 	slide create instance i[subst $pcName][expr 2*$uiSides + $i] g[subst $pcName] $pcInstTriangle \
 		-xforms [list [list translate 0 0 [expr $fInscribedRadius]] \
 		              [list rotate 0 1 0 [expr 180.0 - $fPhi0]] \
 		              [list rotate 0 0 1 [expr $fTheta]] ]
	incr j
	set fTheta [expr $fTheta + $fThetaDelta]
    }
    set fTheta -180.0
    set fThetaDelta [expr 72.0]
    for { set i 0 } { $i < $uiSides } { incr i } {
 	slide create instance i[subst $pcName][subst $j] g[subst $pcName] $pcInstTriangle \
 		-xforms [list [list translate 0 0 [expr $fInscribedRadius]] \
 		              [list rotate 0 1 0 [expr 180.0 + $fPhi1]] \
 		              [list rotate 0 0 1 [expr $fTheta]] ]
	incr j
	set fTheta [expr $fTheta + $fThetaDelta]
    }

    return g[subst $pcName]
}

##################################################
# CreateSLIDEDodecahedronIcosahedronSymmetry
#   Parameters
#     pcName       - A unique name that will be used as part of the 
#                    name of each SLIDE primitive which is created 
#     pcInstTriangle - The name of the object to be instanced
#     pcInstPentagon - The name of the object to be instanced
#
#   Returns the name of the top level group "g$pcName"
#
proc CreateSLIDEDodecahedronIcosahedronSymmetry { pcName pcInstTriangle pcInstPentagon } {
    global SLF_PI

    set uiSides 5

    slide create group g[subst $pcName]

    ##################################################
    # Dodecahedron

    # 36 degrees = $SLF_PI/5.0 radians

    set fEdge 1.0
    set fEdge [expr $fEdge/cos($SLF_PI/5.0)]

    set fInteriorAngle [expr $SLF_PI * ( $uiSides - 2.0 ) / $uiSides]
    set fRadius [expr $fEdge / (2.0 * cos($fInteriorAngle/2.0))]
    set fProj [expr $fEdge * tan($fInteriorAngle/2.0) / 2.0]

    set fAlpha [expr acos(1.0/(2.0*sin($SLF_PI/5.0)))]
    set fBeta [expr atan(tan($fAlpha)/sin($SLF_PI/5.0))]
    set fGamma [expr $SLF_PI/2.0 - $fAlpha - $fBeta]

    set fCircumscribedRadius [expr $fEdge / (2.0*sin($SLF_PI/5.0)*sin($fBeta))]
    set fInscribedRadius [expr $fEdge / (2.0*tan($SLF_PI/5.0)*tan($fAlpha))]

    set fPhi [expr 2.0*$fAlpha*(180.0/$SLF_PI)]

    set j 0

    slide create instance i[subst $pcName][subst $j] g[subst $pcName] $pcInstPentagon \
	    -xforms [list [list translate 0 0 $fInscribedRadius] \
                          [list rotate 0 0 1 36.0] ]
    incr j

    set fTheta 36.0
    set fThetaDelta 72.0
    for { set i 0 } { $i < $uiSides } { incr i } {
	slide create instance i[subst $pcName][subst $j] g[subst $pcName] $pcInstPentagon \
		-xforms [list [list translate 0 0 $fInscribedRadius] \
		              [list rotate 0 0 1 36.0] \
		              [list rotate 0 1 0 [expr $fPhi]] \
		              [list rotate 0 0 1 [expr $fTheta]] ]
	incr j
	set fTheta [expr $fTheta + $fThetaDelta]
    }

    set fTheta 36.0
    set fThetaDelta 72.0
    for { set i 0 } { $i < $uiSides } { incr i } {
	slide create instance i[subst $pcName][subst $j] g[subst $pcName] $pcInstPentagon \
		-xforms [list [list translate 0 0 $fInscribedRadius] \
		              [list rotate 0 0 1 36.0] \
		              [list rotate 0 1 0 180] \
		              [list rotate 0 1 0 [expr $fPhi]] \
		              [list rotate 0 0 1 [expr $fTheta]] ]
	incr j

	set fTheta [expr $fTheta + $fThetaDelta]
    }

    slide create instance i[subst $pcName][subst $j] g[subst $pcName] $pcInstPentagon \
	    -xforms [list [list translate 0 0 $fInscribedRadius] \
	                  [list rotate 0 0 1 36.0] \
	                  [list rotate 0 1 0 180] ]
    incr j

    ##################################################
    # Icosahedron

    # 36 degrees = $SLF_PI/5.0 radians

    set fEdge 1.0
    set fEdge [expr 2.0*$fEdge]

    set fInteriorAngle [expr $SLF_PI * ( $uiSides - 2.0 ) / $uiSides]
    set fRadius [expr $fEdge / (2.0 * cos($fInteriorAngle/2.0))]
    set fProj [expr $fEdge * tan($fInteriorAngle/2.0) / 2.0]

    set fAlpha [expr acos(1.0/(2.0*sin($SLF_PI/5.0)))]
    set fBeta [expr asin(sin($fAlpha)*2.0/sqrt(3.0))]
    set fGamma [expr $SLF_PI/2.0 - $fAlpha - $fBeta]

    set fCircumscribedRadius [expr $fEdge / (2.0*sin($fAlpha))]
    set fInscribedRadius [expr $fEdge / (sqrt(3.0)*tan($fBeta))]

    set fPhi0 [expr $fBeta]
    set fPhi1 [expr $fPhi0 + 2.0*$fGamma]
    
    set fPhi0 [expr $fPhi0 * (180.0/$SLF_PI)]
    set fPhi1 [expr $fPhi1 * (180.0/$SLF_PI)]
    
    set fTheta [expr 36.0]
    set fThetaDelta [expr 72.0]
    for { set i 0 } { $i < $uiSides } { incr i } {
	slide create instance i[subst $pcName][subst $j] g[subst $pcName] $pcInstTriangle \
		-xforms [list [list translate 0 0 [expr $fInscribedRadius]] \
		              [list rotate 0 0 1 60.0] \
		              [list rotate 0 1 0 [expr -$fPhi0]] \
		              [list rotate 0 0 1 [expr $fTheta]] ]
	incr j
	set fTheta [expr $fTheta + $fThetaDelta]
    }
    set fTheta [expr 36.0 - 180.0]
    set fThetaDelta [expr 72.0]
    for { set i 0 } { $i < $uiSides } { incr i } {
	slide create instance i[subst $pcName][subst $j] g[subst $pcName] $pcInstTriangle \
		-xforms [list [list translate 0 0 [expr $fInscribedRadius]] \
		              [list rotate 0 0 1 60.0] \
		              [list rotate 0 1 0 [expr $fPhi1]] \
		              [list rotate 0 0 1 [expr $fTheta]] ]
	incr j
	set fTheta [expr $fTheta + $fThetaDelta]
    }
    set fTheta [expr 36.0]
    set fThetaDelta [expr 72.0]
    for { set i 0 } { $i < $uiSides } { incr i } {
 	slide create instance i[subst $pcName][subst $j] g[subst $pcName] $pcInstTriangle \
 		-xforms [list [list translate 0 0 [expr $fInscribedRadius]] \
		              [list rotate 0 0 1 60.0] \
 		              [list rotate 0 1 0 [expr 180.0 - $fPhi0]] \
 		              [list rotate 0 0 1 [expr $fTheta]] ]
	incr j
	set fTheta [expr $fTheta + $fThetaDelta]
    }
    set fTheta [expr 36.0 - 180.0]
    set fThetaDelta [expr 72.0]
    for { set i 0 } { $i < $uiSides } { incr i } {
 	slide create instance i[subst $pcName][subst $j] g[subst $pcName] $pcInstTriangle \
 		-xforms [list [list translate 0 0 [expr $fInscribedRadius]] \
		              [list rotate 0 0 1 60.0] \
 		              [list rotate 0 1 0 [expr 180.0 + $fPhi1]] \
 		              [list rotate 0 0 1 [expr $fTheta]] ]
	incr j
	set fTheta [expr $fTheta + $fThetaDelta]
    }

    return g[subst $pcName]
}

##################################################
# CreateSLIDETetrahedron
#   Parameters
#     pcName     - A unique name that will be used as part of the 
#                  name of each SLIDE primitive which is created 
#
#   Returns the name of the top level group "g$pcName"
#
proc CreateSLIDETetrahedron { pcName } {
    set pcObjectName [CreateSLIDERegularPolygon $pcName 3]

    return [CreateSLIDETetrahedronSymmetry $pcName $pcObjectName]
}

##################################################
# CreateSLIDECube
#   Parameters
#     pcName     - A unique name that will be used as part of the 
#                  name of each SLIDE primitive which is created 
#
#   Returns the name of the top level group "g$pcName"
#
proc CreateSLIDECube { pcName } {
    set pcObjectName [CreateSLIDERegularPolygon $pcName 4]

    return [CreateSLIDECubeSymmetry $pcName $pcObjectName]
}

##################################################
# CreateSLIDEOctahedron
#   Parameters
#     pcName     - A unique name that will be used as part of the 
#                  name of each SLIDE primitive which is created 
#
#   Returns the name of the top level group "g$pcName"
#
proc CreateSLIDEOctahedron { pcName } {
    set pcObjectName [CreateSLIDERegularPolygon $pcName 3]

    return [CreateSLIDEOctahedronSymmetry $pcName $pcObjectName]
}

##################################################
# CreateSLIDECubeOctahedron
#   Parameters
#     pcName     - A unique name that will be used as part of the 
#                  name of each SLIDE primitive which is created 
#
#   Returns the name of the top level group "g$pcName"
#
proc CreateSLIDECubeOctahedron { pcName } {
    set pcTriangleName [CreateSLIDERegularPolygon $pcName 3]
    set pcSquareName [CreateSLIDERegularPolygon $pcName 4]

    return [CreateSLIDECubeOctahedronSymmetry $pcName $pcTriangleName $pcSquareName]
}

##################################################
# CreateSLIDEOctaStar
#   Parameters
#     pcName     - A unique name that will be used as part of the 
#                  name of each SLIDE primitive which is created 
#
#   Returns the name of the top level group "g$pcName"
#
proc CreateSLIDEOctaStar { pcName } {
    set pcTriangle [CreateSLIDERegularPyramid ${pcName}Triangle 3 [expr 1.0/sqrt(6.0)]]
    set pcSquare [CreateSLIDERegularPyramid ${pcName}Square 4 [expr 1.0/sqrt(2.0)]]

    return [CreateSLIDECubeOctahedronSymmetry $pcName $pcTriangle $pcSquare]
}

##################################################
# CreateSLIDEDodecahedron
#   Parameters
#     pcName     - A unique name that will be used as part of the 
#                  name of each SLIDE primitive which is created 
#
#   Returns the name of the top level group "g$pcName"
#
proc CreateSLIDEDodecahedron { pcName } {
    set pcObjectName [CreateSLIDERegularPolygon $pcName 5]

    return [CreateSLIDEDodecahedronSymmetry $pcName $pcObjectName]
}

##################################################
# CreateSLIDEDodecaStar
#   Parameters
#     pcName     - A unique name that will be used as part of the 
#                  name of each SLIDE primitive which is created 
#
#   Returns the name of the top level group "g$pcName"
#
proc CreateSLIDEDodecaStar { pcName } {
    global SLF_PI
    set fEdge 1.0
    set fTan36 [expr tan($SLF_PI/5.0)]
    set fTan72 [expr tan(2.0*$SLF_PI/5.0)]
    set fHeight [expr ($fEdge/2.0)*sqrt($fTan72*$fTan72 - 1.0/($fTan36*$fTan36))]

    set pcObjectName [CreateSLIDERegularPyramid $pcName 5 $fHeight]

    return [CreateSLIDEDodecahedronSymmetry $pcName $pcObjectName]
}

##################################################
# CreateSLIDEIcosahedron
#   Parameters
#     pcName     - A unique name that will be used as part of the 
#                  name of each SLIDE primitive which is created 
#
#   Returns the name of the top level group "g$pcName"
#
proc CreateSLIDEIcosahedron { pcName } {
    set pcObjectName [CreateSLIDERegularPolygon $pcName 3]

    return [CreateSLIDEIcosahedronSymmetry $pcName $pcObjectName]
}

##################################################
# CreateSLIDEDodecahedronIcosahedron
#   Parameters
#     pcName     - A unique name that will be used as part of the 
#                  name of each SLIDE primitive which is created 
#
#   Returns the name of the top level group "g$pcName"
#
proc CreateSLIDEDodecahedronIcosahedron { pcName } {
    set pcTriangleName [CreateSLIDERegularPolygon $pcName 3]
    set pcPentagonName [CreateSLIDERegularPolygon $pcName 5]

    return [CreateSLIDEDodecahedronIcosahedronSymmetry $pcName $pcTriangleName $pcPentagonName]
}

##################################################
# CreateSLIDEPolyhedrons
#   Parameters
#     pcName     - A unique name that will be used as part of the 
#                  name of each SLIDE primitive which is created 
#
#   Returns the name of the top level group "g$pcName"
#
proc CreateSLIDEPolyhedrons {} {
    set lPolyhedrons {}

    # Polygons
    set pcTriangle [CreateSLIDERegularPolygon Triangle 3]
    set pcSquare [CreateSLIDERegularPolygon Square 4]
    set pcPentagon [CreateSLIDERegularPolygon Pentagon 5]

    # Pyramids of Equilateral Triangles
    set pcTrianglePyramid [CreateSLIDEEquilateralPyramid TrianglePyramid $pcTriangle 3]
    set pcSquarePyramid [CreateSLIDEEquilateralPyramid SquarePyramid $pcTriangle 4]
    set pcPentagonPyramid [CreateSLIDEEquilateralPyramid PentagonPyramid $pcTriangle 5]

    set pcTrianglePyramid1 [CreateSLIDEEquilateralPyramid TrianglePyramid1 $pcTrianglePyramid 3]
    set pcSquarePyramid1 [CreateSLIDEEquilateralPyramid SquarePyramid1 $pcTrianglePyramid 4]
    set pcPentagonPyramid1 [CreateSLIDEEquilateralPyramid PentagonPyramid1 $pcTrianglePyramid 5]

    # Tetrahedrons
    set pcTetrahedron [CreateSLIDETetrahedronSymmetry Tetrahedron $pcTriangle]
    lappend lPolyhedrons $pcTetrahedron
    set pcTetrahedron1 [CreateSLIDETetrahedronSymmetry Tetrahedron1 $pcTrianglePyramid]
    lappend lPolyhedrons $pcTetrahedron1
    set pcTetrahedron2 [CreateSLIDETetrahedronSymmetry Tetrahedron2 $pcTrianglePyramid1]
    lappend lPolyhedrons $pcTetrahedron2

    # Cubes
    set pcCube [CreateSLIDECubeSymmetry Cube $pcSquare]
    lappend lPolyhedrons $pcCube
    set pcCube1 [CreateSLIDECubeSymmetry Cube1 $pcSquarePyramid]
    lappend lPolyhedrons $pcCube1
    set pcCube2 [CreateSLIDECubeSymmetry Cube2 $pcSquarePyramid1]
    lappend lPolyhedrons $pcCube2

    # Octahedrons
    set pcOctahedron [CreateSLIDEOctahedronSymmetry Octahedron $pcTriangle]
    lappend lPolyhedrons $pcOctahedron
    set pcOctahedron1 [CreateSLIDEOctahedronSymmetry Octahedron1 $pcTrianglePyramid]
    lappend lPolyhedrons $pcOctahedron1

    # CubeOctahedrons
    set pcCubeOctahedron [CreateSLIDECubeOctahedronSymmetry CubeOctahedron $pcTriangle $pcSquare]
    lappend lPolyhedrons $pcCubeOctahedron
    set pcCubeOctahedron1 [CreateSLIDECubeOctahedronSymmetry CubeOctahedron1 $pcTrianglePyramid $pcSquarePyramid]
    lappend lPolyhedrons $pcCubeOctahedron1

    # Dodecahedrons
    set pcDodecahedron [CreateSLIDEDodecahedronSymmetry Dodecahedron $pcPentagon]
    lappend lPolyhedrons $pcDodecahedron
    set pcDodecahedron1 [CreateSLIDEDodecahedronSymmetry Dodecahedron1 $pcPentagonPyramid]
    lappend lPolyhedrons $pcDodecahedron1
    set pcDodecahedron2 [CreateSLIDEDodecahedronSymmetry Dodecahedron2 $pcPentagonPyramid1]
    lappend lPolyhedrons $pcDodecahedron2

    # Icosahedrons
    set pcIcosahedron [CreateSLIDEIcosahedronSymmetry Icosahedron $pcTriangle]
    lappend lPolyhedrons $pcIcosahedron
    set pcIcosahedron1 [CreateSLIDEIcosahedronSymmetry Icosahedron1 $pcTrianglePyramid]
    lappend lPolyhedrons $pcIcosahedron1

    # Dodecahedron-Icosahedrons
    set pcDodecahedronIcosahedron [CreateSLIDEDodecahedronIcosahedronSymmetry DodecahedronIcosahedron $pcTriangle $pcPentagon]
    lappend lPolyhedrons $pcDodecahedronIcosahedron
    set pcDodecahedronIcosahedron1 [CreateSLIDEDodecahedronIcosahedronSymmetry DodecahedronIcosahedron1 $pcTrianglePyramid $pcPentagonPyramid]
    lappend lPolyhedrons $pcDodecahedronIcosahedron1

    return $lPolyhedrons
}

proc SLIDEChooser { var field value } {
    global $var
    global SLF_FULL SLF_OFF

    set curr 0
    if { $field == {} } {
	set curr [subst $$var]
    } else {
	set curr [subst $${var}($field)]
    }

    if { $curr == $value } {
	return $SLF_FULL
    } else {
	return $SLF_OFF
    }
}

##################################################
# CreateSLIDEPolyhedronChooser
#   Parameters
#     pcName     - A unique name that will be used as part of the 
#                  name of each SLIDE primitive which is created 
#
#   Returns the name of the top level group "g$pcName"
#
proc CreateSLIDEPolyhedronChooser { pcName pTclObject lPolyhedrons } {

    slide create group g[subst $pcName] \
	    -lod [list expr $${pTclObject}(lod)] \
	    -shading [list expr $${pTclObject}(shading)]

    set i 0
    foreach pcPolyhedron $lPolyhedrons {
	slide create instance i[subst $pcName][subst $i] g[subst $pcName] $pcPolyhedron \
		-lod [list SLIDEChooser $pTclObject polyhedron $i]
	incr i
    }

    return g[subst $pcName]
}

######################################################################
# CreateSLIDEPolyhedronChooserObject
#   Parameters:
#     name     - name of the PolyhedronChooser
#
#   Actions:
#     Creates an associative array with initialized fields for a PolyhedronChooser
#
#   Returns:
#     The name of the associative array
#
proc CreateSLIDEPolyhedronChooserObject { name } {
    set this $name

    global $this

    # shading
    CreateSLIDEShadingField $this

    # lod
    CreateSLIDELODField $this

    set ${this}(polyhedron) 0

    return $this
}

######################################################################
# CreateSLIDEPolyhedronChooserUI
#   Parameters:
#     parent   - parent widget name
#     name     - name of the PolyhedronChooser
#
#   Actions:
#     Creates a PolyhedronChooser UI contained in a frame widget
#
#   Returns:
#     The name of the unpacked frame widget
#
proc CreateSLIDEPolyhedronChooserUI { parent name lPolyhedrons } {
    set root $parent.f$name
    set this $name

    global $this

    frame $root

    ##################################################
    # Enums Frame
    frame $root.fEnums
    
    ##############################
    # lod
    set widget [CreateSLIDELODEnumFrame $root.fEnums lod \
	    [subst $this](lod) "LOD"]
    pack $widget -side left -fill y

    ##############################
    # shading
    set widget [CreateSLIDEShadingEnumFrame $root.fEnums shading \
	    [subst $this](shading) "Shading"]
    pack $widget -side left -fill y

    ##############################
    # polyhedron
    set lEnums {}
    set i 0
    foreach pcPolyhedron $lPolyhedrons {
	lappend lEnums [list $pcPolyhedron $i]
	incr i
    }
    set widget [CreateRadioButtonFrame $root.fEnums fPolyhedron \
	    ${this}(polyhedron) "Polyhedron" $lEnums]
    pack $widget -side left -fill y

    pack $root.fEnums -side top -fill x

    ##################################################
    # Scales Frame
    #frame $root.fScales
    
    #pack $root.fScales -side top -fill x

    return $root
}

