##################################################
# Matrix operations in TCL
#

proc MatrixIdentity {} {
    return {{ 1.0 0.0 0.0 0.0 } \
	    { 0.0 1.0 0.0 0.0 } \
	    { 0.0 0.0 1.0 0.0 } \
	    { 0.0 0.0 0.0 1.0 }}

}

proc MatrixRotateZ { fRadians } {
    return [list [list [expr cos($fRadians)] [expr -sin($fRadians)] 0.0 0.0] \
	         [list [expr sin($fRadians)] [expr cos($fRadians)] 0.0 0.0] \
	         [list 0.0 0.0 1.0 0.0] \
	         [list 0.0 0.0 0.0 1.0] ]
}

proc MatrixTranslate { vTrans } {
    return [list [list 1.0 0.0 0.0 [lindex $vTrans 0]] \
	         [list 0.0 1.0 0.0 [lindex $vTrans 1]] \
	         [list 0.0 0.0 1.0 [lindex $vTrans 2]] \
	         [list 0.0 0.0 0.0 1.0] ]
}

proc MatrixScale { vScale } {
    return [list [list [lindex $vScale 0] 0.0 0.0 0.0] \
	         [list 0.0 [lindex $vScale 1] 0.0 0.0] \
	         [list 0.0 0.0 [lindex $vScale 2] 0.0] \
	         [list 0.0 0.0 0.0 1.0] ]
}

proc MatrixScaleAll { fScale } {
    return [list [list $fScale 0.0 0.0 0.0] \
	         [list 0.0 $fScale 0.0 0.0] \
	         [list 0.0 0.0 $fScale 0.0] \
	         [list 0.0 0.0 0.0 1.0] ]
}

proc Matrix2DTransformTRS { vTrans fRadians fScale } {
    set mScale [MatrixScaleAll $fScale]
    set mRotate [MatrixRotateZ $fRadians]
    set mTranslate [MatrixTranslate $vTrans]

    set mResult [MatrixMultMatrix $mTranslate [MatrixMultMatrix $mRotate $mScale]]

    return $mResult
}

proc MatrixMultMatrix { mA mB } {
    set uiRowsA [llength $mA]
    set uiColsA [llength [lindex $mA 0] ]
    set uiRowsB [llength $mB]
    set uiColsB [llength [lindex $mB 0] ]

    #puts "uiRowsA = $uiRowsA"
    #puts "uiColsA = $uiColsA"
    #puts "uiRowsB = $uiRowsB"
    #puts "uiColsB = $uiColsB"
    
    if { $uiColsA != $uiRowsB } {
	puts "ERROR: MatrixMultMatrix: Inner dimensions do not match: $uiColsA != $uiRowsB"
	return {}
    }

    set mResult {}
    for { set uiRow 0 } { $uiRow < $uiRowsA } { incr uiRow } {
	set pfRow {}
	for { set uiCol 0 } { $uiCol < $uiColsB } { incr uiCol } {
	    set fValue 0.0
	    for { set i 0 } { $i < $uiColsA } { incr i } {
		set fValue [expr $fValue + [lindex [lindex $mA $uiRow] $i] * [lindex [lindex $mB $i] $uiCol] ]
	    }
	    lappend pfRow $fValue
	}
	lappend mResult $pfRow
    }

    return $mResult
}

proc MatrixAddMatrix { mA mB } {
    set uiRowsA [llength $mA]
    set uiColsA [llength [lindex $mA 0] ]
    set uiRowsB [llength $mB]
    set uiColsB [llength [lindex $mB 0] ]
    
    if { !( $uiRowsA == $uiRowsB && $uiColsA == $uiColsB ) } {
	puts "ERROR: MatrixAddMatrix: Matrix dimensions do not match: ($uiRowsA, $uiColsA) != ($uiRowsB, $uiColsB)"
	return {}
    }

    set mResult {}
    for { set uiRow 0 } { $uiRow < $uiRowsA } { incr uiRow } {
	set pfRow {}
	for { set uiCol 0 } { $uiCol < $uiColsB } { incr uiCol } {
	    lappend pfRow [expr [lindex [lindex $mA $uiRow] $uiCol] + [lindex [lindex $mB $uiRow] $uiCol] ]
	}
	lappend mResult $pfRow
    }

    return $mResult
}

proc MatrixSubtMatrix { mA mB } {
    set uiRowsA [llength $mA]
    set uiColsA [llength [lindex $mA 0] ]
    set uiRowsB [llength $mB]
    set uiColsB [llength [lindex $mB 0] ]
    
    if { !( $uiRowsA == $uiRowsB && $uiColsA == $uiColsB ) } {
	puts "ERROR: MatrixSubtMatrix: Matrix dimensions do not match: ($uiRowsA, $uiColsA) != ($uiRowsB, $uiColsB)"
	return {}
    }

    set mResult {}
    for { set uiRow 0 } { $uiRow < $uiRowsA } { incr uiRow } {
	set pfRow {}
	for { set uiCol 0 } { $uiCol < $uiColsB } { incr uiCol } {
	    lappend pfRow [expr [lindex [lindex $mA $uiRow] $uiCol] - [lindex [lindex $mB $uiRow] $uiCol] ]
	}
	lappend mResult $pfRow
    }

    return $mResult
}
