package require slideui

######################################################################
# START: SLFCONSTS.tcl
#

set SLF_PI 3.1415927

# SLIDE Flag Constants
#
# To be used while debugging tcl code with the "wish" program

# On/Off Flags
set SLF_OFF 0
set SLF_ON 1

# Solid Flags
set SLF_SOLID 0
set SLF_HOLLOW 1

# Shading Flags
set SLF_INHERIT 0
set SLF_WIRE 1
set SLF_FLAT 2
set SLF_GOURAUD 3
set SLF_PHONG 4

# LOD Flags
set SLF_OFF 0
set SLF_BOUND 1
set SLF_EDGES 2
set SLF_FULL 3

# Projection Flags
set SLF_PARALLEL 0
set SLF_PERSPECTIVE 1

# Lighting Type Flags
set SLF_AMBIENT 0
set SLF_DIRECTIONAL 1
set SLF_POINT 2
set SLF_SPOT 3

# Window size Flags
set SLF_FIXED 0
set SLF_ANY 1
set SLF_FULL_SCREEN 2

# Subdivision Type Flags
set SLF_SUBDIVISION_DOO_SABIN 0
set SLF_SUBDIVISION_DOO_SABIN_SELECTIVE 1
set SLF_SUBDIVISION_DOO_SABIN_SELECTIVE_LIMIT 2
set SLF_SUBDIVISION_CATMULL_CLARK 3
set SLF_SUBDIVISION_SHARP_CATMULL_CLARK 4
set SLF_SUBDIVISION_CATMULL_CLARK_TESSELATE 5
set SLF_SUBDIVISION_CATMULL_CLARK_INTERPOLATE 6
set SLF_SUBDIVISION_CATMULL_CLARK_SELECTIVE 7
set SLF_SUBDIVISION_CATMULL_CLARK_OLD 8
set SLF_SUBDIVISION_CORNER_CUTTING 9
set SLF_SUBDIVISION_CORNER_ROUNDING_TESSELATE 10
set SLF_SUBDIVISION_CORNER_ROUNDING 11
set SLF_SUBDIVISION_CORNER_ROUNDING_LOOP 12
set SLF_SUBDIVISION_CORNER_ROUNDING_SHARP_LOOP 13
set SLF_SUBDIVISION_CORNER_ROUNDING_BUTTERFLY 14
set SLF_SUBDIVISION_CORNER_ROUNDING_ZORIN 15
set SLF_SUBDIVISION_CORNER_ROUNDING_SELECTIVE 16
set SLF_SUBDIVISION_CORNER_ROUNDING_SELECTIVE_LIMIT 17
set SLF_SUBDIVISION_CORNER_ROUNDING_SHARP_SELECTIVE_LIMIT 18
set SLF_SUBDIVISION_CORNER_ROUNDING_FACET_SHARP_SELECTIVE_LIMIT 19
set SLF_SUBDIVISION_TRIANGLE_TESSELATE 20
set SLF_SUBDIVISION_TRIANGLE_LOOP 21
set SLF_SUBDIVISION_TRIANGLE_LOOP_EDGE_RULE 22
set SLF_SUBDIVISION_TRIANGLE_BUTTERFLY 23
set SLF_SUBDIVISION_TRIANGLE_ZORIN 24
set SLF_SUBDIVISION_TRIANGLE_SPHERE 25
set SLF_SUBDIVISION_SQRT_3 26
set SLF_SUBDIVISION_SQRT_3_SPHERE_BARYCENTER 27
set SLF_SUBDIVISION_SQRT_3_SPHERE_CENTER 28
set SLF_SUBDIVISION_SQRT_3_SPHERE_RADIUS 29
set SLF_SUBDIVISION_SQRT_3_SPHERE_CAP_AREA 30
set SLF_SUBDIVISION_SQRT_3_SPHERE_TRIANGLE_AREA 31
set SLF_SUBDIVISION_SQRT_3_SPHERE_TANGENT 32
set SLF_SUBDIVISION_SQRT_3_SELECTIVE 33
set SLF_SUBDIVISION_SQRT_3_SELECTIVE_FACET 34
set SLF_SUBDIVISION_SQRT_4 35
set SLF_SUBDIVISION_SQRT_4_SELECTIVE 36
set SLF_SUBDIVISION_PROGRESSIVE 37
set SLF_SUBDIVISION_HETEROGENEOUS 38
set SLF_SUBDIVISION_HETEROGENEOUS_TESSELATE 39
set SLF_SUBDIVISION_PENTAGON 40
set SLF_SUBDIVISION_PENTAGON_TESSELATE 41
set SLF_SUBDIVISION_GEOMETRIC 42
set SLF_SUBDIVISION_GEOMETRIC_TESSELATE 43

# Error Metric Flags
set SLF_ERROR_METRIC_BEST_FIT_PLANE 0
set SLF_ERROR_METRIC_LIMIT_DISTANCE 1

# Offset Type Flags
set SLF_OFFSET_ANGULAR 0
set SLF_OFFSET_ROUNDED 1
set SLF_OFFSET_PERFORATED 2

# Terrain Type Flags
set SLF_TERRAIN_DELAUNAY_INCREMENTAL 0
set SLF_TERRAIN_DELAUNAY_INCREMENTAL_SLOW 1
set SLF_TERRAIN_DELAUNAY_INCREMENTAL_NO_BOUND 2
set SLF_TERRAIN_DELAUNAY_DIVIDE_AND_CONQUER 3
set SLF_TERRAIN_DELAUNAY_DIVIDE_AND_CONQUER_ALTERNATING 4
set SLF_TERRAIN_DELAUNAY_CONVEX_HULL 5

# Voronoi Type Flags
set SLF_VORONOI_OFF 0
set SLF_VORONOI_DELAUNAY 1
set SLF_VORONOI_DIVIDE_AND_CONQUER 2
set SLF_VORONOI_DIVIDE_AND_CONQUER_ALTERNATING 3
set SLF_VORONOI_CONES 4
set SLF_VORONOI_PLANES 5

# Accuracy Type Flags
set SLF_FLOAT_NAIVE 0
set SLF_FLOAT 1
set SLF_EXACT 2

#
# END  : SLFCONSTS.tcl
######################################################################

######################################################################
# START: MATH.tcl
#

proc Sign { a } {
    if { $a < 0.0 } {
	return -1.0
    } else {
	return 1.0
    }
}

proc DegToRad { degrees } {
    global SLF_PI

    while { $degrees > 180 } {
	set degrees [expr $degrees - 360]
    }
    while { $degrees < -180 } {
	set degrees [expr $degrees + 360]
    }

    return [expr $degrees * $SLF_PI / 180]
}

proc RadToDeg { radians } {
    global SLF_PI
    set degrees [expr $radians * 180 / $SLF_PI]
    set degrees $degrees

    while { $degrees > 180 } {
	set degrees [expr $degrees - 360]
    }
    while { $degrees < -180 } {
	set degrees [expr $degrees + 360]
    }

    return $degrees
}

proc Min { a b } {
  set min $a

  if { $b < $a } {
    set min $b
  }

  return $min
}

proc Max { a b } {
  set max $a

  if { $b > $a } {
    set max $b
  }

  return $max
}

proc Oscillate {} {
  global SLF_TIME
  return [expr (cos($SLF_TIME) + 1) / 2]
}

proc ShearAlpha { sh } {
    #puts "Shear = $sh"

    #puts "Alpha = [RadToDeg [expr [Sign $sh] * atan( 1.0 / ( 1.0 + abs($sh)))]]"
    return [RadToDeg [expr [Sign $sh] * atan( 1.0 / ( 1.0 + abs($sh)))]]
}

proc ShearScaleYOnly { sh } {
    set mag [expr abs($sh)]

    #puts "Sy = [expr sqrt( (1.0 + $mag) * (1.0 + $mag + $mag*$mag) )]"
    return [expr sqrt( (1.0 + $mag) * (1.0 + $mag + $mag*$mag) )]
}

proc ShearBeta { sh } {
    set mag [expr abs($sh)]

    #puts "Beta = [RadToDeg [expr -1.0 * [Sign $sh] * atan( sqrt( (1.0 + $mag + $mag*$mag) / (1.0 + $mag)))]]"
    return [RadToDeg [expr -1.0 * [Sign $sh] * atan( sqrt( (1.0 + $mag + $mag*$mag) / (1.0 + $mag)))]]
}

proc ShearScaleX { sh } {
    set mag [expr abs($sh)]

    #puts "X = [expr 1.0 / sqrt( 1.0 + $mag )]"
    return [expr 1.0 / sqrt( 1.0 + $mag )]
}

proc ShearScaleY { sh } {
    set mag [expr abs($sh)]

    #puts "Y = [expr 1.0 / sqrt( 1.0 + $mag + $mag*$mag )]"
    return [expr 1.0 / sqrt( 1.0 + $mag + $mag*$mag )]
}

#
# END  : MATH.tcl
######################################################################

######################################################################
# CreateRadioButtonFrame 
#   Parameters:
#     parent   - parent widget name
#     name     - name of the radio button frame widget (convention: rbf<name>)
#     var      - name of the global variable which will be modified
#     text     - text name to be displayed on the widget
#     enumList - enumerated list of fields and values { {field0 value0} ... }
#
#   Actions:
#     Creates a container frame widget
#     Creates and packs a title label and radio buttons inside the frame
#
#   Returns:
#     The name of the unpacked container frame widget
#
proc CreateRadioButtonFrame { parent name var text enumList } {
    set root $parent.f$name

    frame $root -relief raised -bd 2

    #global $var
    #set $var [lindex [lindex $enumList $default] 1]

    label $root.lTitle -text $text
    pack $root.lTitle -side top -fill x

    foreach choice $enumList {
	set field [lindex $choice 0]
	set value [lindex $choice 1]
	set widgetButton $root.rb$field

	radiobutton $widgetButton \
		-text $field \
		-variable $var \
		-value $value \
		-anchor w
	pack $widgetButton -side top -fill x
    }

    return $root
}

######################################################################
# CreateComboBoxFrame 
#   Parameters:
#     parent   - parent widget name
#     name     - name of the radio button frame widget (convention: rbf<name>)
#     var      - name of the global variable which will be modified
#     text     - text name to be displayed on the widget
#     enumList - enumerated list of fields and values { {field0 value0} ... }
#
#   Actions:
#     Creates a container frame widget
#     Creates and packs a title label and radio buttons inside the frame
#
#   Returns:
#     The name of the unpacked container frame widget
#
proc CreateComboBoxFrame { parent name var text enumList } {
    set root $parent.f$name

    frame $root -relief raised -bd 2

    label $root.lTitle -text $text
    pack $root.lTitle -side top -fill x

    menubutton $root.mbValue -text [lindex [lindex $enumList 0] 0] -relief raised \
	    -menu $root.mbValue.menu
    pack $root.mbValue -side top -fill x

    menu $root.mbValue.menu

    foreach choice $enumList {
        set field [lindex $choice 0]
        set value [lindex $choice 1]
        set widgetButton $root.rb$field

        $root.mbValue.menu add radiobutton \
                -label $field \
                -variable $var \
                -value $value \
                -command "$root.mbValue conf -text $field"
    }

    return $root
}

proc CalculateTickInterval { min max tickints } {
    return [expr ($max - $min)/$tickints]
}

######################################################################
# CreateScale 
#   Parameters:
#     parent   - parent widget name
#     name     - name of the widget (convention: scale<name>)
#     var      - name of the global variable which will be modified
#     text     - text name to be displayed on the widget
#     min      - minimum value for the variable
#     max      - maximum value for the variable
#     resolution - resolution for changing the value
#     tickints - how many tick marks on the scale
#     orient   - horizontal or vertical
#     length   - length of the scale
#     cmd      - command to be called when the scale changes value
#
#   Actions:
#     Creates a scale widget
#
#   Returns:
#     The name of the unpacked scale widget
#
proc CreateScale { parent name var text min max resolution {cmd {}} {tickints 1} {orient horizontal} {length 0} } {
    set widgetScale $parent.$name
    
    #global $var
    #set $var $default

    set tickinterval [CalculateTickInterval $min $max $tickints]

    if { $cmd == {} } {
	if { $length > 0 } {
	    scale $widgetScale \
		    -label $text \
		    -from $min \
		    -to $max \
		    -resolution $resolution \
		    -tickint $tickinterval \
		    -length $length \
		    -orient $orient \
		    -variable $var
	} else {
	    scale $widgetScale \
		    -label $text \
		    -from $min \
		    -to $max \
		    -resolution $resolution \
		    -tickint $tickinterval \
		    -orient $orient \
		    -variable $var
	}
    } else {
	if { $length > 0 } {
	    scale $widgetScale \
		    -label $text \
		    -from $min \
		    -to $max \
		    -resolution $resolution \
		    -tickint $tickinterval \
		    -length $length \
		    -orient $orient \
		    -variable $var \
		    -command $cmd
	} else {
	    scale $widgetScale \
		    -label $text \
		    -from $min \
		    -to $max \
		    -resolution $resolution \
		    -tickint $tickinterval \
		    -orient $orient \
		    -variable $var \
		    -command $cmd
	}
    }
    
    return $widgetScale
}

######################################################################
# CreateMultiScale 
#   Parameters:
#     parent   - parent widget name
#     name     - name of the widget (convention: scale<name>)
#     var      - name of the global variable which will be modified
#     text     - text name to be displayed on the widget
#     min      - minimum value for the variable
#     max      - maximum value for the variable
#     resolution - resolution for changing the value
#     tickints - how many tick marks on the scale
#     orient   - horizontal or vertical
#     length   - length of the scale
#
#   Actions:
#     Creates a multiscale widget
#
#   Returns:
#     The name of the unpacked scale widget
#
proc CreateMultiScale { parent name vars text min max resolution {spacing 0.0} {commands {}} {tickints 1} {orient horizontal} {length 0} } {
    set widgetMultiScale $parent.$name
    
    frame $widgetMultiScale

    label $widgetMultiScale.lTitle -text $text
    pack $widgetMultiScale.lTitle -side top -fill x

    frame $widgetMultiScale.fBottom

    label $widgetMultiScale.fBottom.lMin -text $min
    pack $widgetMultiScale.fBottom.lMin -side left -fill y
    
    set tickinterval [CalculateTickInterval $min $max $tickints]

    if { $length > 0 } {
	multiscale $widgetMultiScale.fBottom.ms \
		-label $text \
		-from $min \
		-to $max \
		-resolution $resolution \
		-spacing $spacing \
		-tickint $tickinterval \
		-length $length \
		-orient $orient \
		-variables $vars \
		-commands $commands
    } else {
	multiscale $widgetMultiScale.fBottom.ms \
		-label $text \
		-from $min \
		-to $max \
		-resolution $resolution \
		-spacing $spacing \
		-tickint $tickinterval \
		-orient $orient \
		-variables $vars \
		-commands $commands
    }
    pack $widgetMultiScale.fBottom.ms -side left -fill y

    label $widgetMultiScale.fBottom.lMax -text $max
    pack $widgetMultiScale.fBottom.lMax -side left -fill y

    pack $widgetMultiScale.fBottom -side top -fill x
    
    return $widgetMultiScale
}

######################################################################
# CreateSLIDEEnumList
#   Parameters:
#     fieldList - list of SLIDE constants { field0 field1 ... }
#
#   Actions:
#
#   Returns:
#     { {field0 value0} {field1 value1} ... }
#
proc CreateSLIDEEnumList { fieldList } {
    set enumList {}

    foreach field $fieldList {
	global $field
	lappend enumList "[subst $field] [subst $$field]"
    }

    return $enumList
}

######################################################################
# CreateSLIDEOnOffEnumFrame
#   Parameters:
#     parent   - parent widget name
#     name     - name of the widget
#     var      - name of the global variable which will be modified
#     text     - text name to be displayed on the widget
#
#   Actions:
#     Creates an On/Off UI contained in a frame widget
#
#   Returns:
#     The name of the unpacked frame widget
#
proc CreateSLIDEOnOffEnumFrame { parent name var text } {
    set lodList [list SLF_OFF SLF_ON]
    set enumList [CreateSLIDEEnumList $lodList]

    return [CreateComboBoxFrame $parent $name $var $text $enumList]
}

######################################################################
# CreateSLIDETextureFields
#   Parameters:
#     this     - name of the associative array
#
#   Actions:
#     Creates the 4 texture coordinate point fields for a standard surface
#
#   Returns:
#     True
#
proc CreateSLIDETextureFields { this } {
    global $this

    for { set i 0 } { $i < 4 } { incr i } {
	for { set j 0 } { $j < 3 } { incr j } {
	    set [subst $this](texture_[subst $i]_[subst $j]) \
		    [if { ( $j == 0 && ( $i == 1 || $i == 2 ) ) || \
		          ( $j == 1 && ( $i & 0x2 ) ) } \
			  { expr 1.0 } else { expr 0.0 }]
	    set [subst $this](texture_[subst $i]_[subst $j]_min) 0.0
	    set [subst $this](texture_[subst $i]_[subst $j]_max) 4.0
	    set [subst $this](texture_[subst $i]_[subst $j]_resolution) 0.01
	}
    }

    return 1
}

######################################################################
# CreateSLIDESolidField
#   Parameters:
#     this     - name of the associative array
#
#   Actions:
#     Creates a field for the solid enumerated type
#
#   Returns:
#     Initial value for the field
#
proc CreateSLIDESolidField { this } {
    global $this
    global SLF_SOLID

    set [subst $this](solid) $SLF_SOLID

    return [subst $[subst $this](solid)]
}

######################################################################
# CreateSLIDESolidEnumFrame
#   Parameters:
#     parent   - parent widget name
#     name     - name of the widget
#     var      - name of the global variable which will be modified
#     text     - text name to be displayed on the widget
#
#   Actions:
#     Creates a Solid UI contained in a frame widget
#
#   Returns:
#     The name of the unpacked frame widget
#
proc CreateSLIDESolidEnumFrame { parent name var text } {
    set lodList [list SLF_SOLID SLF_HOLLOW]
    set enumList [CreateSLIDEEnumList $lodList]

    return [CreateComboBoxFrame $parent $name $var $text $enumList]
}

######################################################################
# CreateSLIDEShadingField
#   Parameters:
#     this     - name of the associative array
#
#   Actions:
#     Creates a field for the shading enumerated type
#
#   Returns:
#     Initial value for the field
#
proc CreateSLIDEShadingField { this } {
    global $this
    global SLF_INHERIT

    set [subst $this](shading) $SLF_INHERIT

    return [subst $[subst $this](shading)]
}

######################################################################
# CreateSLIDEShadingEnumFrame
#   Parameters:
#     parent   - parent widget name
#     name     - name of the widget
#     var      - name of the global variable which will be modified
#     text     - text name to be displayed on the widget
#
#   Actions:
#     Creates a Shading UI contained in a frame widget
#
#   Returns:
#     The name of the unpacked frame widget
#
proc CreateSLIDEShadingEnumFrame { parent name var text } {
    set lodList [list SLF_INHERIT SLF_WIRE SLF_FLAT SLF_GOURAUD]
    set enumList [CreateSLIDEEnumList $lodList]

    return [CreateComboBoxFrame $parent $name $var $text $enumList]
}

######################################################################
# CreateSLIDELODField
#   Parameters:
#     this     - name of the associative array
#
#   Actions:
#     Creates a field for the LOD enumerated type
#
#   Returns:
#     Initial value for the field
#
proc CreateSLIDELODField { this } {
    global $this
    global SLF_FULL

    set [subst $this](lod) $SLF_FULL

    return [subst $[subst $this](lod)]
}

######################################################################
# CreateSLIDELODEnumFrame
#   Parameters:
#     parent   - parent widget name
#     name     - name of the widget
#     var      - name of the global variable which will be modified
#     text     - text name to be displayed on the widget
#
#   Actions:
#     Creates a LOD UI contained in a frame widget
#
#   Returns:
#     The name of the unpacked frame widget
#
proc CreateSLIDELODEnumFrame { parent name var text } {
    set lodList [list SLF_OFF SLF_BOUND SLF_EDGES SLF_FULL]
    set enumList [CreateSLIDEEnumList $lodList]

    return [CreateComboBoxFrame $parent $name $var $text $enumList]
}

######################################################################
# CreateSLIDEProjectionField
#   Parameters:
#     this     - name of the associative array
#
#   Actions:
#     Creates a field for the projection enumerated type
#
#   Returns:
#     Initial value for the field
#
proc CreateSLIDEProjectionField { this } {
    global $this
    global SLF_PERSPECTIVE

    set [subst $this](projection) $SLF_PERSPECTIVE

    return [subst $[subst $this](projection)]
}

######################################################################
# CreateSLIDEProjectionEnumFrame
#   Parameters:
#     parent   - parent widget name
#     name     - name of the widget
#     var      - name of the global variable which will be modified
#     text     - text name to be displayed on the widget
#
#   Actions:
#     Creates a Projection UI contained in a frame widget
#
#   Returns:
#     The name of the unpacked frame widget
#
proc CreateSLIDEProjectionEnumFrame { parent name var text } {
    set lodList [list SLF_PERSPECTIVE SLF_PARALLEL]
    set enumList [CreateSLIDEEnumList $lodList]

    return [CreateComboBoxFrame $parent $name $var $text $enumList]
}

######################################################################
# CreateSLIDELightEnumFrame
#   Parameters:
#     parent   - parent widget name
#     name     - name of the widget
#     var      - name of the global variable which will be modified
#     text     - text name to be displayed on the widget
#
#   Actions:
#     Creates a Light Enum UI contained in a frame widget
#
#   Returns:
#     The name of the unpacked frame widget
#
proc CreateSLIDELightEnumFrame { parent name var text } {
    set lodList [list SLF_AMBIENT SLF_DIRECTIONAL SLF_POINT SLF_SPOT]
    set enumList [CreateSLIDEEnumList $lodList]

    return [CreateComboBoxFrame $parent $name $var $text $enumList]
}

######################################################################
# CreateSLIDEFogEnumFrame
#   Parameters:
#     parent   - parent widget name
#     name     - name of the widget
#     var      - name of the global variable which will be modified
#     text     - text name to be displayed on the widget
#
#   Actions:
#     Creates a Fog Enum UI contained in a frame widget
#
#   Returns:
#     The name of the unpacked frame widget
#
proc CreateSLIDEFogEnumFrame { parent name var text } {
    set lodList [list SLF_LINEAR SLF_EXP SLF_EXP2]
    set enumList [CreateSLIDEEnumList $lodList]

    return [CreateComboBoxFrame $parent $name $var $text $enumList]
}

######################################################################
# CreateSLIDEStencilEnumFrame
#   Parameters:
#     parent   - parent widget name
#     name     - name of the widget
#     var      - name of the global variable which will be modified
#     text     - text name to be displayed on the widget
#
#   Actions:
#     Creates a Stencil Enum UI contained in a frame widget
#
#   Returns:
#     The name of the unpacked frame widget
#
proc CreateSLIDEStencilEnumFrame { parent name var text } {
    set lodList [list SLF_BOTH SLF_ODD SLF_EVEN]
    set enumList [CreateSLIDEEnumList $lodList]

    return [CreateComboBoxFrame $parent $name $var $text $enumList]
}

######################################################################
# CreateSLIDERatioEnumFrame
#   Parameters:
#     parent   - parent widget name
#     name     - name of the widget
#     var      - name of the global variable which will be modified
#     text     - text name to be displayed on the widget
#
#   Actions:
#     Creates a Ratio Enum UI contained in a frame widget
#
#   Returns:
#     The name of the unpacked frame widget
#
proc CreateSLIDERatioEnumFrame { parent name var text } {
    set lodList [list SLF_FIXED SLF_ANY SLF_FULL_SCREEN]
    set enumList [CreateSLIDEEnumList $lodList]

    return [CreateComboBoxFrame $parent $name $var $text $enumList]
}

######################################################################
# CreateSLIDESubdivisionField
#   Parameters:
#     this     - name of the associative array
#
#   Actions:
#     Creates a field for the subdivision enumerated type
#
#   Returns:
#     Initial value for the field
#
proc CreateSLIDESubdivisionField { this } {
    global $this
    global SLF_SUBDIVISION_SHARP_CATMULL_CLARK SLF_SUBDIVISION_CORNER_ROUNDING_SHARP_LOOP

    set [subst $this](type) $SLF_SUBDIVISION_SHARP_CATMULL_CLARK
    #set [subst $this](type) $SLF_SUBDIVISION_CORNER_ROUNDING_SHARP_LOOP
    return [subst $[subst $this](type)]
}

######################################################################
# CreateSLIDESubdivisionEnumFrame
#   Parameters:
#     parent   - parent widget name
#     name     - name of the widget
#     var      - name of the global variable which will be modified
#     text     - text name to be displayed on the widget
#
#   Actions:
#     Creates a Subdivision Enum UI contained in a frame widget
#
#   Returns:
#     The name of the unpacked frame widget
#
proc CreateSLIDESubdivisionEnumFrame { parent name var text } {
    set subdivisionList [list SLF_SUBDIVISION_DOO_SABIN SLF_SUBDIVISION_DOO_SABIN_SELECTIVE SLF_SUBDIVISION_DOO_SABIN_SELECTIVE_LIMIT SLF_SUBDIVISION_CATMULL_CLARK SLF_SUBDIVISION_SHARP_CATMULL_CLARK SLF_SUBDIVISION_CATMULL_CLARK_TESSELATE SLF_SUBDIVISION_CATMULL_CLARK_INTERPOLATE SLF_SUBDIVISION_CATMULL_CLARK_SELECTIVE SLF_SUBDIVISION_CATMULL_CLARK_OLD SLF_SUBDIVISION_CORNER_CUTTING SLF_SUBDIVISION_CORNER_ROUNDING_TESSELATE SLF_SUBDIVISION_CORNER_ROUNDING SLF_SUBDIVISION_CORNER_ROUNDING_LOOP SLF_SUBDIVISION_CORNER_ROUNDING_SHARP_LOOP SLF_SUBDIVISION_CORNER_ROUNDING_BUTTERFLY SLF_SUBDIVISION_CORNER_ROUNDING_ZORIN SLF_SUBDIVISION_CORNER_ROUNDING_SELECTIVE SLF_SUBDIVISION_CORNER_ROUNDING_SELECTIVE_LIMIT SLF_SUBDIVISION_CORNER_ROUNDING_SHARP_SELECTIVE_LIMIT SLF_SUBDIVISION_CORNER_ROUNDING_FACET_SHARP_SELECTIVE_LIMIT SLF_SUBDIVISION_TRIANGLE_TESSELATE SLF_SUBDIVISION_TRIANGLE_LOOP SLF_SUBDIVISION_TRIANGLE_LOOP_EDGE_RULE SLF_SUBDIVISION_TRIANGLE_BUTTERFLY SLF_SUBDIVISION_TRIANGLE_ZORIN SLF_SUBDIVISION_TRIANGLE_SPHERE SLF_SUBDIVISION_SQRT_3 SLF_SUBDIVISION_SQRT_3_SPHERE_BARYCENTER SLF_SUBDIVISION_SQRT_3_SPHERE_CENTER SLF_SUBDIVISION_SQRT_3_SPHERE_RADIUS SLF_SUBDIVISION_SQRT_3_SPHERE_CAP_AREA SLF_SUBDIVISION_SQRT_3_SPHERE_TRIANGLE_AREA SLF_SUBDIVISION_SQRT_3_SPHERE_TANGENT SLF_SUBDIVISION_SQRT_3_SELECTIVE SLF_SUBDIVISION_SQRT_3_SELECTIVE_FACET SLF_SUBDIVISION_SQRT_4 SLF_SUBDIVISION_SQRT_4_SELECTIVE SLF_SUBDIVISION_PROGRESSIVE SLF_SUBDIVISION_HETEROGENEOUS SLF_SUBDIVISION_HETEROGENEOUS_TESSELATE SLF_SUBDIVISION_PENTAGON SLF_SUBDIVISION_PENTAGON_TESSELATE SLF_SUBDIVISION_GEOMETRIC SLF_SUBDIVISION_GEOMETRIC_TESSELATE]

    set enumList [CreateSLIDEEnumList $subdivisionList]

    return [CreateComboBoxFrame $parent $name $var $text $enumList]
}

######################################################################
# CreateSLIDEErrorMetricField
#   Parameters:
#     this     - name of the associative array
#
#   Actions:
#     Creates a field for the error metric enumerated type
#
#   Returns:
#     Initial value for the field
#
proc CreateSLIDEErrorMetricField { this } {
    global $this
    global SLF_ERROR_METRIC_BEST_FIT_PLANE

    set [subst $this](errormetric) $SLF_ERROR_METRIC_BEST_FIT_PLANE

    return [subst $[subst $this](errormetric)]
}

######################################################################
# CreateSLIDEErrorMetricEnumFrame
#   Parameters:
#     parent   - parent widget name
#     name     - name of the widget
#     var      - name of the global variable which will be modified
#     text     - text name to be displayed on the widget
#
#   Actions:
#     Creates a Error Metric Enum UI contained in a frame widget
#
#   Returns:
#     The name of the unpacked frame widget
#
proc CreateSLIDEErrorMetricEnumFrame { parent name var text } {
    set errorMetricList [list SLF_ERROR_METRIC_BEST_FIT_PLANE SLF_ERROR_METRIC_LIMIT_DISTANCE]

    set enumList [CreateSLIDEEnumList $errorMetricList]

    return [CreateComboBoxFrame $parent $name $var $text $enumList]
}

######################################################################
# CreateSLIDEOffsetField
#   Parameters:
#     this     - name of the associative array
#
#   Actions:
#     Creates a field for the offset enumerated type
#
#   Returns:
#     Initial value for the field
#
proc CreateSLIDEOffsetField { this } {
    global $this
    global SLF_OFFSET_ANGULAR

    set [subst $this](type) $SLF_OFFSET_ANGULAR

    return [subst $[subst $this](type)]
}

######################################################################
# CreateSLIDEOffsetEnumFrame
#   Parameters:
#     parent   - parent widget name
#     name     - name of the widget
#     var      - name of the global variable which will be modified
#     text     - text name to be displayed on the widget
#
#   Actions:
#     Creates a Offset Enum UI contained in a frame widget
#
#   Returns:
#     The name of the unpacked frame widget
#
proc CreateSLIDEOffsetEnumFrame { parent name var text } {
    set offsetList [list SLF_OFFSET_ANGULAR SLF_OFFSET_ROUNDED SLF_OFFSET_PERFORATED]
    set enumList [CreateSLIDEEnumList $offsetList]

    return [CreateComboBoxFrame $parent $name $var $text $enumList]
}

######################################################################
# CreateSLIDETerrainField
#   Parameters:
#     this     - name of the associative array
#
#   Actions:
#     Creates a field for the terrain enumerated type
#
#   Returns:
#     Initial value for the field
#
proc CreateSLIDETerrainField { this } {
    global $this
    #global SLF_TERRAIN_DELAUNAY_INCREMENTAL
    #global SLF_TERRAIN_DELAUNAY_CONVEX_HULL
    global SLF_TERRAIN_DELAUNAY_DIVIDE_AND_CONQUER_ALTERNATING

    #set [subst $this](type) $SLF_TERRAIN_DELAUNAY_INCREMENTAL
    #set [subst $this](type) $SLF_TERRAIN_DELAUNAY_CONVEX_HULL
    set [subst $this](type) $SLF_TERRAIN_DELAUNAY_DIVIDE_AND_CONQUER_ALTERNATING

    return [subst $[subst $this](type)]
}

######################################################################
# CreateSLIDETerrainEnumFrame
#   Parameters:
#     parent   - parent widget name
#     name     - name of the widget
#     var      - name of the global variable which will be modified
#     text     - text name to be displayed on the widget
#
#   Actions:
#     Creates a Terrain Enum UI contained in a frame widget
#
#   Returns:
#     The name of the unpacked frame widget
#
proc CreateSLIDETerrainEnumFrame { parent name var text } {
    set terrainList [list SLF_TERRAIN_DELAUNAY_INCREMENTAL SLF_TERRAIN_DELAUNAY_INCREMENTAL_SLOW SLF_TERRAIN_DELAUNAY_INCREMENTAL_NO_BOUND SLF_TERRAIN_DELAUNAY_DIVIDE_AND_CONQUER SLF_TERRAIN_DELAUNAY_DIVIDE_AND_CONQUER_ALTERNATING SLF_TERRAIN_DELAUNAY_CONVEX_HULL ]
    set enumList [CreateSLIDEEnumList $terrainList]

    return [CreateComboBoxFrame $parent $name $var $text $enumList]
}

######################################################################
# CreateSLIDEVoronoiField
#   Parameters:
#     this     - name of the associative array
#
#   Actions:
#     Creates a field for the Voronoi enumerated type
#
#   Returns:
#     Initial value for the field
#
proc CreateSLIDEVoronoiField { this } {
    global $this
    #global SLF_VORONOI_DIVIDE_AND_CONQUER
    global SLF_VORONOI_OFF

    set [subst $this](voronoi) $SLF_VORONOI_OFF

    return [subst $[subst $this](voronoi)]
}

######################################################################
# CreateSLIDEVoronoiEnumFrame
#   Parameters:
#     parent   - parent widget name
#     name     - name of the widget
#     var      - name of the global variable which will be modified
#     text     - text name to be displayed on the widget
#
#   Actions:
#     Creates a Voronoi Enum UI contained in a frame widget
#
#   Returns:
#     The name of the unpacked frame widget
#
proc CreateSLIDEVoronoiEnumFrame { parent name var text } {
    set voronoiList [list SLF_VORONOI_OFF SLF_VORONOI_DELAUNAY SLF_VORONOI_DIVIDE_AND_CONQUER SLF_VORONOI_DIVIDE_AND_CONQUER_ALTERNATING SLF_VORONOI_CONES SLF_VORONOI_PLANES ]
    set enumList [CreateSLIDEEnumList $voronoiList]

    return [CreateComboBoxFrame $parent $name $var $text $enumList]
}

######################################################################
# CreateSLIDEAccuracyField
#   Parameters:
#     this     - name of the associative array
#
#   Actions:
#     Creates a field for the accuracy enumerated type
#
#   Returns:
#     Initial value for the field
#
proc CreateSLIDEAccuracyField { this } {
    global $this
    global SLF_FLOAT

    set [subst $this](accuracy) $SLF_FLOAT

    return [subst $[subst $this](accuracy)]
}

######################################################################
# CreateSLIDEAccuracyEnumFrame
#   Parameters:
#     parent   - parent widget name
#     name     - name of the widget
#     var      - name of the global variable which will be modified
#     text     - text name to be displayed on the widget
#
#   Actions:
#     Creates a Accuracy Enum UI contained in a frame widget
#
#   Returns:
#     The name of the unpacked frame widget
#
proc CreateSLIDEAccuracyEnumFrame { parent name var text } {
    set accuracyList [list SLF_FLOAT_NAIVE SLF_FLOAT SLF_EXACT ]
    set enumList [CreateSLIDEEnumList $accuracyList]

    return [CreateComboBoxFrame $parent $name $var $text $enumList]
}

######################################################################
# CreateSLIDESquareObject
#   Parameters:
#     name     - name of the square
#
#   Actions:
#     Creates an associative array with initialized fields for a square
#
#   Returns:
#     The name of the associative array
#
proc CreateSLIDESquareObject { name } {
    set this $name

    global $this

    # texture
    CreateSLIDETextureFields $this

    # solid
    CreateSLIDESolidField $this

    # shading
    CreateSLIDEShadingField $this

    # lod
    CreateSLIDELODField $this

    return $this
}

######################################################################
# CreateSLIDESquareUI
#   Parameters:
#     parent   - parent widget name
#     name     - name of the square
#
#   Actions:
#     Creates a square UI contained in a frame widget
#
#   Returns:
#     The name of the unpacked frame widget
#
proc CreateSLIDESquareUI { parent name } {
    set root $parent.f$name
    set this $name

    global $this

    frame $root

    ##################################################
    # Enums Frame
    frame $root.fEnums
    
    ##############################
    # lod
    set widget [CreateSLIDELODEnumFrame $root.fEnums lod \
	    [subst $this](lod) "LOD"]
    pack $widget -side left -fill y

    ##############################
    # shading
    set widget [CreateSLIDEShadingEnumFrame $root.fEnums shading \
	    [subst $this](shading) "Shading"]
    pack $widget -side left -fill y

    ##############################
    # solid
    set widget [CreateSLIDESolidEnumFrame $root.fEnums solid \
	    [subst $this](solid) "Solid"]
    pack $widget -side left -fill y

    pack $root.fEnums -side top -fill x

    ##################################################
    # Scales Frame
    frame $root.fScales
    
    ##############################
    # texture
    for { set i 0 } { $i < 4 } { incr i } {
	frame $root.fScales.scaleTexture_[subst $i]
	for { set j 0 } { $j < 2 } { incr j } {
	    set widget [CreateScale $root.fScales.scaleTexture_[subst $i] \
		    scaleTexture_[subst $i]_[subst $j] \
		    [subst $this](texture_[subst $i]_[subst $j]) \
		    "Texture($i)($j)" \
		    [subst $[subst $this](texture_[subst $i]_[subst $j]_min)] \
		    [subst $[subst $this](texture_[subst $i]_[subst $j]_max)] \
		    [subst $[subst $this](texture_[subst $i]_[subst $j]_resolution)]]
	    pack $widget -side left -fill y
	}
	pack $root.fScales.scaleTexture_[subst $i] -side top -fill x
    }
    pack $root.fScales -side top -fill x

    return $root
}

######################################################################
# CreateSLIDEObjectObject
#   Parameters:
#     name     - name of the object
#
#   Actions:
#     Creates an associative array with initialized fields for a object
#
#   Returns:
#     The name of the associative array
#
proc CreateSLIDEObjectObject { name } {
    set this $name

    global $this

    # solid
    CreateSLIDESolidField $this

    # shading
    CreateSLIDEShadingField $this

    # lod
    CreateSLIDELODField $this

    return $this
}

######################################################################
# CreateSLIDEObjectUI
#   Parameters:
#     parent   - parent widget name
#     name     - name of the object
#
#   Actions:
#     Creates a object UI contained in a frame widget
#
#   Returns:
#     The name of the unpacked frame widget
#
proc CreateSLIDEObjectUI { parent name } {
    set root $parent.f$name
    set this $name

    global $this

    frame $root

    ##################################################
    # Enums Frame
    frame $root.fEnums
    
    ##############################
    # lod
    set widget [CreateSLIDELODEnumFrame $root.fEnums lod \
	    [subst $this](lod) "LOD"]
    pack $widget -side left -fill y

    ##############################
    # shading
    set widget [CreateSLIDEShadingEnumFrame $root.fEnums shading \
	    [subst $this](shading) "Shading"]
    pack $widget -side left -fill y

    ##############################
    # solid
    set widget [CreateSLIDESolidEnumFrame $root.fEnums solid \
	    [subst $this](solid) "Solid"]
    pack $widget -side left -fill y

    pack $root.fEnums -side top -fill x

    ##################################################
    # Scales Frame
    #frame $root.fScales
    
    #pack $root.fScales -side top -fill x

    return $root
}

######################################################################
# CreateSLIDEGroupObject
#   Parameters:
#     name     - name of the group
#
#   Actions:
#     Creates an associative array with initialized fields for a group
#
#   Returns:
#     The name of the associative array
#
proc CreateSLIDEGroupObject { name } {
    set this $name

    global $this

    # shading
    CreateSLIDEShadingField $this

    # lod
    CreateSLIDELODField $this

    return $this
}

######################################################################
# CreateSLIDEGroupUI
#   Parameters:
#     parent   - parent widget name
#     name     - name of the group
#
#   Actions:
#     Creates a group UI contained in a frame widget
#
#   Returns:
#     The name of the unpacked frame widget
#
proc CreateSLIDEGroupUI { parent name } {
    set root $parent.f$name
    set this $name

    global $this

    frame $root

    ##################################################
    # Enums Frame
    frame $root.fEnums
    
    ##############################
    # lod
    set widget [CreateSLIDELODEnumFrame $root.fEnums lod \
	    [subst $this](lod) "LOD"]
    pack $widget -side left -fill y

    ##############################
    # shading
    set widget [CreateSLIDEShadingEnumFrame $root.fEnums shading \
	    [subst $this](shading) "Shading"]
    pack $widget -side left -fill y

    pack $root.fEnums -side top -fill x

    ##################################################
    # Scales Frame
    #frame $root.fScales
    
    #pack $root.fScales -side top -fill x

    return $root
}

######################################################################
# CreateSLIDESubdivisionObject
#   Parameters:
#     name     - name of the subdivision
#
#   Actions:
#     Creates an associative array with initialized fields for a subdivision
#
#   Returns:
#     The name of the associative array
#
proc CreateSLIDESubdivisionObject { name } {
    set this $name

    global $this

    # shading
    CreateSLIDEShadingField $this

    # lod
    CreateSLIDELODField $this

    # subdivision
    CreateSLIDESubdivisionField $this

    # error metric
    CreateSLIDEErrorMetricField $this

    # use display lists
    set [subst $this](usedisplaylists) 0

    # draw input control mesh
    set [subst $this](drawcontrols) 0

    # draw current control mesh
    set [subst $this](drawcurrent) 0

    # draw vertices
    set [subst $this](drawvertices) 0

    # subdivisions
    set [subst $this](subdivisions) 0
    set [subst $this](subdivisions_min) 0
    set [subst $this](subdivisions_max) 10
    set [subst $this](subdivisions_resolution) 1

    # weightededges
    set [subst $this](weightededges) 0
    set [subst $this](weightededges_min) 0
    set [subst $this](weightededges_max) 180
    set [subst $this](weightededges_resolution) 1

    # weight
    set [subst $this](weight) 1.0
    set [subst $this](weight_min) 0.01
    set [subst $this](weight_max) 5.0
    set [subst $this](weight_resolution) 0.01

    # a == override compiled weights
    set [subst $this](a) 0
    set [subst $this](a_min) 0
    set [subst $this](a_max) 1
    set [subst $this](a_resolution) 1

    # b == which compiled model
    set [subst $this](b) 0
    set [subst $this](b_min) 0
    set [subst $this](b_max) 5
    set [subst $this](b_resolution) 1

    ##################################################
    # tolerance
    set [subst $this](tolerance) 1.0
    set [subst $this](tolerance_min) 0.001
    set [subst $this](tolerance_max) 1.0
    set [subst $this](tolerance_resolution) 0.001

    set [subst $this](tolerance) 0.4
    set [subst $this](tolerance_min) 0.0001
    set [subst $this](tolerance_max) 0.4
    set [subst $this](tolerance_resolution) 0.0001
    # tolerance
    ##################################################

    # facets max
    set [subst $this](facetsmax) 0
    set [subst $this](facetsmax_min) 0
    #set [subst $this](facetsmax_max) 35000
    #set [subst $this](facetsmax_max) 30000
    set [subst $this](facetsmax_max) 12000
    #set [subst $this](facetsmax_max) 6000
    set [subst $this](facetsmax_resolution) 1

    return $this
}

######################################################################
# CreateSLIDESubdivisionUI
#   Parameters:
#     parent   - parent widget name
#     name     - name of the subdivision
#
#   Actions:
#     Creates a subdivision UI contained in a frame widget
#
#   Returns:
#     The name of the unpacked frame widget
#
proc CreateSLIDESubdivisionUI { parent name } {
    set root $parent.f$name
    set this $name

    global $this

    frame $root

    ##################################################
    # Enums Frame
    frame $root.fEnums
    
    ##############################
    # lod
    set widget [CreateSLIDELODEnumFrame $root.fEnums lod \
	    [subst $this](lod) "LOD"]
    pack $widget -side left -fill y

    ##############################
    # shading
    set widget [CreateSLIDEShadingEnumFrame $root.fEnums shading \
	    [subst $this](shading) "Shading"]
    pack $widget -side left -fill y

    ##############################
    # use display lists
    set widget [CreateSLIDEOnOffEnumFrame $root.fEnums usedisplaylists \
	    [subst $this](usedisplaylists) "Use Disp Lists"]
    pack $widget -side left -fill y

    ##############################
    # draw input control mesh
    set widget [CreateSLIDEOnOffEnumFrame $root.fEnums drawcontrols \
	    [subst $this](drawcontrols) "Draw Controls"]
    pack $widget -side left -fill y

    ##############################
    # draw current control mesh
    set widget [CreateSLIDEOnOffEnumFrame $root.fEnums drawcurrent \
	    [subst $this](drawcurrent) "Draw Current"]
    pack $widget -side left -fill y

    ##############################
    # draw vertices
    set widget [CreateSLIDEOnOffEnumFrame $root.fEnums drawvertices \
	    [subst $this](drawvertices) "Draw Vertices"]
    pack $widget -side left -fill y

    pack $root.fEnums -side top -fill x

    ##################################################
    # Enums Frame
    frame $root.fEnumSubdivision
    
    ##############################
    # subdivision type
    set widget [CreateSLIDESubdivisionEnumFrame $root.fEnumSubdivision subdivision \
	    [subst $this](type) "Subdivision"]
    pack $widget -side left -fill y

    pack $root.fEnumSubdivision -side top -fill x

    ##################################################
    # Error Metric Enums Frame
    frame $root.fEnumErrorMetric
    
    ##############################
    # error metric
    set widget [CreateSLIDEErrorMetricEnumFrame $root.fEnumErrorMetric errormetric \
	    [subst $this](errormetric) "Error Metric"]
    pack $widget -side left -fill y

    pack $root.fEnumErrorMetric -side top -fill x

    ##################################################
    # Scales Frame
    frame $root.fScales
    
    ##############################
    # subdivisions
    set widget [CreateScale $root.fScales scaleSubdivisions \
	    [subst $this](subdivisions) "Subdivisions" \
	    [subst $[subst $this](subdivisions_min)] \
	    [subst $[subst $this](subdivisions_max)] \
	    [subst $[subst $this](subdivisions_resolution)]]
    pack $widget -side top -fill x

    ##############################
    # tolerance
    set widget [CreateScale $root.fScales scaleTolerance \
	    [subst $this](tolerance) "Tolerance" \
	    [subst $[subst $this](tolerance_min)] \
	    [subst $[subst $this](tolerance_max)] \
	    [subst $[subst $this](tolerance_resolution)]]
    pack $widget -side top -fill x

    ##############################
    # facets max
    set widget [CreateScale $root.fScales scaleFacetsMax \
	    [subst $this](facetsmax) "Facets Max" \
	    [subst $[subst $this](facetsmax_min)] \
	    [subst $[subst $this](facetsmax_max)] \
	    [subst $[subst $this](facetsmax_resolution)]]
    pack $widget -side top -fill x

    ##############################
    # weightededges
    set widget [CreateScale $root.fScales scaleWeightedEdges \
	    [subst $this](weightededges) "Weighted Edges" \
	    [subst $[subst $this](weightededges_min)] \
	    [subst $[subst $this](weightededges_max)] \
	    [subst $[subst $this](weightededges_resolution)]]
    pack $widget -side top -fill x

    ##############################
    # weight
    set widget [CreateScale $root.fScales scaleWeight \
	    [subst $this](weight) "Weight" \
	    [subst $[subst $this](weight_min)] \
	    [subst $[subst $this](weight_max)] \
	    [subst $[subst $this](weight_resolution)]]
    pack $widget -side top -fill x

    ##############################
    # a == Override Compiled Weights
    set widget [CreateScale $root.fScales scaleA \
	    [subst $this](a) "Override compiled weights?" \
	    [subst $[subst $this](a_min)] \
	    [subst $[subst $this](a_max)] \
	    [subst $[subst $this](a_resolution)]]
    pack $widget -side top -fill x

    ##############################
    # b == Compiled Model
    set widget [CreateScale $root.fScales scaleB \
	    [subst $this](b) "Choose compiled model" \
	    [subst $[subst $this](b_min)] \
	    [subst $[subst $this](b_max)] \
	    [subst $[subst $this](b_resolution)]]
    pack $widget -side top -fill x

    pack $root.fScales -side top -fill x

    return $root
}

######################################################################
# CreateSLIDEOffsetObject
#   Parameters:
#     name     - name of the offset
#
#   Actions:
#     Creates an associative array with initialized fields for a offset
#
#   Returns:
#     The name of the associative array
#
proc CreateSLIDEOffsetObject { name } {
    set this $name

    global $this

    # shading
    CreateSLIDEShadingField $this

    # lod
    CreateSLIDELODField $this

    # offset type
    CreateSLIDEOffsetField $this

    # draw mesh
    set [subst $this](drawcontrols) 0

    # height
    set [subst $this](height) 0.0
    set [subst $this](height_min) -5.0
    set [subst $this](height_max) 5.0
    set [subst $this](height_resolution) 0.01

    # width
    set [subst $this](width) 0.0
    set [subst $this](width_min) -5.0
    set [subst $this](width_max) 5.0
    set [subst $this](width_resolution) 0.01

    return $this
}

######################################################################
# CreateSLIDEOffsetUI
#   Parameters:
#     parent   - parent widget name
#     name     - name of the offset
#
#   Actions:
#     Creates a offset UI contained in a frame widget
#
#   Returns:
#     The name of the unpacked frame widget
#
proc CreateSLIDEOffsetUI { parent name } {
    set root $parent.f$name
    set this $name

    global $this

    frame $root

    ##################################################
    # Enums Frame
    frame $root.fEnums
    
    ##############################
    # lod
    set widget [CreateSLIDELODEnumFrame $root.fEnums lod \
	    [subst $this](lod) "LOD"]
    pack $widget -side left -fill y

    ##############################
    # shading
    set widget [CreateSLIDEShadingEnumFrame $root.fEnums shading \
	    [subst $this](shading) "Shading"]
    pack $widget -side left -fill y

    ##############################
    # offset type
    set widget [CreateSLIDEOffsetEnumFrame $root.fEnums offset \
	    [subst $this](type) "Offset"]
    pack $widget -side left -fill y

    ##############################
    # draw mesh type
    set widget [CreateSLIDEOnOffEnumFrame $root.fEnums drawcontrols \
	    [subst $this](drawcontrols) "Draw Controls"]
    pack $widget -side left -fill y

    pack $root.fEnums -side top -fill x

    ##################################################
    # Scales Frame
    frame $root.fScales
    
    # height
    set widget [CreateScale $root.fScales scaleHeight \
	    [subst $this](height) "Height" \
	    [subst $[subst $this](height_min)] \
	    [subst $[subst $this](height_max)] \
	    [subst $[subst $this](height_resolution)]]
    pack $widget -side top -fill x

    # width
    set widget [CreateScale $root.fScales scaleWidth \
	    [subst $this](width) "Strut Width" \
	    [subst $[subst $this](width_min)] \
	    [subst $[subst $this](width_max)] \
	    [subst $[subst $this](width_resolution)]]
    pack $widget -side top -fill x

    pack $root.fScales -side top -fill x

    return $root
}

######################################################################
# CreateSLIDEDualObject
#   Parameters:
#     name     - name of the dual
#
#   Actions:
#     Creates an associative array with initialized fields for a dual
#
#   Returns:
#     The name of the associative array
#
proc CreateSLIDEDualObject { name } {
    set this $name

    global $this

    # shading
    CreateSLIDEShadingField $this

    # lod
    CreateSLIDELODField $this

    # draw mesh
    set [subst $this](drawcontrols) 0

    return $this
}

######################################################################
# CreateSLIDEDualUI
#   Parameters:
#     parent   - parent widget name
#     name     - name of the dual
#
#   Actions:
#     Creates a dual UI contained in a frame widget
#
#   Returns:
#     The name of the unpacked frame widget
#
proc CreateSLIDEDualUI { parent name } {
    set root $parent.f$name
    set this $name

    global $this

    frame $root

    ##################################################
    # Enums Frame
    frame $root.fEnums
    
    ##############################
    # lod
    set widget [CreateSLIDELODEnumFrame $root.fEnums lod \
	    [subst $this](lod) "LOD"]
    pack $widget -side left -fill y

    ##############################
    # shading
    set widget [CreateSLIDEShadingEnumFrame $root.fEnums shading \
	    [subst $this](shading) "Shading"]
    pack $widget -side left -fill y

    ##############################
    # draw mesh type
    set widget [CreateSLIDEOnOffEnumFrame $root.fEnums drawcontrols \
	    [subst $this](drawcontrols) "Draw Controls"]
    pack $widget -side left -fill y

    pack $root.fEnums -side top -fill x

    return $root
}

######################################################################
# CreateSLIDEMirrorObject
#   Parameters:
#     name     - name of the mirror
#
#   Actions:
#     Creates an associative array with initialized fields for a mirror
#
#   Returns:
#     The name of the associative array
#
proc CreateSLIDEMirrorObject { name } {
    set this $name

    global $this

    # shading
    CreateSLIDEShadingField $this

    # lod
    CreateSLIDELODField $this

    # draw mesh
    set [subst $this](drawcontrols) 0

    return $this
}

######################################################################
# CreateSLIDEMirrorUI
#   Parameters:
#     parent   - parent widget name
#     name     - name of the mirror
#
#   Actions:
#     Creates a mirror UI contained in a frame widget
#
#   Returns:
#     The name of the unpacked frame widget
#
proc CreateSLIDEMirrorUI { parent name } {
    set root $parent.f$name
    set this $name

    global $this

    frame $root

    ##################################################
    # Enums Frame
    frame $root.fEnums
    
    ##############################
    # lod
    set widget [CreateSLIDELODEnumFrame $root.fEnums lod \
	    [subst $this](lod) "LOD"]
    pack $widget -side left -fill y

    ##############################
    # shading
    set widget [CreateSLIDEShadingEnumFrame $root.fEnums shading \
	    [subst $this](shading) "Shading"]
    pack $widget -side left -fill y

    ##############################
    # draw mesh type
    set widget [CreateSLIDEOnOffEnumFrame $root.fEnums drawcontrols \
	    [subst $this](drawcontrols) "Draw Controls"]
    pack $widget -side left -fill y

    pack $root.fEnums -side top -fill x

    return $root
}

######################################################################
# CreateSLIDEGeodesicObject
#   Parameters:
#     name     - name of the geodesic
#
#   Actions:
#     Creates an associative array with initialized fields for a geodesic
#
#   Returns:
#     The name of the associative array
#
proc CreateSLIDEGeodesicObject { name } {
    set this $name

    global $this

    # shading
    CreateSLIDEShadingField $this

    # lod
    CreateSLIDELODField $this

    # draw mesh
    set [subst $this](drawcontrols) 0

    # theta
    set [subst $this](theta) 0
    set [subst $this](theta_resolution) 1
    set [subst $this](theta_min) 0
    set [subst $this](theta_max) 360

    # intersections
    set [subst $this](intersections) 1
    set [subst $this](intersections_resolution) 1
    set [subst $this](intersections_min) 0
    set [subst $this](intersections_max) 1000

    return $this
}

######################################################################
# CreateSLIDEGeodesicUI
#   Parameters:
#     parent   - parent widget name
#     name     - name of the geodesic
#
#   Actions:
#     Creates a geodesic UI contained in a frame widget
#
#   Returns:
#     The name of the unpacked frame widget
#
proc CreateSLIDEGeodesicUI { parent name } {
    set root $parent.f$name
    set this $name

    global $this

    frame $root

    ##################################################
    # Enums Frame
    frame $root.fEnums
    
    ##############################
    # lod
    set widget [CreateSLIDELODEnumFrame $root.fEnums lod \
	    [subst $this](lod) "LOD"]
    pack $widget -side left -fill y

    ##############################
    # shading
    set widget [CreateSLIDEShadingEnumFrame $root.fEnums shading \
	    [subst $this](shading) "Shading"]
    pack $widget -side left -fill y

    ##############################
    # draw mesh type
    set widget [CreateSLIDEOnOffEnumFrame $root.fEnums drawcontrols \
	    [subst $this](drawcontrols) "Draw Controls"]
    pack $widget -side left -fill y

    pack $root.fEnums -side top -fill x

    ##################################################
    # Scales Frame
    frame $root.fScales
    
    ##############################
    # theta
    set widget [CreateScale $root.fScales scaleTheta \
	    [subst $this](theta) "Theta" \
	    [subst $[subst $this](theta_min)] \
	    [subst $[subst $this](theta_max)] \
	    [subst $[subst $this](theta_resolution)]]
    pack $widget -side top -fill x

    ##############################
    # intersections
    set widget [CreateScale $root.fScales scaleIntersections \
	    [subst $this](intersections) "Intersections" \
	    [subst $[subst $this](intersections_min)] \
	    [subst $[subst $this](intersections_max)] \
	    [subst $[subst $this](intersections_resolution)]]
    pack $widget -side top -fill x

    pack $root.fScales -side top -fill x

    return $root
}

######################################################################
# CreateSLIDEInstanceObject
#   Parameters:
#     name     - name of the instance
#
#   Actions:
#     Creates an associative array with initialized fields for a instance
#
#   Returns:
#     The name of the associative array
#
proc CreateSLIDEInstanceObject { name } {
    set this $name

    global $this

    # shading
    CreateSLIDEShadingField $this

    # lod
    CreateSLIDELODField $this

    return $this
}

######################################################################
# CreateSLIDEInstanceUI
#   Parameters:
#     parent   - parent widget name
#     name     - name of the instance
#
#   Actions:
#     Creates a instance UI contained in a frame widget
#
#   Returns:
#     The name of the unpacked frame widget
#
proc CreateSLIDEInstanceUI { parent name } {
    set root $parent.f$name
    set this $name

    global $this

    frame $root

    ##################################################
    # Enums Frame
    frame $root.fEnums
    
    ##############################
    # lod
    set widget [CreateSLIDELODEnumFrame $root.fEnums lod \
	    [subst $this](lod) "LOD"]
    pack $widget -side left -fill y

    ##############################
    # shading
    set widget [CreateSLIDEShadingEnumFrame $root.fEnums shading \
	    [subst $this](shading) "Shading"]
    pack $widget -side left -fill y

    ##################################################
    # Scales Frame
    #frame $root.fScales
    
    #pack $root.fScales -side top -fill x

    return $root
}

######################################################################
# CreateSLIDETerrainObject
#   Parameters:
#     name     - name of the terrain
#
#   Actions:
#     Creates an associative array with initialized fields for a terrain
#
#   Returns:
#     The name of the associative array
#
proc CreateSLIDETerrainObject { name } {
    set this $name

    global $this

    # texture
    #CreateSLIDETextureFields $this

    # terrain type
    CreateSLIDETerrainField $this

    # voronoi type
    CreateSLIDEVoronoiField $this

    # accuracy type
    CreateSLIDEAccuracyField $this

    # randomize type
    set [subst $this](random) 0

    # drawdelaunay type
    set [subst $this](drawdelaunay) 1

    # drawvertices type
    set [subst $this](drawvertices) 0

    # solid
    CreateSLIDESolidField $this

    # shading
    CreateSLIDEShadingField $this

    # lod
    CreateSLIDELODField $this

    return $this
}

######################################################################
# CreateSLIDETerrainUI
#   Parameters:
#     parent   - parent widget name
#     name     - name of the terrain
#
#   Actions:
#     Creates a terrain UI contained in a frame widget
#
#   Returns:
#     The name of the unpacked frame widget
#
proc CreateSLIDETerrainUI { parent name } {
    set root $parent.f$name
    set this $name

    global $this

    frame $root

    ##################################################
    # Enums Frame
    frame $root.fEnums

    frame $root.fEnums.f1
    frame $root.fEnums.f2
    frame $root.fEnums.f3
    
    ##############################
    # lod
    set widget [CreateSLIDELODEnumFrame $root.fEnums.f1 lod \
	    [subst $this](lod) "LOD"]
    pack $widget -side left -fill y

    ##############################
    # shading
    set widget [CreateSLIDEShadingEnumFrame $root.fEnums.f1 shading \
	    [subst $this](shading) "Shading"]
    pack $widget -side left -fill y

    ##############################
    # solid
    set widget [CreateSLIDESolidEnumFrame $root.fEnums.f1 solid \
	    [subst $this](solid) "Solid"]
    pack $widget -side left -fill y

    ##############################
    # terrain type
    set widget [CreateSLIDETerrainEnumFrame $root.fEnums.f2 type \
	    [subst $this](type) "Type"]
    pack $widget -side left -fill y

    ##############################
    # voronoi 
    set widget [CreateSLIDEVoronoiEnumFrame $root.fEnums.f3 voronoi \
	    [subst $this](voronoi) "Voronoi"]
    pack $widget -side left -fill y

    ##############################
    # accuracy 
    set widget [CreateSLIDEAccuracyEnumFrame $root.fEnums.f1 accuracy \
	    [subst $this](accuracy) "Accuracy"]
    pack $widget -side left -fill y

    ##############################
    # random
    set widget [CreateSLIDEOnOffEnumFrame $root.fEnums.f1 random \
	    [subst $this](random) "Random"]
    pack $widget -side left -fill y

    ##############################
    # drawdelaunay
    set widget [CreateSLIDEOnOffEnumFrame $root.fEnums.f2 drawdelaunay \
	    [subst $this](drawdelaunay) "Draw Delaunay"]
    pack $widget -side left -fill y

    ##############################
    # drawvertices
    set widget [CreateSLIDEOnOffEnumFrame $root.fEnums.f3 drawvertices \
	    [subst $this](drawvertices) "Draw Vertices"]
    pack $widget -side left -fill y

    pack $root.fEnums.f1 $root.fEnums.f2 $root.fEnums.f3 -side top -fill x
    pack $root.fEnums -side top -fill x

    return $root
}

######################################################################
# CreateSLIDEGeneralfaceObject
#   Parameters:
#     name     - name of the generalface
#
#   Actions:
#     Creates an associative array with initialized fields for a generalface
#
#   Returns:
#     The name of the associative array
#
proc CreateSLIDEGeneralFaceObject { name } {
    set this $name

    global $this

    # texture
    #CreateSLIDETextureFields $this

    # generalface type
    CreateSLIDETerrainField $this

    # voronoi type
    CreateSLIDEVoronoiField $this

    # accuracy type
    CreateSLIDEAccuracyField $this
    global SLF_EXACT
    set [subst $this](accuracy) $SLF_EXACT

    # randomize type
    set [subst $this](random) 0

    # milling contour
    set [subst $this](millingcontour) 0

    # clean up contour
    set [subst $this](cleanupcontour) 0

    # offsets
    set [subst $this](offsets) 0
    set [subst $this](offsets_resolution) 1
    set [subst $this](offsets_min) 0
    set [subst $this](offsets_max) 10

    # offsetmin
    set [subst $this](offsetmin) 0.0
    set [subst $this](offsetmin_resolution) 0.001
    set [subst $this](offsetmin_min) -10.0
    set [subst $this](offsetmin_max) 3.0

    # offsetmax
    set [subst $this](offsetmax) 0.0
    set [subst $this](offsetmax_resolution) 0.001
    set [subst $this](offsetmax_min) -10.0
    set [subst $this](offsetmax_max) 3.0

    # individualvoronoimin
    set [subst $this](individualvoronoimin) 0
    set [subst $this](individualvoronoimin_resolution) 1
    set [subst $this](individualvoronoimin_min) 0
    set [subst $this](individualvoronoimin_max) 10

    # individualvoronoimax
    set [subst $this](individualvoronoimax) 0
    set [subst $this](individualvoronoimax_resolution) 1
    set [subst $this](individualvoronoimax_min) 0
    set [subst $this](individualvoronoimax_max) 10

    # subdivisions
    set [subst $this](subdivisions) 0
    set [subst $this](subdivisions_resolution) 1
    set [subst $this](subdivisions_min) 0
    set [subst $this](subdivisions_max) 10

    # tolerance
    set [subst $this](tolerance) 0.001
    set [subst $this](tolerance_resolution) 0.001
    set [subst $this](tolerance_min) 0.0
    set [subst $this](tolerance_max) 1.0

    # drawcontours type
    set [subst $this](drawcontours) 0

    # drawvertices type
    set [subst $this](drawvertices) 0

    # drawCDT type
    set [subst $this](drawCDT) 1

    # drawindividualvoronoi type
    set [subst $this](drawindividualvoronoi) 0

    # drawvoronoi type
    set [subst $this](drawvoronoi) 0

    # drawheight type
    set [subst $this](drawheight) 0

    # drawoffset type
    set [subst $this](drawoffset) 0

    # draw milling contour
    set [subst $this](drawmillingcontour) 0

    # solid
    CreateSLIDESolidField $this

    # shading
    CreateSLIDEShadingField $this

    # lod
    CreateSLIDELODField $this

    return $this
}

######################################################################
# CreateSLIDEGeneralfaceUI
#   Parameters:
#     parent   - parent widget name
#     name     - name of the generalface
#
#   Actions:
#     Creates a generalface UI contained in a frame widget
#
#   Returns:
#     The name of the unpacked frame widget
#
proc CreateSLIDEGeneralFaceUI { parent name } {
    set root $parent.f$name
    set this $name

    global $this

    frame $root

    ##################################################
    # Enums Frame
    frame $root.fEnums

    frame $root.fEnums.f1
    frame $root.fEnums.f2
    frame $root.fEnums.f3
    
    ##############################
    # lod
    set widget [CreateSLIDELODEnumFrame $root.fEnums.f1 lod \
	    [subst $this](lod) "LOD"]
    pack $widget -side left -fill y

    ##############################
    # shading
    set widget [CreateSLIDEShadingEnumFrame $root.fEnums.f1 shading \
	    [subst $this](shading) "Shading"]
    pack $widget -side left -fill y

    ##############################
    # solid
    set widget [CreateSLIDESolidEnumFrame $root.fEnums.f1 solid \
	    [subst $this](solid) "Solid"]
    pack $widget -side left -fill y

    ##############################
    # terrain type
    set widget [CreateSLIDETerrainEnumFrame $root.fEnums.f2 type \
	    [subst $this](type) "Type"]
    pack $widget -side left -fill y

    ##############################
    # voronoi 
    set widget [CreateSLIDEVoronoiEnumFrame $root.fEnums.f3 voronoi \
	    [subst $this](voronoi) "Voronoi"]
    pack $widget -side left -fill y

    ##############################
    # milling contour
    set widget [CreateSLIDEOnOffEnumFrame $root.fEnums.f1 millingcontour \
	    [subst $this](millingcontour) "Milling Contour"]
    pack $widget -side left -fill y

    ##############################
    # clean up contour
    set widget [CreateSLIDEOnOffEnumFrame $root.fEnums.f1 cleanupcontour \
	    [subst $this](cleanupcontour) "Clean Up Contour"]
    pack $widget -side left -fill y

    ##############################
    # accuracy 
    set widget [CreateSLIDEAccuracyEnumFrame $root.fEnums.f1 accuracy \
	    [subst $this](accuracy) "Accuracy"]
    pack $widget -side left -fill y

    ##############################
    # random
    set widget [CreateSLIDEOnOffEnumFrame $root.fEnums.f1 random \
	    [subst $this](random) "Random"]
    pack $widget -side left -fill y

    ##############################
    # drawCDT
    set widget [CreateSLIDEOnOffEnumFrame $root.fEnums.f2 drawCDT \
	    [subst $this](drawCDT) "Draw CDT"]
    pack $widget -side left -fill y

    ##############################
    # drawcontours
    set widget [CreateSLIDEOnOffEnumFrame $root.fEnums.f2 drawcontours \
	    [subst $this](drawcontours) "Draw Contours"]
    pack $widget -side left -fill y

    ##############################
    # drawvertices
    set widget [CreateSLIDEOnOffEnumFrame $root.fEnums.f2 drawvertices \
	    [subst $this](drawvertices) "Draw Vertices"]
    pack $widget -side left -fill y

    ##############################
    # drawindividualvoronoi
    set widget [CreateSLIDEOnOffEnumFrame $root.fEnums.f3 drawindividualvoronoi \
	    [subst $this](drawindividualvoronoi) "Draw Individual Voronoi"]
    pack $widget -side left -fill y

    ##############################
    # drawvoronoi
    set widget [CreateSLIDEOnOffEnumFrame $root.fEnums.f3 drawvoronoi \
	    [subst $this](drawvoronoi) "Draw Voronoi"]
    pack $widget -side left -fill y

    ##############################
    # drawoffset
    set widget [CreateSLIDEOnOffEnumFrame $root.fEnums.f3 drawoffset \
	    [subst $this](drawoffset) "Draw Offset"]
    pack $widget -side left -fill y

    ##############################
    # drawmillingcontour
    set widget [CreateSLIDEOnOffEnumFrame $root.fEnums.f3 drawmillingcontour \
	    [subst $this](drawmillingcontour) "Draw Milling Contour"]
    pack $widget -side left -fill y

    ##############################
    # drawheight
    set widget [CreateSLIDEOnOffEnumFrame $root.fEnums.f3 drawheight \
	    [subst $this](drawheight) "Draw Height"]
    pack $widget -side left -fill y

    pack $root.fEnums.f1 $root.fEnums.f2 $root.fEnums.f3 -side top -fill x
    pack $root.fEnums -side top -fill x

    ##################################################
    # Scales Frame
    frame $root.fScales
    
    ##############################
    # offsets
    set widget [CreateScale $root.fScales scaleOffsets \
	    [subst $this](offsets) "Offsets" \
	    [subst $[subst $this](offsets_min)] \
	    [subst $[subst $this](offsets_max)] \
	    [subst $[subst $this](offsets_resolution)]]
    pack $widget -side top -fill x

    ##############################
    # offsetmin
    set widget [CreateScale $root.fScales scaleOffsetMin \
	    [subst $this](offsetmin) "Offset Min" \
	    [subst $[subst $this](offsetmin_min)] \
	    [subst $[subst $this](offsetmin_max)] \
	    [subst $[subst $this](offsetmin_resolution)]]
    pack $widget -side top -fill x

    ##############################
    # offsetmax
    set widget [CreateScale $root.fScales scaleOffsetMax \
	    [subst $this](offsetmax) "Offset Max" \
	    [subst $[subst $this](offsetmax_min)] \
	    [subst $[subst $this](offsetmax_max)] \
	    [subst $[subst $this](offsetmax_resolution)]]
    pack $widget -side top -fill x

    ##############################
    # individualvoronoimin
    set widget [CreateScale $root.fScales scaleIndividualVoronoiMin \
	    [subst $this](individualvoronoimin) "Individual Voronoi Min" \
	    [subst $[subst $this](individualvoronoimin_min)] \
	    [subst $[subst $this](individualvoronoimin_max)] \
	    [subst $[subst $this](individualvoronoimin_resolution)]]
    pack $widget -side top -fill x

    ##############################
    # individualvoronoimax
    set widget [CreateScale $root.fScales scaleIndividualVoronoiMax \
	    [subst $this](individualvoronoimax) "Individual Voronoi Max" \
	    [subst $[subst $this](individualvoronoimax_min)] \
	    [subst $[subst $this](individualvoronoimax_max)] \
	    [subst $[subst $this](individualvoronoimax_resolution)]]
    pack $widget -side top -fill x

    ##############################
    # subdivisions
    set widget [CreateScale $root.fScales scaleSubdivisions \
	    [subst $this](subdivisions) "Subdivisions" \
	    [subst $[subst $this](subdivisions_min)] \
	    [subst $[subst $this](subdivisions_max)] \
	    [subst $[subst $this](subdivisions_resolution)]]
    pack $widget -side top -fill x

    ##############################
    # tolerance
    set widget [CreateScale $root.fScales scaleTolerance \
	    [subst $this](tolerance) "Tolerance" \
	    [subst $[subst $this](tolerance_min)] \
	    [subst $[subst $this](tolerance_max)] \
	    [subst $[subst $this](tolerance_resolution)]]
    pack $widget -side top -fill x

    pack $root.fScales -side top -fill x

    return $root
}

######################################################################
# CreateSLIDESphereObject
#   Parameters:
#     name     - name of the sphere
#
#   Actions:
#     Creates an associative array with initialized fields for a sphere
#
#   Returns:
#     The name of the associative array
#
proc CreateSLIDESphereObject { name } {
    set this $name

    global $this

    # radius
    set [subst $this](radius) 1.0
    set [subst $this](radius_resolution) 0.01
    set [subst $this](radius_min) [subst $[subst $this](radius_resolution)]
    set [subst $this](radius_max) 10.0

    # zmin
    set [subst $this](zmin) -1.0
    set [subst $this](zmin_min) -1.0
    set [subst $this](zmin_max) 1.0
    set [subst $this](zmin_resolution) 0.01

    # zmax
    set [subst $this](zmax) 1.0
    set [subst $this](zmax_min) -1.0
    set [subst $this](zmax_max) 1.0
    set [subst $this](zmax_resolution) 0.01

    # thetamax
    set [subst $this](thetamax) 360
    set [subst $this](thetamax_min) -360
    set [subst $this](thetamax_max) 360
    set [subst $this](thetamax_resolution) 1

    # zslices
    set [subst $this](zslices) 8
    set [subst $this](zslices_min) 1
    set [subst $this](zslices_max) 100
    set [subst $this](zslices_resolution) 1

    # thetaslices
    set [subst $this](thetaslices) 8
    set [subst $this](thetaslices_min) 1
    set [subst $this](thetaslices_max) 100
    set [subst $this](thetaslices_resolution) 1

    # texture
    CreateSLIDETextureFields $this

    # solid
    CreateSLIDESolidField $this

    # shading
    CreateSLIDEShadingField $this

    # lod
    CreateSLIDELODField $this

    return $this
}

######################################################################
# CreateSLIDESphereUI
#   Parameters:
#     parent   - parent widget name
#     name     - name of the sphere
#
#   Actions:
#     Creates a sphere UI contained in a frame widget
#
#   Returns:
#     The name of the unpacked frame widget
#
proc CreateSLIDESphereUI { parent name } {
    set root $parent.f$name
    set this $name

    global $this

    frame $root

    ##################################################
    # Enums Frame
    frame $root.fEnums
    
    ##############################
    # lod
    set widget [CreateSLIDELODEnumFrame $root.fEnums lod \
	    [subst $this](lod) "LOD"]
    pack $widget -side left -fill y

    ##############################
    # shading
    set widget [CreateSLIDEShadingEnumFrame $root.fEnums shading \
	    [subst $this](shading) "Shading"]
    pack $widget -side left -fill y

    ##############################
    # solid
    set widget [CreateSLIDESolidEnumFrame $root.fEnums solid \
	    [subst $this](solid) "Solid"]
    pack $widget -side left -fill y

    pack $root.fEnums -side top -fill x

    ##################################################
    # Scales Frame
    frame $root.fScales
    
    ##############################
    # radius
    set widget [CreateScale $root.fScales scaleRadius \
	    [subst $this](radius) "Radius" \
	    [subst $[subst $this](radius_min)] \
	    [subst $[subst $this](radius_max)] \
	    [subst $[subst $this](radius_resolution)]]
    pack $widget -side top -fill x

    ##############################
    # zmin
    set widget [CreateScale $root.fScales scaleZMin \
	    [subst $this](zmin) "Z Min" \
	    [subst $[subst $this](zmin_min)] \
	    [subst $[subst $this](zmin_max)] \
	    [subst $[subst $this](zmin_resolution)]]
    pack $widget -side top -fill x

    ##############################
    # zmax
    set widget [CreateScale $root.fScales scaleZMax \
	    [subst $this](zmax) "Z Max" \
	    [subst $[subst $this](zmax_min)] \
	    [subst $[subst $this](zmax_max)] \
	    [subst $[subst $this](zmax_resolution)]]
    pack $widget -side top -fill x

    ##############################
    # thetamax
    set widget [CreateScale $root.fScales scaleThetaMax \
	    [subst $this](thetamax) "Theta Max" \
	    [subst $[subst $this](thetamax_min)] \
	    [subst $[subst $this](thetamax_max)] \
	    [subst $[subst $this](thetamax_resolution)]]
    pack $widget -side top -fill x

    ##############################
    # zslices
    set widget [CreateScale $root.fScales scaleZSlices \
	    [subst $this](zslices) "Z Slices" \
	    [subst $[subst $this](zslices_min)] \
	    [subst $[subst $this](zslices_max)] \
	    [subst $[subst $this](zslices_resolution)]]
    pack $widget -side top -fill x

    ##############################
    # thetaslices
    set widget [CreateScale $root.fScales scaleThetaSlices \
	    [subst $this](thetaslices) "Theta Slices" \
	    [subst $[subst $this](thetaslices_min)] \
	    [subst $[subst $this](thetaslices_max)] \
	    [subst $[subst $this](thetaslices_resolution)]]
    pack $widget -side top -fill x

    ##############################
    # texture
    for { set i 0 } { $i < 4 } { incr i } {
	frame $root.fScales.scaleTexture_[subst $i]
	for { set j 0 } { $j < 2 } { incr j } {
	    set widget [CreateScale $root.fScales.scaleTexture_[subst $i] \
		    scaleTexture_[subst $i]_[subst $j] \
		    [subst $this](texture_[subst $i]_[subst $j]) \
		    "Texture($i)($j)" \
		    [subst $[subst $this](texture_[subst $i]_[subst $j]_min)] \
		    [subst $[subst $this](texture_[subst $i]_[subst $j]_max)] \
		    [subst $[subst $this](texture_[subst $i]_[subst $j]_resolution)]]
	    pack $widget -side left -fill y
	}
	pack $root.fScales.scaleTexture_[subst $i] -side top -fill x
    }

    pack $root.fScales -side top -fill x

    return $root
}

######################################################################
# CreateSLIDECylinderObject
#   Parameters:
#     name     - name of the cylinder
#
#   Actions:
#     Creates an associative array with initialized fields for a cylinder
#
#   Returns:
#     The name of the associative array
#
proc CreateSLIDECylinderObject { name } {
    set this $name

    global $this

    # radius
    set [subst $this](radius) 1.0
    set [subst $this](radius_resolution) 0.01
    set [subst $this](radius_min) [subst $[subst $this](radius_resolution)]
    set [subst $this](radius_max) 10.0

    # zmin
    set [subst $this](zmin) 0.0
    set [subst $this](zmin_min) -1.0
    set [subst $this](zmin_max) 1.0
    set [subst $this](zmin_resolution) 0.01

    # zmax
    set [subst $this](zmax) 1.0
    set [subst $this](zmax_min) -1.0
    set [subst $this](zmax_max) 1.0
    set [subst $this](zmax_resolution) 0.01

    # thetamax
    set [subst $this](thetamax) 360
    set [subst $this](thetamax_min) -360
    set [subst $this](thetamax_max) 360
    set [subst $this](thetamax_resolution) 1

    # zslices
    set [subst $this](zslices) 1
    set [subst $this](zslices_min) 1
    set [subst $this](zslices_max) 512
    set [subst $this](zslices_resolution) 1

    # thetaslices
    set [subst $this](thetaslices) 8
    set [subst $this](thetaslices_min) 1
    set [subst $this](thetaslices_max) 512
    set [subst $this](thetaslices_resolution) 1

    # texture
    CreateSLIDETextureFields $this

    # solid
    CreateSLIDESolidField $this

    # shading
    CreateSLIDEShadingField $this

    # lod
    CreateSLIDELODField $this

    return $this
}

######################################################################
# CreateSLIDECylinderUI
#   Parameters:
#     parent   - parent widget name
#     name     - name of the cylinder
#
#   Actions:
#     Creates a cylinder UI contained in a frame widget
#
#   Returns:
#     The name of the unpacked frame widget
#
proc CreateSLIDECylinderUI { parent name } {
    set root $parent.f$name
    set this $name

    global $this

    frame $root

    ##################################################
    # Enums Frame
    frame $root.fEnums
    
    ##############################
    # lod
    set widget [CreateSLIDELODEnumFrame $root.fEnums lod \
	    [subst $this](lod) "LOD"]
    pack $widget -side left -fill y

    ##############################
    # shading
    set widget [CreateSLIDEShadingEnumFrame $root.fEnums shading \
	    [subst $this](shading) "Shading"]
    pack $widget -side left -fill y

    ##############################
    # solid
    set widget [CreateSLIDESolidEnumFrame $root.fEnums solid \
	    [subst $this](solid) "Solid"]
    pack $widget -side left -fill y

    pack $root.fEnums -side top -fill x

    ##################################################
    # Scales Frame
    frame $root.fScales
    
    ##############################
    # radius
    set widget [CreateScale $root.fScales scaleRadius \
	    [subst $this](radius) "Radius" \
	    [subst $[subst $this](radius_min)] \
	    [subst $[subst $this](radius_max)] \
	    [subst $[subst $this](radius_resolution)]]
    pack $widget -side top -fill x

    ##############################
    # zmin
    set widget [CreateScale $root.fScales scaleZMin \
	    [subst $this](zmin) "Z Min" \
	    [subst $[subst $this](zmin_min)] \
	    [subst $[subst $this](zmin_max)] \
	    [subst $[subst $this](zmin_resolution)]]
    pack $widget -side top -fill x

    ##############################
    # zmax
    set widget [CreateScale $root.fScales scaleZMax \
	    [subst $this](zmax) "Z Max" \
	    [subst $[subst $this](zmax_min)] \
	    [subst $[subst $this](zmax_max)] \
	    [subst $[subst $this](zmax_resolution)]]
    pack $widget -side top -fill x

    ##############################
    # thetamax
    set widget [CreateScale $root.fScales scaleThetaMax \
	    [subst $this](thetamax) "Theta Max" \
	    [subst $[subst $this](thetamax_min)] \
	    [subst $[subst $this](thetamax_max)] \
	    [subst $[subst $this](thetamax_resolution)]]
    pack $widget -side top -fill x

    ##############################
    # zslices
    set widget [CreateScale $root.fScales scaleZSlices \
	    [subst $this](zslices) "Z Slices" \
	    [subst $[subst $this](zslices_min)] \
	    [subst $[subst $this](zslices_max)] \
	    [subst $[subst $this](zslices_resolution)]]
    pack $widget -side top -fill x

    ##############################
    # thetaslices
    set widget [CreateScale $root.fScales scaleThetaSlices \
	    [subst $this](thetaslices) "Theta Slices" \
	    [subst $[subst $this](thetaslices_min)] \
	    [subst $[subst $this](thetaslices_max)] \
	    [subst $[subst $this](thetaslices_resolution)]]
    pack $widget -side top -fill x

    ##############################
    # texture
    for { set i 0 } { $i < 4 } { incr i } {
	frame $root.fScales.scaleTexture_[subst $i]
	for { set j 0 } { $j < 2 } { incr j } {
	    set widget [CreateScale $root.fScales.scaleTexture_[subst $i] \
		    scaleTexture_[subst $i]_[subst $j] \
		    [subst $this](texture_[subst $i]_[subst $j]) \
		    "Texture($i)($j)" \
		    [subst $[subst $this](texture_[subst $i]_[subst $j]_min)] \
		    [subst $[subst $this](texture_[subst $i]_[subst $j]_max)] \
		    [subst $[subst $this](texture_[subst $i]_[subst $j]_resolution)]]
	    pack $widget -side left -fill y
	}
	pack $root.fScales.scaleTexture_[subst $i] -side top -fill x
    }
    pack $root.fScales -side top -fill x

    return $root
}

######################################################################
# CreateSLIDEConeObject
#   Parameters:
#     name     - name of the cone
#
#   Actions:
#     Creates an associative array with initialized fields for a cone
#
#   Returns:
#     The name of the associative array
#
proc CreateSLIDEConeObject { name } {
    set this $name

    global $this

    # radius
    set [subst $this](radius) 1.0
    set [subst $this](radius_resolution) 0.01
    set [subst $this](radius_min) [subst $[subst $this](radius_resolution)]
    set [subst $this](radius_max) 10.0

    # height
    set [subst $this](height) 1.0
    set [subst $this](height_min) 0.0
    set [subst $this](height_max) 10.0
    set [subst $this](height_resolution) 0.01

    # thetamax
    set [subst $this](thetamax) 360
    set [subst $this](thetamax_min) -360
    set [subst $this](thetamax_max) 360
    set [subst $this](thetamax_resolution) 1

    # zslices
    set [subst $this](zslices) 1
    set [subst $this](zslices_min) 1
    set [subst $this](zslices_max) 100
    set [subst $this](zslices_resolution) 1

    # thetaslices
    set [subst $this](thetaslices) 8
    set [subst $this](thetaslices_min) 1
    set [subst $this](thetaslices_max) 100
    set [subst $this](thetaslices_resolution) 1

    # texture
    CreateSLIDETextureFields $this

    # solid
    CreateSLIDESolidField $this

    # shading
    CreateSLIDEShadingField $this

    # lod
    CreateSLIDELODField $this

    return $this
}

######################################################################
# CreateSLIDEConeUI
#   Parameters:
#     parent   - parent widget name
#     name     - name of the cone
#
#   Actions:
#     Creates a cone UI contained in a frame widget
#
#   Returns:
#     The name of the unpacked frame widget
#
proc CreateSLIDEConeUI { parent name } {
    set root $parent.f$name
    set this $name

    global $this

    frame $root

    ##################################################
    # Enums Frame
    frame $root.fEnums
    
    ##############################
    # lod
    set widget [CreateSLIDELODEnumFrame $root.fEnums lod \
	    [subst $this](lod) "LOD"]
    pack $widget -side left -fill y

    ##############################
    # shading
    set widget [CreateSLIDEShadingEnumFrame $root.fEnums shading \
	    [subst $this](shading) "Shading"]
    pack $widget -side left -fill y

    ##############################
    # solid
    set widget [CreateSLIDESolidEnumFrame $root.fEnums solid \
	    [subst $this](solid) "Solid"]
    pack $widget -side left -fill y

    pack $root.fEnums -side top -fill x

    ##################################################
    # Scales Frame
    frame $root.fScales
    
    ##############################
    # radius
    set widget [CreateScale $root.fScales scaleRadius \
	    [subst $this](radius) "Radius" \
	    [subst $[subst $this](radius_min)] \
	    [subst $[subst $this](radius_max)] \
	    [subst $[subst $this](radius_resolution)]]
    pack $widget -side top -fill x

    ##############################
    # height
    set widget [CreateScale $root.fScales scaleHeight \
	    [subst $this](height) "Height" \
	    [subst $[subst $this](height_min)] \
	    [subst $[subst $this](height_max)] \
	    [subst $[subst $this](height_resolution)]]
    pack $widget -side top -fill x

    ##############################
    # thetamax
    set widget [CreateScale $root.fScales scaleThetaMax \
	    [subst $this](thetamax) "Theta Max" \
	    [subst $[subst $this](thetamax_min)] \
	    [subst $[subst $this](thetamax_max)] \
	    [subst $[subst $this](thetamax_resolution)]]
    pack $widget -side top -fill x

    ##############################
    # zslices
    set widget [CreateScale $root.fScales scaleZSlices \
	    [subst $this](zslices) "Z Slices" \
	    [subst $[subst $this](zslices_min)] \
	    [subst $[subst $this](zslices_max)] \
	    [subst $[subst $this](zslices_resolution)]]
    pack $widget -side top -fill x

    ##############################
    # thetaslices
    set widget [CreateScale $root.fScales scaleThetaSlices \
	    [subst $this](thetaslices) "Theta Slices" \
	    [subst $[subst $this](thetaslices_min)] \
	    [subst $[subst $this](thetaslices_max)] \
	    [subst $[subst $this](thetaslices_resolution)]]
    pack $widget -side top -fill x

    ##############################
    # texture
    for { set i 0 } { $i < 4 } { incr i } {
	frame $root.fScales.scaleTexture_[subst $i]
	for { set j 0 } { $j < 2 } { incr j } {
	    set widget [CreateScale $root.fScales.scaleTexture_[subst $i] \
		    scaleTexture_[subst $i]_[subst $j] \
		    [subst $this](texture_[subst $i]_[subst $j]) \
		    "Texture($i)($j)" \
		    [subst $[subst $this](texture_[subst $i]_[subst $j]_min)] \
		    [subst $[subst $this](texture_[subst $i]_[subst $j]_max)] \
		    [subst $[subst $this](texture_[subst $i]_[subst $j]_resolution)]]
	    pack $widget -side left -fill y
	}
	pack $root.fScales.scaleTexture_[subst $i] -side top -fill x
    }
    pack $root.fScales -side top -fill x

    return $root
}

######################################################################
# CreateSLIDETorusObject
#   Parameters:
#     name     - name of the torus
#
#   Actions:
#     Creates an associative array with initialized fields for a torus
#
#   Returns:
#     The name of the associative array
#
proc CreateSLIDETorusObject { name } {
    set this $name

    global $this

    # majorradius
    set [subst $this](majorradius) 1.0
    set [subst $this](majorradius_resolution) 0.01
    set [subst $this](majorradius_min) [subst $[subst $this](majorradius_resolution)]
    set [subst $this](majorradius_max) 10.0

    # majorradius
    set [subst $this](minorradius) 0.5
    set [subst $this](minorradius_resolution) 0.01
    set [subst $this](minorradius_min) [subst $[subst $this](minorradius_resolution)]
    set [subst $this](minorradius_max) 10.0

    # phimin
    set [subst $this](phimin) 0
    set [subst $this](phimin_min) -360
    set [subst $this](phimin_max) 360
    set [subst $this](phimin_resolution) 1

    # phimax
    set [subst $this](phimax) 360
    set [subst $this](phimax_min) -360
    set [subst $this](phimax_max) 360
    set [subst $this](phimax_resolution) 1

    # thetamax
    set [subst $this](thetamax) 360
    set [subst $this](thetamax_min) -360
    set [subst $this](thetamax_max) 360
    set [subst $this](thetamax_resolution) 1

    # phislices
    set [subst $this](phislices) 8
    set [subst $this](phislices_min) 1
    set [subst $this](phislices_max) 100
    set [subst $this](phislices_resolution) 1

    # thetaslices
    set [subst $this](thetaslices) 8
    set [subst $this](thetaslices_min) 1
    set [subst $this](thetaslices_max) 100
    set [subst $this](thetaslices_resolution) 1

    # texture
    CreateSLIDETextureFields $this

    # solid
    CreateSLIDESolidField $this

    # shading
    CreateSLIDEShadingField $this

    # lod
    CreateSLIDELODField $this

    return $this
}

######################################################################
# CreateSLIDETorusUI
#   Parameters:
#     parent   - parent widget name
#     name     - name of the torus
#
#   Actions:
#     Creates a torus UI contained in a frame widget
#
#   Returns:
#     The name of the unpacked frame widget
#
proc CreateSLIDETorusUI { parent name } {
    set root $parent.f$name
    set this $name

    global $this

    frame $root

    ##################################################
    # Enums Frame
    frame $root.fEnums
    
    ##############################
    # lod
    set widget [CreateSLIDELODEnumFrame $root.fEnums lod \
	    [subst $this](lod) "LOD"]
    pack $widget -side left -fill y

    ##############################
    # shading
    set widget [CreateSLIDEShadingEnumFrame $root.fEnums shading \
	    [subst $this](shading) "Shading"]
    pack $widget -side left -fill y

    ##############################
    # solid
    set widget [CreateSLIDESolidEnumFrame $root.fEnums solid \
	    [subst $this](solid) "Solid"]
    pack $widget -side left -fill y

    pack $root.fEnums -side top -fill x

    ##################################################
    # Scales Frame
    frame $root.fScales
    
    ##############################
    # majorradius
    set widget [CreateScale $root.fScales scaleMajorRadius \
	    [subst $this](majorradius) "Major Radius" \
	    [subst $[subst $this](majorradius_min)] \
	    [subst $[subst $this](majorradius_max)] \
	    [subst $[subst $this](majorradius_resolution)]]
    pack $widget -side top -fill x

    ##############################
    # minorradius
    set widget [CreateScale $root.fScales scaleMinorRadius \
	    [subst $this](minorradius) "Minor Radius" \
	    [subst $[subst $this](minorradius_min)] \
	    [subst $[subst $this](minorradius_max)] \
	    [subst $[subst $this](minorradius_resolution)]]
    pack $widget -side top -fill x

    ##############################
    # phimin
    set widget [CreateScale $root.fScales scalePhiMin \
	    [subst $this](phimin) "Phi Min" \
	    [subst $[subst $this](phimin_min)] \
	    [subst $[subst $this](phimin_max)] \
	    [subst $[subst $this](phimin_resolution)]]
    pack $widget -side top -fill x

    ##############################
    # phimax
    set widget [CreateScale $root.fScales scalePhiMax \
	    [subst $this](phimax) "Phi Max" \
	    [subst $[subst $this](phimax_min)] \
	    [subst $[subst $this](phimax_max)] \
	    [subst $[subst $this](phimax_resolution)]]
    pack $widget -side top -fill x

    ##############################
    # thetamax
    set widget [CreateScale $root.fScales scaleThetaMax \
	    [subst $this](thetamax) "Theta Max" \
	    [subst $[subst $this](thetamax_min)] \
	    [subst $[subst $this](thetamax_max)] \
	    [subst $[subst $this](thetamax_resolution)]]
    pack $widget -side top -fill x

    ##############################
    # phislices
    set widget [CreateScale $root.fScales scalePhiSlices \
	    [subst $this](phislices) "Phi Slices" \
	    [subst $[subst $this](phislices_min)] \
	    [subst $[subst $this](phislices_max)] \
	    [subst $[subst $this](phislices_resolution)]]
    pack $widget -side top -fill x

    ##############################
    # thetaslices
    set widget [CreateScale $root.fScales scaleThetaSlices \
	    [subst $this](thetaslices) "Theta Slices" \
	    [subst $[subst $this](thetaslices_min)] \
	    [subst $[subst $this](thetaslices_max)] \
	    [subst $[subst $this](thetaslices_resolution)]]
    pack $widget -side top -fill x

    ##############################
    # texture
    for { set i 0 } { $i < 4 } { incr i } {
	frame $root.fScales.scaleTexture_[subst $i]
	for { set j 0 } { $j < 2 } { incr j } {
	    set widget [CreateScale $root.fScales.scaleTexture_[subst $i] \
		    scaleTexture_[subst $i]_[subst $j] \
		    [subst $this](texture_[subst $i]_[subst $j]) \
		    "Texture($i)($j)" \
		    [subst $[subst $this](texture_[subst $i]_[subst $j]_min)] \
		    [subst $[subst $this](texture_[subst $i]_[subst $j]_max)] \
		    [subst $[subst $this](texture_[subst $i]_[subst $j]_resolution)]]
	    pack $widget -side left -fill y
	}
	pack $root.fScales.scaleTexture_[subst $i] -side top -fill x
    }
    pack $root.fScales -side top -fill x

    return $root
}

######################################################################
# CreateSLIDECyclideObject
#   Parameters:
#     name     - name of the cyclide
#
#   Actions:
#     Creates an associative array with initialized fields for a cyclide
#
#   Returns:
#     The name of the associative array
#
proc CreateSLIDECyclideObject { name } {
    set this $name

    global $this

    # drawellipse
    set [subst $this](drawellipse) 0
    set [subst $this](drawellipse_min) 0
    set [subst $this](drawellipse_max) 1
    set [subst $this](drawellipse_resolution) 1

    # drawhyperbola
    set [subst $this](drawhyperbola) 0
    set [subst $this](drawhyperbola_min) 0
    set [subst $this](drawhyperbola_max) 1
    set [subst $this](drawhyperbola_resolution) 1

    # a
    set [subst $this](a) 5.0
    set [subst $this](a_resolution) 0.01
    set [subst $this](a_min) [subst $[subst $this](a_resolution)]
    set [subst $this](a_max) 10.0

    # b
    set [subst $this](b) 4.0
    set [subst $this](b_resolution) 0.01
    set [subst $this](b_min) [subst $[subst $this](b_resolution)]
    set [subst $this](b_max) 10.0

    # stringlength
    set [subst $this](stringlength) 8.0
    set [subst $this](stringlength_resolution) 0.01
    set [subst $this](stringlength_min) [subst $[subst $this](stringlength_resolution)]
    set [subst $this](stringlength_max) 20.0

    # phimin
    set [subst $this](phimin) 0
    set [subst $this](phimin_min) -360
    set [subst $this](phimin_max) 360
    set [subst $this](phimin_resolution) 1

    # phimax
    set [subst $this](phimax) 360
    set [subst $this](phimax_min) -360
    set [subst $this](phimax_max) 360
    set [subst $this](phimax_resolution) 1

    # thetamin
    set [subst $this](thetamin) 0
    set [subst $this](thetamin_min) -360
    set [subst $this](thetamin_max) 360
    set [subst $this](thetamin_resolution) 1

    # thetamax
    set [subst $this](thetamax) 360
    set [subst $this](thetamax_min) -360
    set [subst $this](thetamax_max) 360
    set [subst $this](thetamax_resolution) 1

    # phislices
    set [subst $this](phislices) 8
    set [subst $this](phislices_min) 1
    set [subst $this](phislices_max) 100
    set [subst $this](phislices_resolution) 1

    # thetaslices
    set [subst $this](thetaslices) 8
    set [subst $this](thetaslices_min) 1
    set [subst $this](thetaslices_max) 100
    set [subst $this](thetaslices_resolution) 1

    # texture
    CreateSLIDETextureFields $this

    # solid
    CreateSLIDESolidField $this

    # shading
    CreateSLIDEShadingField $this

    # lod
    CreateSLIDELODField $this

    return $this
}

######################################################################
# CreateSLIDECyclideUI
#   Parameters:
#     parent   - parent widget name
#     name     - name of the cyclide
#
#   Actions:
#     Creates a cyclide UI contained in a frame widget
#
#   Returns:
#     The name of the unpacked frame widget
#
proc CreateSLIDECyclideUI { parent name } {
    set root $parent.f$name
    set this $name

    global $this

    frame $root

    ##################################################
    # Enums Frame
    frame $root.fEnums
    
    ##############################
    # lod
    set widget [CreateSLIDELODEnumFrame $root.fEnums lod \
	    [subst $this](lod) "LOD"]
    pack $widget -side left -fill y

    ##############################
    # shading
    set widget [CreateSLIDEShadingEnumFrame $root.fEnums shading \
	    [subst $this](shading) "Shading"]
    pack $widget -side left -fill y

    ##############################
    # solid
    set widget [CreateSLIDESolidEnumFrame $root.fEnums solid \
	    [subst $this](solid) "Solid"]
    pack $widget -side left -fill y

    pack $root.fEnums -side top -fill x

    ##################################################
    # Scales Frame
    frame $root.fScales
    
    ##############################
    # drawellipse
    set widget [CreateScale $root.fScales scaleDrawEllipse \
	    [subst $this](drawellipse) "Draw Ellipse" \
	    [subst $[subst $this](drawellipse_min)] \
	    [subst $[subst $this](drawellipse_max)] \
	    [subst $[subst $this](drawellipse_resolution)]]
    pack $widget -side top -fill x

    ##############################
    # drawhyperbola
    set widget [CreateScale $root.fScales scaleDrawHyperbola \
	    [subst $this](drawhyperbola) "Draw Hyperbola" \
	    [subst $[subst $this](drawhyperbola_min)] \
	    [subst $[subst $this](drawhyperbola_max)] \
	    [subst $[subst $this](drawhyperbola_resolution)]]
    pack $widget -side top -fill x

    ##############################
    # a
    set widget [CreateScale $root.fScales scaleA \
	    [subst $this](a) "A" \
	    [subst $[subst $this](a_min)] \
	    [subst $[subst $this](a_max)] \
	    [subst $[subst $this](a_resolution)]]
    pack $widget -side top -fill x

    ##############################
    # b
    set widget [CreateScale $root.fScales scaleB \
	    [subst $this](b) "B" \
	    [subst $[subst $this](b_min)] \
	    [subst $[subst $this](b_max)] \
	    [subst $[subst $this](b_resolution)]]
    pack $widget -side top -fill x

    ##############################
    # stringlength
    set widget [CreateScale $root.fScales scaleStringLength \
	    [subst $this](stringlength) "String Length" \
	    [subst $[subst $this](stringlength_min)] \
	    [subst $[subst $this](stringlength_max)] \
	    [subst $[subst $this](stringlength_resolution)]]
    pack $widget -side top -fill x

    ##############################
    # phimin
    set widget [CreateScale $root.fScales scalePhiMin \
	    [subst $this](phimin) "Phi Min" \
	    [subst $[subst $this](phimin_min)] \
	    [subst $[subst $this](phimin_max)] \
	    [subst $[subst $this](phimin_resolution)]]
    pack $widget -side top -fill x

    ##############################
    # phimax
    set widget [CreateScale $root.fScales scalePhiMax \
	    [subst $this](phimax) "Phi Max" \
	    [subst $[subst $this](phimax_min)] \
	    [subst $[subst $this](phimax_max)] \
	    [subst $[subst $this](phimax_resolution)]]
    pack $widget -side top -fill x

    ##############################
    # thetamin
    set widget [CreateScale $root.fScales scaleThetaMin \
	    [subst $this](thetamin) "Theta Min" \
	    [subst $[subst $this](thetamin_min)] \
	    [subst $[subst $this](thetamin_max)] \
	    [subst $[subst $this](thetamin_resolution)]]
    pack $widget -side top -fill x

    ##############################
    # thetamax
    set widget [CreateScale $root.fScales scaleThetaMax \
	    [subst $this](thetamax) "Theta Max" \
	    [subst $[subst $this](thetamax_min)] \
	    [subst $[subst $this](thetamax_max)] \
	    [subst $[subst $this](thetamax_resolution)]]
    pack $widget -side top -fill x

    ##############################
    # phislices
    set widget [CreateScale $root.fScales scalePhiSlices \
	    [subst $this](phislices) "Phi Slices" \
	    [subst $[subst $this](phislices_min)] \
	    [subst $[subst $this](phislices_max)] \
	    [subst $[subst $this](phislices_resolution)]]
    pack $widget -side top -fill x

    ##############################
    # thetaslices
    set widget [CreateScale $root.fScales scaleThetaSlices \
	    [subst $this](thetaslices) "Theta Slices" \
	    [subst $[subst $this](thetaslices_min)] \
	    [subst $[subst $this](thetaslices_max)] \
	    [subst $[subst $this](thetaslices_resolution)]]
    pack $widget -side top -fill x

    ##############################
    # texture
    for { set i 0 } { $i < 4 } { incr i } {
	frame $root.fScales.scaleTexture_[subst $i]
	for { set j 0 } { $j < 2 } { incr j } {
	    set widget [CreateScale $root.fScales.scaleTexture_[subst $i] \
		    scaleTexture_[subst $i]_[subst $j] \
		    [subst $this](texture_[subst $i]_[subst $j]) \
		    "Texture($i)($j)" \
		    [subst $[subst $this](texture_[subst $i]_[subst $j]_min)] \
		    [subst $[subst $this](texture_[subst $i]_[subst $j]_max)] \
		    [subst $[subst $this](texture_[subst $i]_[subst $j]_resolution)]]
	    pack $widget -side left -fill y
	}
	pack $root.fScales.scaleTexture_[subst $i] -side top -fill x
    }
    pack $root.fScales -side top -fill x

    return $root
}

######################################################################
# CreateSLIDEBSplinePatchObject
#   Parameters:
#     name     - name of the bsplinepatch
#
#   Actions:
#     Creates an associative array with initialized fields for a bsplinepatch
#
#   Returns:
#     The name of the associative array
#
proc CreateSLIDEBSplinePatchObject { name } {
    set this $name

    global $this

    # drawcontrols
    set [subst $this](drawcontrols) 0
    set [subst $this](drawcontrols_min) 0
    set [subst $this](drawcontrols_max) 1
    set [subst $this](drawcontrols_resolution) 1

    # uorder
    set [subst $this](uorder) 4
    set [subst $this](uorder_min) 2
    set [subst $this](uorder_max) 6
    set [subst $this](uorder_resolution) 1

    # vorder
    set [subst $this](vorder) 4
    set [subst $this](vorder_min) 2
    set [subst $this](vorder_max) 6
    set [subst $this](vorder_resolution) 1

    # uslices
    set [subst $this](uslices) 8
    set [subst $this](uslices_min) 1
    set [subst $this](uslices_max) 100
    set [subst $this](uslices_resolution) 1

    # vslices
    set [subst $this](vslices) 8
    set [subst $this](vslices_min) 1
    set [subst $this](vslices_max) 100
    set [subst $this](vslices_resolution) 1

    # texture
    CreateSLIDETextureFields $this

    # solid
    CreateSLIDESolidField $this

    # shading
    CreateSLIDEShadingField $this

    # lod
    CreateSLIDELODField $this

    return $this
}

######################################################################
# CreateSLIDEBSplinePatchUI
#   Parameters:
#     parent   - parent widget name
#     name     - name of the bsplinepatch
#
#   Actions:
#     Creates a bsplinepatch UI contained in a frame widget
#
#   Returns:
#     The name of the unpacked frame widget
#
proc CreateSLIDEBSplinePatchUI { parent name } {
    set root $parent.f$name
    set this $name

    global $this

    frame $root

    ##################################################
    # Enums Frame
    frame $root.fEnums
    
    ##############################
    # lod
    set widget [CreateSLIDELODEnumFrame $root.fEnums lod \
	    [subst $this](lod) "LOD"]
    pack $widget -side left -fill y

    ##############################
    # shading
    set widget [CreateSLIDEShadingEnumFrame $root.fEnums shading \
	    [subst $this](shading) "Shading"]
    pack $widget -side left -fill y

    ##############################
    # solid
    set widget [CreateSLIDESolidEnumFrame $root.fEnums solid \
	    [subst $this](solid) "Solid"]
    pack $widget -side left -fill y

    pack $root.fEnums -side top -fill x

    ##################################################
    # Scales Frame
    frame $root.fScales
    
    # drawcontrols
    set widget [CreateScale $root.fScales scaleDrawControls \
	    [subst $this](drawcontrols) "Draw Controls" \
	    [subst $[subst $this](drawcontrols_min)] \
	    [subst $[subst $this](drawcontrols_max)] \
	    [subst $[subst $this](drawcontrols_resolution)]]
    pack $widget -side top -fill x

    ##############################
    # uorder
    set widget [CreateScale $root.fScales scaleUOrder \
	    [subst $this](uorder) "U Order" \
	    [subst $[subst $this](uorder_min)] \
	    [subst $[subst $this](uorder_max)] \
	    [subst $[subst $this](uorder_resolution)]]
    pack $widget -side top -fill x

    ##############################
    # vorder
    set widget [CreateScale $root.fScales scaleVOrder \
	    [subst $this](vorder) "V Order" \
	    [subst $[subst $this](vorder_min)] \
	    [subst $[subst $this](vorder_max)] \
	    [subst $[subst $this](vorder_resolution)]]
    pack $widget -side top -fill x

    ##############################
    # uslices
    set widget [CreateScale $root.fScales scaleUSlices \
	    [subst $this](uslices) "U Slices" \
	    [subst $[subst $this](uslices_min)] \
	    [subst $[subst $this](uslices_max)] \
	    [subst $[subst $this](uslices_resolution)]]
    pack $widget -side top -fill x

    ##############################
    # vslices
    set widget [CreateScale $root.fScales scaleVSlices \
	    [subst $this](vslices) "V Slices" \
	    [subst $[subst $this](vslices_min)] \
	    [subst $[subst $this](vslices_max)] \
	    [subst $[subst $this](vslices_resolution)]]
    pack $widget -side top -fill x

    ##############################
    # texture
    for { set i 0 } { $i < 4 } { incr i } {
	frame $root.fScales.scaleTexture_[subst $i]
	for { set j 0 } { $j < 2 } { incr j } {
	    set widget [CreateScale $root.fScales.scaleTexture_[subst $i] \
		    scaleTexture_[subst $i]_[subst $j] \
		    [subst $this](texture_[subst $i]_[subst $j]) \
		    "Texture($i)($j)" \
		    [subst $[subst $this](texture_[subst $i]_[subst $j]_min)] \
		    [subst $[subst $this](texture_[subst $i]_[subst $j]_max)] \
		    [subst $[subst $this](texture_[subst $i]_[subst $j]_resolution)]]
	    pack $widget -side left -fill y
	}
	pack $root.fScales.scaleTexture_[subst $i] -side top -fill x
    }
    pack $root.fScales -side top -fill x

    return $root
}

######################################################################
# CreateSLIDEBezierPatchObject
#   Parameters:
#     name     - name of the bezierpatch
#
#   Actions:
#     Creates an associative array with initialized fields for a bezierpatch
#
#   Returns:
#     The name of the associative array
#
proc CreateSLIDEBezierPatchObject { name } {
    set this $name

    global $this

    # drawcontrols
    set [subst $this](drawcontrols) 0
    set [subst $this](drawcontrols_min) 0
    set [subst $this](drawcontrols_max) 1
    set [subst $this](drawcontrols_resolution) 1

    # uorder
    set [subst $this](uorder) 4
    set [subst $this](uorder_min) 2
    set [subst $this](uorder_max) 6
    set [subst $this](uorder_resolution) 1

    # vorder
    set [subst $this](vorder) 4
    set [subst $this](vorder_min) 2
    set [subst $this](vorder_max) 6
    set [subst $this](vorder_resolution) 1

    # uslices
    set [subst $this](uslices) 8
    set [subst $this](uslices_min) 1
    set [subst $this](uslices_max) 100
    set [subst $this](uslices_resolution) 1

    # vslices
    set [subst $this](vslices) 8
    set [subst $this](vslices_min) 1
    set [subst $this](vslices_max) 100
    set [subst $this](vslices_resolution) 1

    # texture
    CreateSLIDETextureFields $this

    # solid
    CreateSLIDESolidField $this

    # shading
    CreateSLIDEShadingField $this

    # lod
    CreateSLIDELODField $this

    return $this
}

######################################################################
# CreateSLIDEBezierPatchUI
#   Parameters:
#     parent   - parent widget name
#     name     - name of the bezierpatch
#
#   Actions:
#     Creates a bezierpatch UI contained in a frame widget
#
#   Returns:
#     The name of the unpacked frame widget
#
proc CreateSLIDEBezierPatchUI { parent name } {
    set root $parent.f$name
    set this $name

    global $this

    frame $root

    ##################################################
    # Enums Frame
    frame $root.fEnums
    
    ##############################
    # lod
    set widget [CreateSLIDELODEnumFrame $root.fEnums lod \
	    [subst $this](lod) "LOD"]
    pack $widget -side left -fill y

    ##############################
    # shading
    set widget [CreateSLIDEShadingEnumFrame $root.fEnums shading \
	    [subst $this](shading) "Shading"]
    pack $widget -side left -fill y

    ##############################
    # solid
    set widget [CreateSLIDESolidEnumFrame $root.fEnums solid \
	    [subst $this](solid) "Solid"]
    pack $widget -side left -fill y

    pack $root.fEnums -side top -fill x

    ##################################################
    # Scales Frame
    frame $root.fScales
    
    # drawcontrols
    set widget [CreateScale $root.fScales scaleDrawControls \
	    [subst $this](drawcontrols) "Draw Controls" \
	    [subst $[subst $this](drawcontrols_min)] \
	    [subst $[subst $this](drawcontrols_max)] \
	    [subst $[subst $this](drawcontrols_resolution)]]
    pack $widget -side top -fill x

    ##############################
    # uorder
    set widget [CreateScale $root.fScales scaleUOrder \
	    [subst $this](uorder) "U Order" \
	    [subst $[subst $this](uorder_min)] \
	    [subst $[subst $this](uorder_max)] \
	    [subst $[subst $this](uorder_resolution)]]
    pack $widget -side top -fill x

    ##############################
    # vorder
    set widget [CreateScale $root.fScales scaleVOrder \
	    [subst $this](vorder) "V Order" \
	    [subst $[subst $this](vorder_min)] \
	    [subst $[subst $this](vorder_max)] \
	    [subst $[subst $this](vorder_resolution)]]
    pack $widget -side top -fill x

    ##############################
    # uslices
    set widget [CreateScale $root.fScales scaleUSlices \
	    [subst $this](uslices) "U Slices" \
	    [subst $[subst $this](uslices_min)] \
	    [subst $[subst $this](uslices_max)] \
	    [subst $[subst $this](uslices_resolution)]]
    pack $widget -side top -fill x

    ##############################
    # vslices
    set widget [CreateScale $root.fScales scaleVSlices \
	    [subst $this](vslices) "V Slices" \
	    [subst $[subst $this](vslices_min)] \
	    [subst $[subst $this](vslices_max)] \
	    [subst $[subst $this](vslices_resolution)]]
    pack $widget -side top -fill x

    ##############################
    # texture
    for { set i 0 } { $i < 4 } { incr i } {
	frame $root.fScales.scaleTexture_[subst $i]
	for { set j 0 } { $j < 2 } { incr j } {
	    set widget [CreateScale $root.fScales.scaleTexture_[subst $i] \
		    scaleTexture_[subst $i]_[subst $j] \
		    [subst $this](texture_[subst $i]_[subst $j]) \
		    "Texture($i)($j)" \
		    [subst $[subst $this](texture_[subst $i]_[subst $j]_min)] \
		    [subst $[subst $this](texture_[subst $i]_[subst $j]_max)] \
		    [subst $[subst $this](texture_[subst $i]_[subst $j]_resolution)]]
	    pack $widget -side left -fill y
	}
	pack $root.fScales.scaleTexture_[subst $i] -side top -fill x
    }
    pack $root.fScales -side top -fill x

    return $root
}

######################################################################
# CreateSLIDECameraObject
#   Parameters:
#     name     - name of the camera
#
#   Actions:
#     Creates an associative array with initialized fields for a camera
#
#   Returns:
#     The name of the associative array
#
proc CreateSLIDECameraObject { name } {
    set this $name

    global $this

    # projection
    CreateSLIDEProjectionField $this

    set [subst $this](balance_0) 1
    set [subst $this](balance_1) 1

    # min_0
    set [subst $this](min_0) -0.5
    set [subst $this](min_0_min) -5.0
    set [subst $this](min_0_max) 5.0
    set [subst $this](min_0_resolution) 0.01

    # max_0
    set [subst $this](max_0) 0.5
    set [subst $this](max_0_min) -5.0
    set [subst $this](max_0_max) 5.0
    set [subst $this](max_0_resolution) 0.01

    # min_1
    set [subst $this](min_1) -0.5
    set [subst $this](min_1_min) -5.0
    set [subst $this](min_1_max) 5.0
    set [subst $this](min_1_resolution) 0.01

    # max_1
    set [subst $this](max_1) 0.5
    set [subst $this](max_1_min) -5.0
    set [subst $this](max_1_max) 5.0
    set [subst $this](max_1_resolution) 0.01

    # min_2
    set [subst $this](min_2) -3.0
    set [subst $this](min_2_resolution) 0.01
    set [subst $this](min_2_min) -3.0
    set [subst $this](min_2_max) [subst -$[subst $this](min_2_resolution)]

    # max_2
    set [subst $this](max_2) -1
    set [subst $this](max_2_resolution) 0.01
    set [subst $this](max_2_min) -3.0
    set [subst $this](max_2_max) [subst -$[subst $this](max_2_resolution)]

    # dop
    set [subst $this](dop_0) 0.0
    set [subst $this](dop_1) 0.0
    set [subst $this](dop_2) -1.0

    return $this
}

######################################################################
# UpdateSLIDECameraObject
#   Parameters:
#     name     - name of the camera
#
#   Actions:
#     Updates the associative array fields for a camera
#
#   Returns:
#     The name of the associative array
#
proc UpdateSLIDECameraObject { name } {
    set this $name

    global $this

    # dop
    set [subst $this](dop_0) [expr ([subst $[subst $this](min_0)] + [subst $[subst $this](max_0)]) / 2.0]
    set [subst $this](dop_1) [expr ([subst $[subst $this](min_1)] + [subst $[subst $this](max_1)]) / 2.0]
    set [subst $this](dop_2) -1.0

    return $this
}

proc UpdateSLIDECameraObjectDrag { name coord minmax value } {
    set this $name

    global $this

    if { $coord < 2 } {
	if { [subst $${this}(balance_$coord)] } {
	    if { $minmax == 0 } {
		set curr  min
		set other max

		if { -[subst $${this}(${curr}_$coord)] < [subst $${this}(${curr}_${coord}_resolution)] } {
		    set ${this}(${curr}_$coord) [expr -[subst $${this}(${curr}_${coord}_resolution)]]
		}
	    } else {
		set curr  max
		set other min

		if { [subst $${this}(${curr}_$coord)] < [subst $${this}(${curr}_${coord}_resolution)] } {
		    set ${this}(${curr}_$coord) [expr [subst $${this}(${curr}_${coord}_resolution)]]
		}
	    }

	    set ${this}(${other}_$coord) [expr -[subst $${this}(${curr}_$coord)]]
	}
    }

    UpdateSLIDECameraObject $name
}

proc UpdateSLIDECameraObjectBalance { name coord } {
    set this $name

    global $this

    if { [subst $${this}(balance_$coord)] } {
	set min [subst $${this}(min_$coord)]
	set max [subst $${this}(max_$coord)]

	if { abs($min) < abs($max) } {
	    set ${this}(min_$coord) [expr -[subst $${this}(max_$coord)]]
	} else {
	    set ${this}(max_$coord) [expr -[subst $${this}(min_$coord)]]
	}
    }

    UpdateSLIDECameraObject $name
}

#######################CreateSLIDE###############################################
# CreateSLIDECameraUI
#   Parameters:
#     parent   - parent widget name
#     name     - name of the camera
#
#   Actions:
#     Creates a camera UI contained in a frame widget
#
#   Returns:
#     The name of the unpacked frame widget
#
proc CreateSLIDECameraUI { parent name } {
    set root $parent.f$name
    set this $name

    global $this

    frame $root

    ##################################################
    # Enums Frame
    frame $root.fEnums
    
    ##############################
    # projection
    set widget [CreateSLIDEProjectionEnumFrame $root.fEnums projection \
	    [subst $this](projection) "Projection"]
    pack $widget -side left -fill y

    ##############################
    # Check Buttons
    frame $root.fEnums.fCheckButtons -relief raised -bd 2

    label $root.fEnums.fCheckButtons.lTitle -text "Oblique"
    pack $root.fEnums.fCheckButtons.lTitle -side top -fill x

    checkbutton $root.fEnums.fCheckButtons.cb0 -text "Balance X" -variable ${this}(balance_0) -anchor w -command [list UpdateSLIDECameraObjectBalance $this 0]
    pack $root.fEnums.fCheckButtons.cb0 -side top -fill x
    
    checkbutton $root.fEnums.fCheckButtons.cb1 -text "Balance Y" -variable ${this}(balance_1) -anchor w -command [list UpdateSLIDECameraObjectBalance $this 1]
    pack $root.fEnums.fCheckButtons.cb1 -side top -fill x
    
    pack $root.fEnums.fCheckButtons -side left -fill y

    pack $root.fEnums -side top -fill x

    ##################################################
    # Scales Frame
    frame $root.fScales
    
    ##############################
    # X
    set vars [list ${this}(min_0) ${this}(max_0)]
    set commands [list \
	    [list UpdateSLIDECameraObjectDrag $this 0] \
	    [list UpdateSLIDECameraObjectDrag $this 0] \
	    ]
#    set widget [CreateMultiScale $root.fScales scaleFrustumX \
#	    $vars "Frustum X" \
#	    [subst $[subst $this](min_0_min)] \
#	    [subst $[subst $this](min_0_max)] \
#	    [subst $[subst $this](min_0_resolution)] \
#	    [subst $[subst $this](min_0_resolution)] \
#	    $commands \
#	    1 horizontal 150]
#    pack $widget -side top -fill x

    ##############################
    # Y
    set vars [list ${this}(min_1) ${this}(max_1)]
    set commands [list \
	    [list UpdateSLIDECameraObjectDrag $this 1] \
	    [list UpdateSLIDECameraObjectDrag $this 1] \
	    ]
#    set widget [CreateMultiScale $root.fScales scaleFrustumY \
#	    $vars "Frustum Y" \
#	    [subst $[subst $this](min_1_min)] \
#	    [subst $[subst $this](min_1_max)] \
#	    [subst $[subst $this](min_1_resolution)] \
#	    [subst $[subst $this](min_1_resolution)] \
#	    $commands \
#	    1 horizontal 150]
#    pack $widget -side top -fill x

    ##############################
    # Z
    set vars [list ${this}(min_2) ${this}(max_2)]
    set commands [list \
	    [list UpdateSLIDECameraObjectDrag $this 2] \
	    [list UpdateSLIDECameraObjectDrag $this 2] \
	    ]
#    set widget [CreateMultiScale $root.fScales scaleFrustumZ \
#	    $vars "Frustum Z" \
#	    [subst $[subst $this](min_2_min)] \
#	    [subst $[subst $this](min_2_max)] \
#	    [subst $[subst $this](min_2_resolution)] \
#	    [subst $[subst $this](min_2_resolution)] \
#	    $commands \
#	    1 horizontal 300]
#    pack $widget -side top -fill x

#     ##############################
#     # max_2
#     set widget [CreateScale $root.fScales scaleMax_2 \
# 	    [subst $this](max_2) "Max Z" \
# 	    [subst $[subst $this](max_2_min)] \
# 	    [subst $[subst $this](max_2_max)] \
# 	    [subst $[subst $this](max_2_resolution)]]
#     pack $widget -side top -fill x

#     ##############################
#     # min_2
#     set widget [CreateScale $root.fScales scaleMin_2 \
# 	    [subst $this](min_2) "Min Z" \
# 	    [subst $[subst $this](min_2_min)] \
# 	    [subst $[subst $this](min_2_max)] \
# 	    [subst $[subst $this](min_2_resolution)]]
#     pack $widget -side top -fill x

    pack $root.fScales -side top -fill x

    return $root
}

proc FrustumCorner { this minmax coord z } {
  global $this

  return [expr [subst $[subst $this]([subst $minmax]_$coord)] - ([subst $[subst $this](dop_$coord)] * ([subst $[subst $this](dop_2)] - [subst $[subst $this]($z)]) / [subst $[subst $this](dop_2)])]
}
