# Must start with a vector of the correct size
# must have spaces between all of the arguments
proc vexpr { a args } {
    set size [llength $a]
    set r {}

    set args "[list $a] $args"

    for {set i 0} {$i < $size} {incr i} {
	set sargs {}
	foreach arg $args {
	    set len [expr [llength $arg] - 1]
	    if { $len > $i } {
		set len $i
	    }

	    lappend sargs [lindex $arg $len]
	}
#puts "sargs = $sargs"
	lappend r [expr $sargs]
    }

    return $r
}

proc MM { m n } {
    set result ""

    for { set i 0 } { $i < 4 } { incr i } {
	for { set j 0 } { $j < 4 } { incr j } {
	    set value 0
	    for { set k 0 } { $k < 4 } { incr k } {
		set value [expr $value + [lindex $m [expr $j + 4*$k]] * [lindex $n [expr $k + 4*$i]]]
	    }
	    set result "$result $value"
	}
    }

    return $result
}

proc MV { m v } {
    set result ""

    for { set i 0 } { $i < 4 } { incr i } {
	set value 0
	for { set j 0 } { $j < 4 } { incr j } {
	    set value [expr $value + [lindex $m [expr $i + 4*$j]] * [lindex $v [expr $j]]]
	}
	set result "$result $value"
    }

    return $result
}

proc Vmagnitude { v } {
    return [expr sqrt([VdotV $v $v])]
}

proc Vnormalize { v } {
    set fMag [Vmagnitude $v]

    if { $v == 0.0 } {
	puts "ERROR: Normalizing Null Vector"
	return $v
    }

    return [vexpr $v / $fMag]
}

proc VdotV { v w } {
    set value 0

    for { set i 0 } { $i < 4 } { incr i } {
	set value [expr $value + [lindex $v $i] * [lindex $w $i]]
    }

    return $value
}

proc VcrossV { v w } {
    return [MV [Mcross $v] $w]
}

proc Mindex { m iRow iCol } {
    return [lindex $m [expr 4*$iCol + $iRow]]
}

proc Midentity { } {
    return [list 1 0 0 0 0 1 0 0 0 0 1 0 0 0 0 1]
}

proc Mtranspose { m } {
    set result ""
    for { set i 0 } { $i < 4 } { incr i } {
	for { set j 0 } { $j < 4 } { incr j } {
	    set result "$result [Mindex $m $i $j]"
	}
    }

    return $result
}

proc Minverse { m } {
    set mTrans [Mtranslate [vexpr [lrange $m 12 15] * -1]]
    set mRot [concat [lrange $m 0 11] 0 0 0 1]

    return [MM [Mtranspose $mRot] $mTrans]
}

proc Mtranslate { v } {
    return [concat 1 0 0 0 0 1 0 0 0 0 1 0 [lrange $v 0 2] 1]
}

proc Mrotate { v angle } {
    set m [Mcross $v]

    return [vexpr [Midentity] + ( $m * sin($angle) ) + ( [MM $m $m] * (1.0-cos($angle)) )]
}

proc Mcross { v } {
    return [concat 0 [lindex $v 2] -[lindex $v 1] 0 -[lindex $v 2] 0 [lindex $v 0] 0 [lindex $v 1] -[lindex $v 0] 0 0 0 0 0 0]
}

proc Mtwist { tw angle } {
    set v [lindex $tw 0]
    set w [lindex $tw 1]

    if { [VdotV $w $w] == 0.0 } {
	set mRot [Midentity]
	set mTrans [Mtranslate [vexpr $v * $angle]]
    } else {
	set mRot [Mrotate $w $angle]
	set v2 [vexpr [MV [vexpr [Midentity] - $mRot] [VcrossV $w $v]] + $w * [VdotV $w $v] * $angle]
	set mTrans [Mtranslate $v2]
    }

    return [MM $mTrans $mRot]
}

#proc Mtwist { tw angle } {
#    set v [concat [lrange $tw 0 2] 0]
#    set w [concat [lrange $tw 3 5] 0]
#
#    if { [VdotV $w $w] == 0.0 } {
#	set mRot [Midentity]
#    } else {
#	set mRot [Mrotate $w $angle]
#    }
#
#    set mTrans [Mtranslate [vexpr $v * $angle]]
#
#    return [MM mTrans mRot]
#}

proc Mscale { v } {
    if { [llength $v] == 1 } {
	return [concat $v 0 0 0 0 $v 0 0 0 0 $v 0 0 0 0 1]
    } else {
	return [concat [lindex $v 0] 0 0 0 0 [lindex $v 1] 0 0 0 0 [lindex $v 2] 0 0 0 0 1]
    }
}

proc Mdisplay { m } {
    for { set i 0 } { $i < 4 } { incr i } {
	puts "[Mindex $m $i 0] [Mindex $m $i 1] [Mindex $m $i 2] [Mindex $m $i 3]"
    }
}

#set a {1 2 3 1 1 1 1 1}
#set b {4 5 6 2 2 2 2 2}

#vexpr $a * 2 + $b
